@@ -1,6 +1,6 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
-// Copyright Bob Friesenhahn, 1999, 2000, 2001
+// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002
 //
 // Geometry Definition
 //
@@ -16,19 +16,18 @@
 namespace Magick
 {
 
-  class Geometry;
+  class MagickDLLDecl Geometry;
 
   // Compare two Geometry objects regardless of LHS/RHS
-  int operator == ( const Magick::Geometry& left_, const Magick::Geometry& right_ );
-  int operator != ( const Magick::Geometry& left_, const Magick::Geometry& right_ );
-  int operator >  ( const Magick::Geometry& left_, const Magick::Geometry& right_ );
-  int operator <  ( const Magick::Geometry& left_, const Magick::Geometry& right_ );
-  int operator >= ( const Magick::Geometry& left_, const Magick::Geometry& right_ );
-  int operator <= ( const Magick::Geometry& left_, const Magick::Geometry& right_ );
+  int MagickDLLDecl operator == ( const Magick::Geometry& left_, const Magick::Geometry& right_ );
+  int MagickDLLDecl operator != ( const Magick::Geometry& left_, const Magick::Geometry& right_ );
+  int MagickDLLDecl operator >  ( const Magick::Geometry& left_, const Magick::Geometry& right_ );
+  int MagickDLLDecl operator <  ( const Magick::Geometry& left_, const Magick::Geometry& right_ );
+  int MagickDLLDecl operator >= ( const Magick::Geometry& left_, const Magick::Geometry& right_ );
+  int MagickDLLDecl operator <= ( const Magick::Geometry& left_, const Magick::Geometry& right_ );
 
-  class Geometry
+  class MagickDLLDecl Geometry
   {
-    friend class Image;
   public:
     
     Geometry ( unsigned int width_,
@@ -37,9 +36,9 @@ namespace Magick
 	       unsigned int yOff_ = 0,
 	       bool xNegative_ = false,
 	       bool yNegative_ = false );
-    Geometry ( const std::string geometry_ );
+    Geometry ( const std::string &geometry_ );
     Geometry ( const char * geometry_ );
-    Geometry ( const Geometry & geometry_ );
+    Geometry ( const Geometry &geometry_ );
     Geometry ( );
     ~Geometry ( void );
     
@@ -53,11 +52,11 @@ namespace Magick
     
     // X offset from origin
     void          xOff ( unsigned int xOff_ );
-    int           xOff ( void ) const;
+    unsigned int  xOff ( void ) const;
     
     // Y offset from origin
     void          yOff ( unsigned int yOff_ );
-    int           yOff ( void ) const;
+    unsigned int  yOff ( void ) const;
     
     // Sign of X offset negative? (X origin at right)
     void          xNegative ( bool xNegative_ );
@@ -92,23 +91,20 @@ namespace Magick
     const Geometry& operator = ( const char * geometry_ );
 
     // Assignment operator
-    Geometry operator= ( const Geometry& Geometry_ );
+    Geometry& operator= ( const Geometry& Geometry_ );
     
     // Return geometry string
     operator std::string() const;
     
-    // print object to a stream
-    friend std::ostream& operator<<(std::ostream& stream_, const Geometry& geometry_);
-
     //
     // Public methods below this point are for Magick++ use only.
     //
 
     // Construct from RectangleInfo
-    Geometry ( const MagickLib::RectangleInfo &rectangle_ );
+    Geometry ( const MagickCore::RectangleInfo &rectangle_ );
 
     // Return an ImageMagick RectangleInfo struct
-    operator MagickLib::RectangleInfo() const;
+    operator MagickCore::RectangleInfo() const;
     
   private:
     unsigned int  _width;
@@ -166,7 +162,7 @@ inline void Magick::Geometry::xOff ( unsigned int xOff_ )
 {
   _xOff = xOff_;
 }
-inline int Magick::Geometry::xOff ( void ) const
+inline unsigned int Magick::Geometry::xOff ( void ) const
 {
   return _xOff;
 }
@@ -176,7 +172,7 @@ inline void Magick::Geometry::yOff ( unsigned int yOff_ )
 {
   _yOff = yOff_;
 }
-inline int Magick::Geometry::yOff ( void ) const
+inline unsigned int Magick::Geometry::yOff ( void ) const
 {
   return _yOff;
 }
