@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -117,21 +117,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
   QuantumType
     quantum_type;
 
-  register const IndexPacket
-    *canvas_indexes;
-
-  register const PixelPacket
-    *p;
-
-  register IndexPacket
-    *indexes;
-
   register long
-    i,
-    x;
-
-  register PixelPacket
-    *q;
+    i;
 
   ssize_t
     count;
@@ -175,11 +162,12 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
   /*
     Create virtual canvas to support cropping (i.e. image.cmyk[100x100+10+20]).
   */
-  canvas_image=CloneImage(image,image->extract_info.width,1,MagickTrue,
+  canvas_image=CloneImage(image,image->extract_info.width,1,MagickFalse,
     exception);
-  canvas_image->colorspace=CMYKColorspace;
   (void) SetImageVirtualPixelMethod(canvas_image,BlackVirtualPixelMethod);
   quantum_info=AcquireQuantumInfo(image_info,canvas_image);
+  if (quantum_info == (QuantumInfo *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   pixels=GetQuantumPixels(quantum_info);
   quantum_type=CMYKQuantum;
   if (LocaleCompare(image_info->magick,"CMYKA") == 0)
@@ -214,9 +202,6 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
     image->colorspace=CMYKColorspace;
-    if ((SetImageExtent(image,0,0) == MagickFalse) ||
-        (SetImageExtent(canvas_image,0,0) == MagickFalse))
-      break;
     switch (image_info->interlace)
     {
       case NoInterlace:
@@ -234,11 +219,26 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           }
         for (y=0; y < (long) image->extract_info.height; y++)
         {
+          register const IndexPacket
+            *__restrict canvas_indexes;
+
+          register const PixelPacket
+            *__restrict p;
+
+          register IndexPacket
+            *__restrict indexes;
+
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+          length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
             quantum_info,quantum_type,pixels,exception);
           if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
             break;
@@ -260,6 +260,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
                 q->green=p->green;
                 q->blue=p->blue;
                 indexes[x]=canvas_indexes[image->extract_info.x+x];
+                q->opacity=OpaqueOpacity;
                 if (image->matte != MagickFalse)
                   q->opacity=p->opacity;
                 p++;
@@ -304,6 +305,21 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           }
         for (y=0; y < (long) image->extract_info.height; y++)
         {
+          register const IndexPacket
+            *__restrict canvas_indexes;
+
+          register const PixelPacket
+            *__restrict p;
+
+          register IndexPacket
+            *__restrict indexes;
+
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           for (i=0; i < (image->matte != MagickFalse ? 5 : 4); i++)
           {
             quantum_type=quantum_types[i];
@@ -311,7 +327,7 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
               exception);
             if (q == (PixelPacket *) NULL)
               break;
-            length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+            length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
               quantum_info,quantum_type,pixels,exception);
             if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
               break;
@@ -320,8 +336,8 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
               {
                 p=GetVirtualPixels(canvas_image,canvas_image->extract_info.x,
                   0,canvas_image->columns,1,exception);
-                q=GetAuthenticPixels(image,0,y-image->extract_info.y,image->columns,
-                  1,exception);
+                q=GetAuthenticPixels(image,0,y-image->extract_info.y,
+                  image->columns,1,exception);
                 if ((p == (const PixelPacket *) NULL) ||
                     (q == (PixelPacket *) NULL))
                   break;
@@ -372,11 +388,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           }
         for (y=0; y < (long) image->extract_info.height; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+          length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
             quantum_info,CyanQuantum,pixels,exception);
           if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
             break;
@@ -411,11 +436,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           }
         for (y=0; y < (long) image->extract_info.height; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+          length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
             quantum_info,MagentaQuantum,pixels,exception);
           if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
             break;
@@ -450,11 +484,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           }
         for (y=0; y < (long) image->extract_info.height; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+          length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
             quantum_info,YellowQuantum,pixels,exception);
           if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
             break;
@@ -489,11 +532,26 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           }
         for (y=0; y < (long) image->extract_info.height; y++)
         {
+          register const IndexPacket
+            *__restrict canvas_indexes;
+
+          register const PixelPacket
+            *__restrict p;
+
+          register IndexPacket
+            *__restrict indexes;
+
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+          length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
             quantum_info,BlackQuantum,pixels,exception);
           if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
             break;
@@ -532,11 +590,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           {
             for (y=0; y < (long) image->extract_info.height; y++)
             {
+              register const PixelPacket
+                *__restrict p;
+
+              register long
+                x;
+
+              register PixelPacket
+                *__restrict q;
+
               q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
                 exception);
               if (q == (PixelPacket *) NULL)
                 break;
-              length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+              length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
                 quantum_info,AlphaQuantum,pixels,exception);
               if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
                 break;
@@ -613,11 +680,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           break;
         for (y=0; y < (long) image->extract_info.height; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+          length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
             quantum_info,CyanQuantum,pixels,exception);
           if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
             break;
@@ -673,11 +749,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           break;
         for (y=0; y < (long) image->extract_info.height; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+          length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
             quantum_info,MagentaQuantum,pixels,exception);
           if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
             break;
@@ -733,11 +818,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           break;
         for (y=0; y < (long) image->extract_info.height; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+          length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
             quantum_info,YellowQuantum,pixels,exception);
           if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
             break;
@@ -793,11 +887,26 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
           break;
         for (y=0; y < (long) image->extract_info.height; y++)
         {
+          register const IndexPacket
+            *__restrict canvas_indexes;
+
+          register const PixelPacket
+            *__restrict p;
+
+          register IndexPacket
+            *__restrict indexes;
+
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
             exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+          length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
             quantum_info,BlackQuantum,pixels,exception);
           if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
             break;
@@ -857,11 +966,20 @@ static Image *ReadCMYKImage(const ImageInfo *image_info,
               break;
             for (y=0; y < (long) image->extract_info.height; y++)
             {
+              register const PixelPacket
+                *__restrict p;
+
+              register long
+                x;
+
+              register PixelPacket
+                *__restrict q;
+
               q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,
                 exception);
               if (q == (PixelPacket *) NULL)
                 break;
-              length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,
+              length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,
                 quantum_info,YellowQuantum,pixels,exception);
               if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
                 break;
@@ -1057,9 +1175,6 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
   QuantumType
     quantum_type;
 
-  register const PixelPacket
-    *p;
-
   ssize_t
     count;
 
@@ -1100,11 +1215,13 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
       Convert MIFF to CMYK raster pixels.
     */
     if (image->colorspace != CMYKColorspace)
-      (void) SetImageColorspace(image,CMYKColorspace);
+      (void) TransformImageColorspace(image,CMYKColorspace);
     if ((LocaleCompare(image_info->magick,"CMYKA") == 0) &&
         (image->matte == MagickFalse))
       (void) SetImageAlphaChannel(image,ResetAlphaChannel);
     quantum_info=AcquireQuantumInfo(image_info,image);
+    if (quantum_info == (QuantumInfo *) NULL)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     pixels=GetQuantumPixels(quantum_info);
     switch (image_info->interlace)
     {
@@ -1116,10 +1233,13 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         */
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             quantum_type,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
@@ -1140,32 +1260,35 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         */
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             CyanQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             MagentaQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             YellowQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             BlackQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
             break;
           if (quantum_type == CMYKAQuantum)
             {
-              length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+              length=ExportQuantumPixels(image,(const CacheView *) NULL,
                 quantum_info,AlphaQuantum,pixels,&image->exception);
               count=WriteBlob(image,length,pixels);
               if (count != (ssize_t) length)
@@ -1187,10 +1310,13 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
         */
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             CyanQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
@@ -1204,10 +1330,13 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           }
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             MagentaQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
@@ -1221,10 +1350,13 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           }
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             YellowQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
@@ -1238,10 +1370,13 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           }
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             BlackQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
@@ -1257,11 +1392,14 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           {
             for (y=0; y < (long) image->rows; y++)
             {
+              register const PixelPacket
+                *__restrict p;
+
               p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+              length=ExportQuantumPixels(image,(const CacheView *) NULL,
                 quantum_info,AlphaQuantum,pixels,&image->exception);
               count=WriteBlob(image,length,pixels);
               if (count != (ssize_t) length)
@@ -1297,10 +1435,13 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           return(status);
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             CyanQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
@@ -1320,10 +1461,13 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           return(status);
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             MagentaQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
@@ -1343,10 +1487,13 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           return(status);
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             YellowQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
@@ -1366,10 +1513,13 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
           return(status);
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             BlackQuantum,pixels,&image->exception);
           count=WriteBlob(image,length,pixels);
           if (count != (ssize_t) length)
@@ -1391,11 +1541,14 @@ static MagickBooleanType WriteCMYKImage(const ImageInfo *image_info,
               return(status);
             for (y=0; y < (long) image->rows; y++)
             {
+              register const PixelPacket
+                *__restrict p;
+
               p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+              length=ExportQuantumPixels(image,(const CacheView *) NULL,
                 quantum_info,AlphaQuantum,pixels,&image->exception);
               count=WriteBlob(image,length,pixels);
               if (count != (ssize_t) length)
