@@ -327,8 +327,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o extent: Specifies the extent of the magick string.
 %
@@ -441,14 +440,15 @@ static inline MagickBooleanType IsFloatDefined(const float value)
 {
   union
   {
-    unsigned long
+    unsigned int
       unsigned_value;
 
-    double
+    float
       float_value;
   } quantum;
 
-  quantum.float_value=value;
+  quantum.unsigned_value=0U;
+  quantum.float_value=(double) value;
   if (quantum.unsigned_value == 0U)
     return(MagickFalse);
   return(MagickTrue);
@@ -527,7 +527,7 @@ static void TimeCodeToString(const unsigned long timestamp,char *code)
   shift=4*TimeFields;
   for (i=0; i <= TimeFields; i++)
   {
-    (void) FormatMagickString(code,MaxTextExtent-strlen(code),"%01u",
+    (void) FormatMagickString(code,MaxTextExtent-strlen(code),"%x",
       (unsigned int) ((timestamp >> shift) & 0x0fU));
     code++;
     if (((i % 2) != 0) && (i < TimeFields))
@@ -753,12 +753,12 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           dpx.orientation.y_offset);
       dpx.orientation.x_center=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.orientation.x_center)!= MagickFalse)
+      if (IsFloatDefined(dpx.orientation.x_center) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:orientation.x_center","%g",
           dpx.orientation.x_center);
       dpx.orientation.y_center=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.orientation.y_center)!= MagickFalse)
+      if (IsFloatDefined(dpx.orientation.y_center) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:orientation.y_center","%g",
           dpx.orientation.y_center);
       dpx.orientation.x_size=ReadBlobLong(image);
@@ -862,12 +862,12 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           dpx.film.held_count);
       dpx.film.frame_rate=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.film.frame_rate)!= MagickFalse)
+      if (IsFloatDefined(dpx.film.frame_rate) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:film.frame_rate","%g",
           dpx.film.frame_rate);
       dpx.film.shutter_angle=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.film.shutter_angle)!= MagickFalse)
+      if (IsFloatDefined(dpx.film.shutter_angle) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:film.shutter_angle","%g",
           dpx.film.shutter_angle);
       offset+=ReadBlob(image,sizeof(dpx.film.frame_id),(unsigned char *)
@@ -888,11 +888,11 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Read DPX television header.
       */
-      dpx.television.time_code=ReadBlobLong(image);
+      dpx.television.time_code=(unsigned int) ReadBlobLong(image);
       offset+=4;
       TimeCodeToString(dpx.television.time_code,value);
       (void) SetImageProperty(image,"dpx:television.time.code",value);
-      dpx.television.user_bits=(unsigned char) ReadBlobLong(image);
+      dpx.television.user_bits=(unsigned int) ReadBlobLong(image);
       offset+=4;
       TimeCodeToString(dpx.television.user_bits,value);
       (void) SetImageProperty(image,"dpx:television.user.bits",value);
@@ -918,53 +918,53 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           dpx.television.padding);
       dpx.television.horizontal_sample_rate=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.television.horizontal_sample_rate)!= MagickFalse)
+      if (IsFloatDefined(dpx.television.horizontal_sample_rate) != MagickFalse)
         (void) FormatImageProperty(image,
           "dpx:television.horizontal_sample_rate","%g",
           dpx.television.horizontal_sample_rate);
       dpx.television.vertical_sample_rate=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.television.vertical_sample_rate)!= MagickFalse)
+      if (IsFloatDefined(dpx.television.vertical_sample_rate) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:television.vertical_sample_rate",
           "%g",dpx.television.vertical_sample_rate);
       dpx.television.frame_rate=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.television.frame_rate)!= MagickFalse)
+      if (IsFloatDefined(dpx.television.frame_rate) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:television.frame_rate","%g",
           dpx.television.frame_rate);
       dpx.television.time_offset=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.television.time_offset)!= MagickFalse)
+      if (IsFloatDefined(dpx.television.time_offset) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:television.time_offset","%g",
           dpx.television.time_offset);
       dpx.television.gamma=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.television.gamma)!= MagickFalse)
+      if (IsFloatDefined(dpx.television.gamma) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:television.gamma","%g",
           dpx.television.gamma);
       dpx.television.black_level=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.television.black_level)!= MagickFalse)
+      if (IsFloatDefined(dpx.television.black_level) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:television.black_level","%g",
           dpx.television.black_level);
       dpx.television.black_gain=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.television.black_gain)!= MagickFalse)
+      if (IsFloatDefined(dpx.television.black_gain) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:television.black_gain","%g",
           dpx.television.black_gain);
       dpx.television.break_point=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.television.break_point)!= MagickFalse)
+      if (IsFloatDefined(dpx.television.break_point) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:television.break_point","%g",
           dpx.television.break_point);
       dpx.television.white_level=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.television.white_level)!= MagickFalse)
+      if (IsFloatDefined(dpx.television.white_level) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:television.white_level","%g",
           dpx.television.white_level);
       dpx.television.integration_times=ReadBlobFloat(image);
       offset+=4;
-      if (IsFloatDefined(dpx.television.integration_times)!= MagickFalse)
+      if (IsFloatDefined(dpx.television.integration_times) != MagickFalse)
         (void) FormatImageProperty(image,"dpx:television.integration_times",
           "%g",dpx.television.integration_times);
       offset+=ReadBlob(image,sizeof(dpx.television.reserve),(unsigned char *)
@@ -1016,6 +1016,11 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
     case CbYACrYA4224ComponentType:
     case CbYCr444ComponentType:
+    {
+      samples_per_pixel=3;
+      quantum_type=CbYCrQuantum;
+      break;
+    }
     case RGBComponentType:
     {
       samples_per_pixel=3;
@@ -1064,7 +1069,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (image->endian == MSBEndian))
     {
       QuantumAny
-        scale;
+        range;
 
       QuantumInfo
         *quantum_info;
@@ -1087,7 +1092,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       pixels=GetQuantumPixels(quantum_info);
       pixel=0U;
       i=0;
-      scale=GetQuantumScale(image->depth);
+      range=GetQuantumRange(image->depth);
       for (y=0; y < (long) image->rows; y++)
       {
         q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
@@ -1100,19 +1105,17 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
             case 0:
             {
               pixel=ReadBlobMSBLong(image);
-              q->red=ScaleAnyToQuantum((pixel >> 0) & 0x3ff,image->depth,scale);
+              q->red=ScaleAnyToQuantum((pixel >> 0) & 0x3ff,range);
               break;
             }
             case 1:
             {
-              q->red=ScaleAnyToQuantum((pixel >> 10) & 0x3ff,image->depth,
-                scale);
+              q->red=ScaleAnyToQuantum((pixel >> 10) & 0x3ff,range);
               break;
             }
             case 2:
             {
-              q->red=ScaleAnyToQuantum((pixel >> 20) & 0x3ff,image->depth,
-                scale);
+              q->red=ScaleAnyToQuantum((pixel >> 20) & 0x3ff,range);
               break;
             }
           }
@@ -1136,7 +1139,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       QuantumInfo
         *quantum_info;
 
-      ViewInfo
+      CacheView
         *image_view;
 
       /*
@@ -1178,7 +1181,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
           continue;
         pixels=GetQuantumPixels(quantum_info);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical
+  #pragma omp critical (MagickCore_ReadDPXImage)
 #endif
         {
           count=ReadBlob(image,extent,pixels);
@@ -1196,8 +1199,8 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
         if (count != (ssize_t) extent)
           status=MagickFalse;
-        q=QueueCacheViewAuthenticPixels(image_view,0,offset,image->columns,
-          1,exception);
+        q=QueueCacheViewAuthenticPixels(image_view,0,offset,image->columns,1,
+          exception);
         if (q == (PixelPacket *) NULL)
           {
             status=MagickFalse;
@@ -1261,6 +1264,7 @@ ModuleExport unsigned long RegisterDPXImage(void)
   entry->decoder=(DecodeImageHandler *) ReadDPXImage;
   entry->encoder=(EncodeImageHandler *) WriteDPXImage;
   entry->magick=(IsImageFormatHandler *) IsDPX;
+  entry->adjoin=MagickFalse;
   entry->description=ConstantString("SMPTE 268M-2003 (DPX 2.0)");
   entry->note=ConstantString(DPXNote);
   entry->module=ConstantString("DPX");
@@ -1329,7 +1333,7 @@ static inline const char *GetDPXProperty(const ImageInfo *image_info,
   return(GetImageProperty(image,property));
 }
 
-static unsigned int DecodeSMPTEBits(const char *key)
+static unsigned int StringToTimeCode(const char *key)
 {
   char
     buffer[2];
@@ -1346,13 +1350,13 @@ static unsigned int DecodeSMPTEBits(const char *key)
   buffer[1]='\0';
   for (i=0; (*key != 0) && (i < 11); i++)
   {
-    if (isdigit((int) ((unsigned char) *key)) == 0)
+    if (isxdigit((int) ((unsigned char) *key)) == 0)
       {
         key++;
         continue;
       }
     buffer[0]=(*key++);
-    value|=(unsigned int) ((strtol(buffer,(char **) NULL,10)) << shift);
+    value|=(unsigned int) ((strtol(buffer,(char **) NULL,16)) << shift);
     shift-=4;
   }
   return(value);
@@ -1370,6 +1374,8 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     dpx;
 
   long
+    horizontal_factor,
+    vertical_factor,
     y;
 
   MagickBooleanType
@@ -1417,6 +1423,30 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  horizontal_factor=4;
+  vertical_factor=4;
+  if (image_info->sampling_factor != (char *) NULL)
+    {
+      GeometryInfo
+        geometry_info;
+
+      MagickStatusType
+        flags;
+
+      flags=ParseGeometry(image_info->sampling_factor,&geometry_info);
+      horizontal_factor=(long) geometry_info.rho;
+      vertical_factor=(long) geometry_info.sigma;
+      if ((flags & SigmaValue) == 0)
+        vertical_factor=horizontal_factor;
+      if ((horizontal_factor != 1) && (horizontal_factor != 2) &&
+          (horizontal_factor != 4) && (vertical_factor != 1) &&
+          (vertical_factor != 2) && (vertical_factor != 4))
+        ThrowWriterException(CorruptImageError,"UnexpectedSamplingFactor");
+    }
+  if ((image->colorspace == YCbCrColorspace) &&
+      ((horizontal_factor == 2) || (vertical_factor == 2)))
+    if ((image->columns % 2) != 0)
+      image->columns++;
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
@@ -1492,6 +1522,9 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   offset+=WriteBlobShort(image,dpx.image.orientation);
   dpx.image.number_elements=1;
   offset+=WriteBlobShort(image,dpx.image.number_elements);
+  if ((image->columns != (unsigned int) image->columns) ||
+      (image->rows != (unsigned int) image->rows))
+    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   offset+=WriteBlobLong(image,(unsigned int) image->columns);
   offset+=WriteBlobLong(image,(unsigned int) image->rows);
   for (i=0; i < 8; i++)
@@ -1508,7 +1541,25 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     offset+=WriteBlobFloat(image,dpx.image.image_element[i].high_quantity);
     dpx.image.image_element[i].descriptor=0;
     if (i == 0)
-      dpx.image.image_element[i].descriptor=RGBComponentType;
+      switch (image->colorspace)
+      {
+        case Rec601YCbCrColorspace:
+        case Rec709YCbCrColorspace:
+        case YCbCrColorspace:
+        {
+          dpx.image.image_element[i].descriptor=CbYCr444ComponentType;
+          if (image->matte != MagickFalse)
+            dpx.image.image_element[i].descriptor=CbYCrA4444ComponentType;
+          break;
+        }
+        default:
+        {
+          dpx.image.image_element[i].descriptor=RGBComponentType;
+          if (image->matte != MagickFalse)
+            dpx.image.image_element[i].descriptor=RGBAComponentType;
+          break;
+        }
+      }
     offset+=WriteBlobByte(image,dpx.image.image_element[i].descriptor);
     dpx.image.image_element[i].transfer=0;
     if (image->colorspace == LogColorspace)
@@ -1712,11 +1763,11 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   */
   value=GetDPXProperty(image_info,image,"dpx:television.time.code");
   if (value != (const char *) NULL)
-    dpx.television.time_code=DecodeSMPTEBits(value);
+    dpx.television.time_code=StringToTimeCode(value);
   offset+=WriteBlobLong(image,dpx.television.time_code);
   value=GetDPXProperty(image_info,image,"dpx:television.user.bits");
   if (value != (const char *) NULL)
-    dpx.television.user_bits=DecodeSMPTEBits(value);
+    dpx.television.user_bits=StringToTimeCode(value);
   offset+=WriteBlobLong(image,dpx.television.user_bits);
   value=GetDPXProperty(image_info,image,"dpx:television.interlace");
   if (value != (const char *) NULL)
@@ -1800,7 +1851,16 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     offset+=WriteBlob(image,GetStringInfoLength(profile),
       GetStringInfoDatum(profile));
   while (offset < (MagickOffsetType) dpx.image.image_element[0].data_offset)
-    offset+=WriteBlobByte(image,0x00);
+  {
+    count=WriteBlobByte(image,0x00);
+    if (count != 1)
+      {
+        ThrowFileException(&image->exception,FileOpenError,"UnableToWriteFile",
+          image->filename);
+        break;
+      }
+    offset+=count;
+  }
   /*
     Convert pixel packets to DPX raster image.
   */
@@ -1809,11 +1869,23 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   quantum_info->pack=dpx.image.image_element[0].packing == 0 ? MagickTrue :
     MagickFalse;
   quantum_type=RGBQuantum;
-  extent=GetBytesPerRow(image->columns,3,image->depth,MagickTrue);
+  if (image->matte != MagickFalse)
+    quantum_type=RGBAQuantum;
+  if (image->colorspace == YCbCrColorspace)
+    {
+      quantum_type=CbYCrQuantum;
+      if (image->matte != MagickFalse)
+        quantum_type=CbYCrAQuantum;
+      if ((horizontal_factor == 2) || (vertical_factor == 2))
+        quantum_type=CbYCrYQuantum;
+    }
+  extent=GetBytesPerRow(image->columns,image->matte != MagickFalse ? 4UL : 3UL,
+    image->depth,MagickTrue);
   if (0)
     {
       quantum_type=GrayQuantum;
-      extent=GetBytesPerRow(image->columns,3,image->depth,MagickTrue);
+      extent=GetBytesPerRow(image->columns,image->matte != MagickFalse ?
+        4UL : 3UL,image->depth,MagickTrue);
     }
   pixels=GetQuantumPixels(quantum_info);
   for (y=0; y < (long) image->rows; y++)
@@ -1821,7 +1893,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+    (void) ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
       quantum_type,pixels,&image->exception);
     count=WriteBlob(image,extent,pixels);
     if (count != (ssize_t) extent)
