@@ -17,7 +17,7 @@
 %                                 April 2007                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -87,8 +87,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -209,15 +208,6 @@ static Image *ReadEXRImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      (void) ImfCloseInputFile(file);
-      if (LocaleCompare(image_info->filename,read_info->filename) != 0)
-        (void) RelinquishUniqueFileResource(read_info->filename);
-      read_info=DestroyImageInfo(read_info);
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   scanline=(ImfRgba *) AcquireQuantumMemory(image->columns,sizeof(*scanline));
   if (scanline == (ImfRgba *) NULL)
     {
@@ -291,7 +281,9 @@ ModuleExport unsigned long RegisterEXRImage(void)
   entry->encoder=(EncodeImageHandler *) WriteEXRImage;
 #endif
   entry->magick=(IsImageFormatHandler *) IsEXR;
+  entry->adjoin=MagickFalse;
   entry->description=ConstantString("High Dynamic-range (HDR)");
+  entry->blob_support=MagickFalse;
   entry->module=ConstantString("EXR");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
@@ -334,7 +326,7 @@ ModuleExport void UnregisterEXRImage(void)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  WriteEXRImage() writes an image to a file the in the high dynamic-range
-% (HDR) file format developed by Industrial Light & Magic.
+%  (HDR) file format developed by Industrial Light & Magic.
 %
 %  The format of the WriteEXRImage method is:
 %
@@ -389,8 +381,7 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
   if (status == MagickFalse)
     return(status);
   write_info=CloneImageInfo(image_info);
-  if (IsPathAccessible(write_info->filename) == MagickFalse)
-    (void) AcquireUniqueFilename(write_info->filename);
+  (void) AcquireUniqueFilename(write_info->filename);
   hdr_info=ImfNewHeader();
   ImfHeaderSetDataWindow(hdr_info,0,0,(int) image->columns-1,(int)
     image->rows-1);
@@ -439,11 +430,8 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
   }
   (void) ImfCloseOutputFile(file);
   scanline=(ImfRgba *) RelinquishMagickMemory(scanline);
-  if (LocaleCompare(image_info->filename,write_info->filename) != 0)
-    {
-      (void) FileToImage(image,write_info->filename);
-      (void) RelinquishUniqueFileResource(write_info->filename);
-    }
+  (void) FileToImage(image,write_info->filename);
+  (void) RelinquishUniqueFileResource(write_info->filename);
   write_info=DestroyImageInfo(write_info);
   (void) CloseBlob(image);
   return(MagickTrue);
