@@ -495,8 +495,8 @@ static Image *ReadICONImage(const ImageInfo *image_info,
             q->green=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
             q->red=ScaleCharToQuantum((unsigned char) ReadBlobByte(image));
             if (icon_info.bits_per_pixel == 32)
-              q->opacity=(Quantum) QuantumRange-ScaleCharToQuantum(
-                (unsigned char) ReadBlobByte(image));
+              q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(
+                (unsigned char) ReadBlobByte(image)));
             q++;
           }
           if (icon_info.bits_per_pixel == 24)
@@ -762,6 +762,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
   (void) WriteBlobLSBShort(image,0);
   (void) WriteBlobLSBShort(image,1);
   (void) WriteBlobLSBShort(image,(unsigned char) scene);
+  (void) ResetMagickMemory(&icon_file,0,sizeof(icon_file));
   (void) ResetMagickMemory(&icon_info,0,sizeof(icon_info));
   scene=0;
   next=image;
@@ -786,7 +787,7 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
       Initialize ICON raster file header.
     */
     if (image->colorspace != RGBColorspace)
-      (void) SetImageColorspace(next,RGBColorspace);
+      (void) TransformImageColorspace(next,RGBColorspace);
     icon_info.file_size=14+12+28;
     icon_info.offset_bits=icon_info.file_size;
     icon_info.compression=BI_RGB;
@@ -992,9 +993,9 @@ static MagickBooleanType WriteICONImage(const ImageInfo *image_info,
             *q++=ScaleQuantumToChar(p->green);
             *q++=ScaleQuantumToChar(p->red);
             if (next->matte == MagickFalse)
-              *q++=ScaleQuantumToChar(255);
+              *q++=ScaleQuantumToChar(QuantumRange);
             else
-              *q++=ScaleQuantumToChar(p->opacity);
+              *q++=ScaleQuantumToChar(QuantumRange-p->opacity);
             p++;
           }
           if (icon_info.bits_per_pixel == 24)
