@@ -17,7 +17,7 @@
 %                                 June 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -116,8 +116,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -151,8 +150,7 @@ static MagickBooleanType IsJP2(const unsigned char *magick,const size_t length)
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -342,6 +340,9 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
+  QuantumAny
+    range[4];
+
   register long
     i,
     x;
@@ -352,7 +353,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned long
     maximum_component_depth,
     number_components,
-    scale[4],
+    pixel,
     x_step[4],
     y_step[4];
 
@@ -385,6 +386,8 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
       (void) jas_stream_close(jp2_stream);
       ThrowReaderException(DelegateError,"UnableToDecodeImageFile");
     }
+  image->columns=jas_image_width(jp2_image);
+  image->rows=jas_image_height(jp2_image);
   switch (jas_clrspc_fam(jas_image_clrspc(jp2_image)))
   {
     case JAS_CLRSPC_FAM_RGB:
@@ -447,18 +450,20 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
       ThrowReaderException(CoderError,"ColorspaceModelIsNotSupported");
     }
   }
-  image->columns=jas_image_width(jp2_image);
-  image->rows=jas_image_height(jp2_image);
   image->compression=JPEG2000Compression;
   for (i=0; i < (long) number_components; i++)
   {
-    if ((((long) (jas_image_cmptwidth(jp2_image,components[i])*
-          jas_image_cmpthstep(jp2_image,components[i])) != (long) image->columns)) ||
-        (((long) (jas_image_cmptheight(jp2_image,components[i])*
-          jas_image_cmptvstep(jp2_image,components[i])) != (long) image->rows)) ||
+    unsigned long
+      height,
+      width;
+
+    width=(unsigned long) (jas_image_cmptwidth(jp2_image,components[i])*
+      jas_image_cmpthstep(jp2_image,components[i]));
+    height=(unsigned long) (jas_image_cmptheight(jp2_image,components[i])*
+      jas_image_cmptvstep(jp2_image,components[i]));
+    if ((width != image->columns) || (height != image->rows) ||
         (jas_image_cmpttlx(jp2_image,components[i]) != 0) ||
-        (jas_image_cmpttly(jp2_image,components[i]) != 0) ||
-        (jas_image_cmptsgnd(jp2_image,components[i]) != MagickFalse))
+        (jas_image_cmpttly(jp2_image,components[i]) != 0))
       {
         (void) jas_stream_close(jp2_stream);
         jas_image_destroy(jp2_image);
@@ -477,11 +482,6 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
       jas_image_destroy(jp2_image);
       return(GetFirstImageInList(image));
     }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   maximum_component_depth=0;
   for (i=0; i < (long) number_components; i++)
   {
@@ -500,17 +500,8 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   else
     image->depth=(unsigned long) MagickMin(MAGICKCORE_QUANTUM_DEPTH,16);
   for (i=0; i < (long) number_components; i++)
-  {
-    scale[i]=1;
-    if (jas_image_cmptprec(jp2_image,components[i]) < 16)
-      scale[i]=(1U << (16-jas_image_cmptprec(jp2_image,components[i])))+1;
-  }
-  if (number_components == 1)
-    {
-      image->colors=(image->depth == 8 ? 256UL : MaxColormapSize);
-      if (AcquireImageColormap(image,image->colors) == MagickFalse)
-        ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-    }
+    range[i]=GetQuantumRange((unsigned long) jas_image_cmptprec(jp2_image,
+      components[i]));
   for (y=0; y < (long) image->rows; y++)
   {
     q=GetAuthenticPixels(image,0,y,image->columns,1,exception);
@@ -524,34 +515,17 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       case 1:
       {
-        register IndexPacket
-          *indexes;
-
-        IndexPacket
-          index;
-
         /*
           Grayscale.
         */
-        indexes=GetAuthenticIndexQueue(image);
-        if (image->depth == 8)
-          for (x=0; x < (long) image->columns; x++)
-          {
-            index=ScaleQuantumToChar(ScaleShortToQuantum((unsigned short)
-              (scale[0]*jas_matrix_getv(pixels[0],x))));
-            index=ConstrainColormapIndex(image,index);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
-          }
-        else
-          for (x=0; x < (long) image->columns; x++)
-          {
-            index=(unsigned short) (scale[0]*jas_matrix_getv(pixels[0],x/
-              x_step[0]));
-            index=ConstrainColormapIndex(image,index);
-            indexes[x]=index;
-            *q++=image->colormap[(long) index];
-          }
+        for (x=0; x < (long) image->columns; x++)
+        {
+          pixel=(QuantumAny) jas_matrix_getv(pixels[0],x/x_step[0]);
+          q->red=ScaleAnyToQuantum(pixel,range[0]);
+          q->green=q->red;
+          q->blue=q->red;
+          q++;
+        }
         break;
       }
       case 3:
@@ -561,12 +535,12 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         for (x=0; x < (long) image->columns; x++)
         {
-          q->red=ScaleShortToQuantum((unsigned short) (scale[0]*
-            jas_matrix_getv(pixels[0],x/x_step[0])));
-          q->green=ScaleShortToQuantum((unsigned short) (scale[1]*
-            jas_matrix_getv(pixels[1],x/x_step[1])));
-          q->blue=ScaleShortToQuantum((unsigned short) (scale[2]*
-            jas_matrix_getv(pixels[2],x/x_step[2])));
+          pixel=(QuantumAny) jas_matrix_getv(pixels[0],x/x_step[0]);
+          q->red=ScaleAnyToQuantum(pixel,range[0]);
+          pixel=(QuantumAny) jas_matrix_getv(pixels[1],x/x_step[1]);
+          q->green=ScaleAnyToQuantum(pixel,range[1]);
+          pixel=(QuantumAny) jas_matrix_getv(pixels[2],x/x_step[2]);
+          q->blue=ScaleAnyToQuantum(pixel,range[2]);
           q++;
         }
         break;
@@ -578,14 +552,14 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         for (x=0; x < (long) image->columns; x++)
         {
-          q->red=ScaleShortToQuantum((unsigned short) (scale[0]*
-            jas_matrix_getv(pixels[0],x/x_step[0])));
-          q->green=ScaleShortToQuantum((unsigned short) (scale[1]*
-            jas_matrix_getv(pixels[1],x/x_step[1])));
-          q->blue=ScaleShortToQuantum((unsigned short) (scale[2]*
-            jas_matrix_getv(pixels[2],x/x_step[2])));
-          q->opacity=(Quantum) QuantumRange-ScaleShortToQuantum((unsigned short)
-            (scale[3]*jas_matrix_getv(pixels[3],x/x_step[3])));
+          pixel=(QuantumAny) jas_matrix_getv(pixels[0],x/x_step[0]);
+          q->red=ScaleAnyToQuantum(pixel,range[0]);
+          pixel=(QuantumAny) jas_matrix_getv(pixels[1],x/x_step[1]);
+          q->green=ScaleAnyToQuantum(pixel,range[1]);
+          pixel=(QuantumAny) jas_matrix_getv(pixels[2],x/x_step[2]);
+          q->blue=ScaleAnyToQuantum(pixel,range[2]);
+          pixel=(QuantumAny) jas_matrix_getv(pixels[3],x/x_step[3]);
+          q->opacity=ScaleAnyToQuantum(pixel,range[3]);
           q++;
         }
         break;
@@ -835,7 +809,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
     Intialize JPEG 2000 API.
   */
   if (image->colorspace != RGBColorspace)
-    (void) SetImageColorspace(image,RGBColorspace);
+    (void) TransformImageColorspace(image,RGBColorspace);
   jp2_stream=JP2StreamManager(image);
   if (jp2_stream == (jas_stream_t *) NULL)
     ThrowWriterException(DelegateError,"UnableToManageJP2Stream");
@@ -843,6 +817,9 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   if ((image_info->type != TrueColorType) &&
       IsGrayImage(image,&image->exception))
     number_components=1;
+  if ((image->columns != (unsigned int) image->columns) ||
+      (image->rows != (unsigned int) image->rows))
+    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   for (i=0; i < (long) number_components; i++)
   {
     component_info[i].tlx=0;
