@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -161,8 +161,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -564,21 +563,21 @@ static boolean ReadProfile(j_decompress_ptr jpeg_info)
         (void) CopyMagickString(name,"exif",MaxTextExtent);
       if ((length > 5) && (LocaleNCompare((char *) p,"http:",5) == 0))
         {
-          ssize_t
-            i;
+          long
+            j;
 
           /*
             Extract namespace from XMP profile.
           */
           p=GetStringInfoDatum(profile);
-          for (i=0; i < (long) GetStringInfoLength(profile); i++)
+          for (j=0; j < (long) GetStringInfoLength(profile); j++)
           {
             if (*p == '\0')
               break;
             p++;
           }
-          if (*p == '\0')
-            DestroyStringInfo(SplitStringInfo(profile,i+1));
+          if (j < (long) GetStringInfoLength(profile))
+            (void) DestroyStringInfo(SplitStringInfo(profile,(size_t) (j+1)));
           (void) CopyMagickString(name,"xmp",MaxTextExtent);
         }
     }
@@ -636,24 +635,229 @@ static void JPEGSourceManager(j_decompress_ptr cinfo,Image *image)
   source->image=image;
 }
 
+static void JPEGSetImageQuality(struct jpeg_decompress_struct *jpeg_info,
+  Image *image)
+{
+  image->quality=UndefinedCompressionQuality;
+#if defined(D_PROGRESSIVE_SUPPORTED)
+  if (image->compression == LosslessJPEGCompression)
+    {
+      image->quality=100;
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "Quality: 100 (lossless)");
+    }
+  else
+#endif
+  {
+    long
+      j,
+      qvalue,
+      sum;
+
+    register long
+      i;
+
+    /*
+      Determine the JPEG compression quality from the quantization tables.
+    */
+    sum=0;
+    for (i=0; i < NUM_QUANT_TBLS; i++)
+    {
+      if (jpeg_info->quant_tbl_ptrs[i] != NULL)
+        for (j=0; j < DCTSIZE2; j++)
+          sum+=jpeg_info->quant_tbl_ptrs[i]->quantval[j];
+     }
+     if ((jpeg_info->quant_tbl_ptrs[0] != NULL) &&
+         (jpeg_info->quant_tbl_ptrs[1] != NULL))
+       {
+         long
+           hash[101] =
+           {
+             1020, 1015,  932,  848,  780,  735,  702,  679,  660,  645,
+              632,  623,  613,  607,  600,  594,  589,  585,  581,  571,
+              555,  542,  529,  514,  494,  474,  457,  439,  424,  410,
+              397,  386,  373,  364,  351,  341,  334,  324,  317,  309,
+              299,  294,  287,  279,  274,  267,  262,  257,  251,  247,
+              243,  237,  232,  227,  222,  217,  213,  207,  202,  198,
+              192,  188,  183,  177,  173,  168,  163,  157,  153,  148,
+              143,  139,  132,  128,  125,  119,  115,  108,  104,   99,
+               94,   90,   84,   79,   74,   70,   64,   59,   55,   49,
+               45,   40,   34,   30,   25,   20,   15,   11,    6,    4,
+                0
+           },
+           sums[101] =
+           {
+             32640, 32635, 32266, 31495, 30665, 29804, 29146, 28599, 28104,
+             27670, 27225, 26725, 26210, 25716, 25240, 24789, 24373, 23946,
+             23572, 22846, 21801, 20842, 19949, 19121, 18386, 17651, 16998,
+             16349, 15800, 15247, 14783, 14321, 13859, 13535, 13081, 12702,
+             12423, 12056, 11779, 11513, 11135, 10955, 10676, 10392, 10208,
+              9928,  9747,  9564,  9369,  9193,  9017,  8822,  8639,  8458,
+              8270,  8084,  7896,  7710,  7527,  7347,  7156,  6977,  6788,
+              6607,  6422,  6236,  6054,  5867,  5684,  5495,  5305,  5128,
+              4945,  4751,  4638,  4442,  4248,  4065,  3888,  3698,  3509,
+              3326,  3139,  2957,  2775,  2586,  2405,  2216,  2037,  1846,
+              1666,  1483,  1297,  1109,   927,   735,   554,   375,   201,
+               128,     0
+           };
+
+         qvalue=(long) (jpeg_info->quant_tbl_ptrs[0]->quantval[2]+
+           jpeg_info->quant_tbl_ptrs[0]->quantval[53]+
+           jpeg_info->quant_tbl_ptrs[1]->quantval[0]+
+           jpeg_info->quant_tbl_ptrs[1]->quantval[DCTSIZE2-1]);
+         for (i=0; i < 100; i++)
+         {
+           if ((qvalue < hash[i]) && (sum < sums[i]))
+             continue;
+           if ((qvalue <= hash[i]) && (sum <= sums[i]))
+             image->quality=(unsigned long) i+1;
+           if (image->debug != MagickFalse)
+             {
+               if (image->quality != UndefinedCompressionQuality)
+                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                   "Quality: %ld",image->quality);
+               else
+                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                   "Quality: %ld (approximate)",i+1);
+             }
+           break;
+         }
+       }
+     else
+       if (jpeg_info->quant_tbl_ptrs[0] != NULL)
+         {
+           long
+             hash[101] =
+             {
+               510,  505,  422,  380,  355,  338,  326,  318,  311,  305,
+               300,  297,  293,  291,  288,  286,  284,  283,  281,  280,
+               279,  278,  277,  273,  262,  251,  243,  233,  225,  218,
+               211,  205,  198,  193,  186,  181,  177,  172,  168,  164,
+               158,  156,  152,  148,  145,  142,  139,  136,  133,  131,
+               129,  126,  123,  120,  118,  115,  113,  110,  107,  105,
+               102,  100,   97,   94,   92,   89,   87,   83,   81,   79,
+                76,   74,   70,   68,   66,   63,   61,   57,   55,   52,
+                50,   48,   44,   42,   39,   37,   34,   31,   29,   26,
+                24,   21,   18,   16,   13,   11,    8,    6,    3,    2,
+                 0
+             },
+             sums[101] =
+             {
+               16320, 16315, 15946, 15277, 14655, 14073, 13623, 13230, 12859,
+               12560, 12240, 11861, 11456, 11081, 10714, 10360, 10027,  9679,
+                9368,  9056,  8680,  8331,  7995,  7668,  7376,  7084,  6823,
+                6562,  6345,  6125,  5939,  5756,  5571,  5421,  5240,  5086,
+                4976,  4829,  4719,  4616,  4463,  4393,  4280,  4166,  4092,
+                3980,  3909,  3835,  3755,  3688,  3621,  3541,  3467,  3396,
+                3323,  3247,  3170,  3096,  3021,  2952,  2874,  2804,  2727,
+                2657,  2583,  2509,  2437,  2362,  2290,  2211,  2136,  2068,
+                1996,  1915,  1858,  1773,  1692,  1620,  1552,  1477,  1398,
+                1326,  1251,  1179,  1109,  1031,   961,   884,   814,   736,
+                 667,   592,   518,   441,   369,   292,   221,   151,    86,
+                  64,     0
+             };
+
+           qvalue=(long) (jpeg_info->quant_tbl_ptrs[0]->quantval[2]+
+             jpeg_info->quant_tbl_ptrs[0]->quantval[53]);
+           for (i=0; i < 100; i++)
+           {
+             if ((qvalue < hash[i]) && (sum < sums[i]))
+               continue;
+             image->quality=(unsigned long) i+1;
+             if (image->debug != MagickFalse)
+               {
+                 if ((qvalue > hash[i]) || (sum > sums[i]))
+                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                     "Quality: %ld (approximate)",i+1);
+                 else
+                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                     "Quality: %ld",i+1);
+               }
+             break;
+           }
+         }
+  }
+}
+
+static void JPEGSetImageSamplingFactor(struct jpeg_decompress_struct *jpeg_info,  Image *image)
+{
+  char
+    sampling_factor[MaxTextExtent];
+
+  switch (jpeg_info->out_color_space)
+  {
+    case JCS_CMYK:
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: CMYK");
+      (void) FormatMagickString(sampling_factor,MaxTextExtent,
+        "%dx%d,%dx%d,%dx%d,%dx%d",jpeg_info->comp_info[0].h_samp_factor,
+        jpeg_info->comp_info[0].v_samp_factor,
+        jpeg_info->comp_info[1].h_samp_factor,
+        jpeg_info->comp_info[1].v_samp_factor,
+        jpeg_info->comp_info[2].h_samp_factor,
+        jpeg_info->comp_info[2].v_samp_factor,
+        jpeg_info->comp_info[3].h_samp_factor,
+        jpeg_info->comp_info[3].v_samp_factor);
+        break;
+    }
+    case JCS_GRAYSCALE:
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "Colorspace: GRAYSCALE");
+      (void) FormatMagickString(sampling_factor,MaxTextExtent,"%dx%d",
+        jpeg_info->comp_info[0].h_samp_factor,
+        jpeg_info->comp_info[0].v_samp_factor);
+      break;
+    }
+    case JCS_RGB:
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: RGB");
+      (void) FormatMagickString(sampling_factor,MaxTextExtent,
+        "%dx%d,%dx%d,%dx%d",jpeg_info->comp_info[0].h_samp_factor,
+        jpeg_info->comp_info[0].v_samp_factor,
+        jpeg_info->comp_info[1].h_samp_factor,
+        jpeg_info->comp_info[1].v_samp_factor,
+        jpeg_info->comp_info[2].h_samp_factor,
+        jpeg_info->comp_info[2].v_samp_factor);
+      break;
+    }
+    default:
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: %d",
+        jpeg_info->out_color_space);
+      (void) FormatMagickString(sampling_factor,MaxTextExtent,
+        "%dx%d,%dx%d,%dx%d,%dx%d",jpeg_info->comp_info[0].h_samp_factor,
+        jpeg_info->comp_info[0].v_samp_factor,
+        jpeg_info->comp_info[1].h_samp_factor,
+        jpeg_info->comp_info[1].v_samp_factor,
+        jpeg_info->comp_info[2].h_samp_factor,
+        jpeg_info->comp_info[2].v_samp_factor,
+        jpeg_info->comp_info[3].h_samp_factor,
+        jpeg_info->comp_info[3].v_samp_factor);
+      break;
+    }
+  }
+  (void) SetImageProperty(image,"jpeg:sampling-factor",sampling_factor);
+  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+    "Sampling Factors: %s",sampling_factor);
+}
+
 static Image *ReadJPEGImage(const ImageInfo *image_info,
   ExceptionInfo *exception)
 {
   char
-    sampling_factor[MaxTextExtent],
     value[MaxTextExtent];
 
   ErrorManager
     error_manager;
 
   IndexPacket
-    index;
+    jindex;
 
   Image
     *image;
 
   long
-    x,
     y;
 
   JSAMPLE
@@ -669,9 +873,6 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   MagickSizeType
     number_pixels;
 
-  register IndexPacket
-    *indexes;
-
   register long
     i;
 
@@ -684,9 +885,6 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   register JSAMPLE
     *p;
 
-  register PixelPacket
-    *q;
-
   unsigned long
     precision,
     units;
@@ -743,12 +941,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   if ((image->colorspace == YCbCrColorspace) ||
       (image->colorspace == Rec601YCbCrColorspace) ||
       (image->colorspace == Rec709YCbCrColorspace))
-    {
-      jpeg_info.out_color_space=JCS_YCbCr;
-      image->colorspace=YCbCrColorspace;
-    }
-  if (jpeg_info.out_color_space == JCS_CMYK)
-    image->colorspace=CMYKColorspace;
+    jpeg_info.out_color_space=JCS_YCbCr;
   /*
     Set image resolution.
   */
@@ -790,7 +983,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       jpeg_info.scale_denom=(unsigned int) image_info->subrange;
       jpeg_calc_output_dimensions(&jpeg_info);
     }
-  precision=jpeg_info.data_precision;
+  precision=(unsigned long) jpeg_info.data_precision;
 #if (JPEG_LIB_VERSION >= 61) && defined(D_PROGRESSIVE_SUPPORTED)
 #if defined(D_LOSSLESS_SUPPORTED)
   image->interlace=jpeg_info.process == JPROC_PROGRESSIVE ?
@@ -812,7 +1005,7 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   image->compression=JPEGCompression;
   image->interlace=JPEGInterlace;
 #endif
-  if ((image_info->colors != 0) && (image_info->colors <= 256))
+  if ((image_info->colors > 8) && (image_info->colors <= 256))
     {
       /*
         Let the JPEG library quantize for us.
@@ -824,6 +1017,10 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   image->columns=jpeg_info.output_width;
   image->rows=jpeg_info.output_height;
   image->depth=(unsigned long) jpeg_info.data_precision;
+  if (jpeg_info.out_color_space == JCS_YCbCr)
+    image->colorspace=YCbCrColorspace;
+  if (jpeg_info.out_color_space == JCS_CMYK)
+    image->colorspace=CMYKColorspace;
   if ((image_info->colors != 0) && (image_info->colors <= 256))
     if (AcquireImageColormap(image,image_info->colors) == MagickFalse)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -850,218 +1047,17 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Geometry: %dx%d",
         (int) jpeg_info.output_width,(int) jpeg_info.output_height);
     }
-  image->quality=UndefinedCompressionQuality;
-#if defined(D_PROGRESSIVE_SUPPORTED)
-  if (image->compression == LosslessJPEGCompression)
-    {
-      image->quality=100;
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-        "Quality: 100 (lossless)");
-    }
-  else
-#endif
-  {
-    long
-      value,
-      sum;
-
-    register long
-      j;
-
-    /*
-      Determine the JPEG compression quality from the quantization tables.
-    */
-    sum=0;
-    for (i=0; i < NUM_QUANT_TBLS; i++)
-    {
-      if (jpeg_info.quant_tbl_ptrs[i] != NULL)
-        for (j=0; j < DCTSIZE2; j++)
-          sum+=jpeg_info.quant_tbl_ptrs[i]->quantval[j];
-     }
-     if ((jpeg_info.quant_tbl_ptrs[0] != NULL) &&
-         (jpeg_info.quant_tbl_ptrs[1] != NULL))
-       {
-         long
-           hash[101] =
-           {
-             1020, 1015,  932,  848,  780,  735,  702,  679,  660,  645,
-              632,  623,  613,  607,  600,  594,  589,  585,  581,  571,
-              555,  542,  529,  514,  494,  474,  457,  439,  424,  410,
-              397,  386,  373,  364,  351,  341,  334,  324,  317,  309,
-              299,  294,  287,  279,  274,  267,  262,  257,  251,  247,
-              243,  237,  232,  227,  222,  217,  213,  207,  202,  198,
-              192,  188,  183,  177,  173,  168,  163,  157,  153,  148,
-              143,  139,  132,  128,  125,  119,  115,  108,  104,   99,
-               94,   90,   84,   79,   74,   70,   64,   59,   55,   49,
-               45,   40,   34,   30,   25,   20,   15,   11,    6,    4,
-                0
-           },
-           sums[101] =
-           {
-             32640, 32635, 32266, 31495, 30665, 29804, 29146, 28599, 28104,
-             27670, 27225, 26725, 26210, 25716, 25240, 24789, 24373, 23946,
-             23572, 22846, 21801, 20842, 19949, 19121, 18386, 17651, 16998,
-             16349, 15800, 15247, 14783, 14321, 13859, 13535, 13081, 12702,
-             12423, 12056, 11779, 11513, 11135, 10955, 10676, 10392, 10208,
-              9928,  9747,  9564,  9369,  9193,  9017,  8822,  8639,  8458,
-              8270,  8084,  7896,  7710,  7527,  7347,  7156,  6977,  6788,
-              6607,  6422,  6236,  6054,  5867,  5684,  5495,  5305,  5128,
-              4945,  4751,  4638,  4442,  4248,  4065,  3888,  3698,  3509,
-              3326,  3139,  2957,  2775,  2586,  2405,  2216,  2037,  1846,
-              1666,  1483,  1297,  1109,   927,   735,   554,   375,   201,
-               128,     0
-           };
-
-         value=(long) (jpeg_info.quant_tbl_ptrs[0]->quantval[2]+
-           jpeg_info.quant_tbl_ptrs[0]->quantval[53]+
-           jpeg_info.quant_tbl_ptrs[1]->quantval[0]+
-           jpeg_info.quant_tbl_ptrs[1]->quantval[DCTSIZE2-1]);
-         for (i=0; i < 100; i++)
-         {
-           if ((value < hash[i]) && (sum < sums[i]))
-             continue;
-           if ((value <= hash[i]) && (sum <= sums[i]))
-             image->quality=(unsigned long) i+1;
-           if (image->debug != MagickFalse)
-             {
-               if ((value > hash[i]) || (sum > sums[i]))
-                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                   "Quality: %ld",image->quality);
-               else
-                 (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                   "Quality: %ld (approximate)",i+1);
-             }
-           break;
-         }
-       }
-     else
-       if (jpeg_info.quant_tbl_ptrs[0] != NULL)
-         {
-           long
-             hash[101] =
-             {
-               510,  505,  422,  380,  355,  338,  326,  318,  311,  305,
-               300,  297,  293,  291,  288,  286,  284,  283,  281,  280,
-               279,  278,  277,  273,  262,  251,  243,  233,  225,  218,
-               211,  205,  198,  193,  186,  181,  177,  172,  168,  164,
-               158,  156,  152,  148,  145,  142,  139,  136,  133,  131,
-               129,  126,  123,  120,  118,  115,  113,  110,  107,  105,
-               102,  100,   97,   94,   92,   89,   87,   83,   81,   79,
-                76,   74,   70,   68,   66,   63,   61,   57,   55,   52,
-                50,   48,   44,   42,   39,   37,   34,   31,   29,   26,
-                24,   21,   18,   16,   13,   11,    8,    6,    3,    2,
-                 0
-             },
-             sums[101] =
-             {
-               16320, 16315, 15946, 15277, 14655, 14073, 13623, 13230, 12859,
-               12560, 12240, 11861, 11456, 11081, 10714, 10360, 10027,  9679,
-                9368,  9056,  8680,  8331,  7995,  7668,  7376,  7084,  6823,
-                6562,  6345,  6125,  5939,  5756,  5571,  5421,  5240,  5086,
-                4976,  4829,  4719,  4616,  4463,  4393,  4280,  4166,  4092,
-                3980,  3909,  3835,  3755,  3688,  3621,  3541,  3467,  3396,
-                3323,  3247,  3170,  3096,  3021,  2952,  2874,  2804,  2727,
-                2657,  2583,  2509,  2437,  2362,  2290,  2211,  2136,  2068,
-                1996,  1915,  1858,  1773,  1692,  1620,  1552,  1477,  1398,
-                1326,  1251,  1179,  1109,  1031,   961,   884,   814,   736,
-                 667,   592,   518,   441,   369,   292,   221,   151,    86,
-                  64,     0
-             };
-
-           value=(long) (jpeg_info.quant_tbl_ptrs[0]->quantval[2]+
-             jpeg_info.quant_tbl_ptrs[0]->quantval[53]);
-           for (i=0; i < 100; i++)
-           {
-             if ((value < hash[i]) && (sum < sums[i]))
-               continue;
-             image->quality=(unsigned long) i+1;
-             if (image->debug != MagickFalse)
-               {
-                 if ((value > hash[i]) || (sum > sums[i]))
-                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                     "Quality: %ld (approximate)",i+1);
-                 else
-                   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-                     "Quality: %ld",i+1);
-               }
-             break;
-           }
-         }
-  }
+  JPEGSetImageQuality(&jpeg_info,image);
+  JPEGSetImageSamplingFactor(&jpeg_info,image);
   (void) FormatMagickString(value,MaxTextExtent,"%ld",(long)
     jpeg_info.out_color_space);
   (void) SetImageProperty(image,"jpeg:colorspace",value);
-  /*
-    Set image sampling factors.
-  */
-  switch (jpeg_info.out_color_space)
-  {
-    case JCS_CMYK:
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: CMYK");
-      (void) FormatMagickString(sampling_factor,MaxTextExtent,
-        "%dx%d,%dx%d,%dx%d,%dx%d",jpeg_info.comp_info[0].h_samp_factor,
-        jpeg_info.comp_info[0].v_samp_factor,
-        jpeg_info.comp_info[1].h_samp_factor,
-        jpeg_info.comp_info[1].v_samp_factor,
-        jpeg_info.comp_info[2].h_samp_factor,
-        jpeg_info.comp_info[2].v_samp_factor,
-        jpeg_info.comp_info[3].h_samp_factor,
-        jpeg_info.comp_info[3].v_samp_factor);
-        break;
-    }
-    case JCS_GRAYSCALE:
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-        "Colorspace: GRAYSCALE");
-      (void) FormatMagickString(sampling_factor,MaxTextExtent,"%dx%d",
-        jpeg_info.comp_info[0].h_samp_factor,
-        jpeg_info.comp_info[0].v_samp_factor);
-      break;
-    }
-    case JCS_RGB:
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: RGB");
-      (void) FormatMagickString(sampling_factor,MaxTextExtent,
-        "%dx%d,%dx%d,%dx%d",jpeg_info.comp_info[0].h_samp_factor,
-        jpeg_info.comp_info[0].v_samp_factor,
-        jpeg_info.comp_info[1].h_samp_factor,
-        jpeg_info.comp_info[1].v_samp_factor,
-        jpeg_info.comp_info[2].h_samp_factor,
-        jpeg_info.comp_info[2].v_samp_factor);
-      break;
-    }
-    default:
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Colorspace: %d",
-        jpeg_info.out_color_space);
-      (void) FormatMagickString(sampling_factor,MaxTextExtent,
-        "%dx%d,%dx%d,%dx%d,%dx%d",jpeg_info.comp_info[0].h_samp_factor,
-        jpeg_info.comp_info[0].v_samp_factor,
-        jpeg_info.comp_info[1].h_samp_factor,
-        jpeg_info.comp_info[1].v_samp_factor,
-        jpeg_info.comp_info[2].h_samp_factor,
-        jpeg_info.comp_info[2].v_samp_factor,
-        jpeg_info.comp_info[3].h_samp_factor,
-        jpeg_info.comp_info[3].v_samp_factor);
-      break;
-    }
-  }
-  (void) SetImageProperty(image,"jpeg:sampling-factor",sampling_factor);
-  (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-    "Sampling Factors: %s",sampling_factor);
   if (image_info->ping != MagickFalse)
     {
       jpeg_destroy_decompress(&jpeg_info);
       (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      jpeg_destroy_decompress(&jpeg_info);
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
   jpeg_pixels=(JSAMPLE *) AcquireQuantumMemory((size_t) image->columns,
     jpeg_info.output_components*sizeof(JSAMPLE));
   if (jpeg_pixels == (JSAMPLE *) NULL)
@@ -1104,6 +1100,15 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
   scanline[0]=(JSAMPROW) jpeg_pixels;
   for (y=0; y < (long) image->rows; y++)
   {
+    register IndexPacket
+      *__restrict indexes;
+
+    register long
+      x;
+
+    register PixelPacket
+      *__restrict q;
+
     if (jpeg_read_scanlines(&jpeg_info,scanline,1) != 1)
       {
         (void) ThrowMagickException(&image->exception,GetMagickModule(),
@@ -1127,9 +1132,9 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
               pixel=(unsigned long) GETJSAMPLE(*p);
             else
               pixel=(unsigned long) ((GETJSAMPLE(*p) ^ 0x80) << 4);
-            index=ConstrainColormapIndex(image,pixel);
-            indexes[x]=index;
-            *q++=image->colormap[(int) index];
+            jindex=ConstrainColormapIndex(image,pixel);
+            indexes[x]=jindex;
+            *q++=image->colormap[(int) jindex];
             p++;
           }
         else
@@ -1164,9 +1169,9 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
       if (jpeg_info.output_components == 1)
         for (x=0; x < (long) image->columns; x++)
         {
-          index=ConstrainColormapIndex(image,(unsigned long) GETJSAMPLE(*p));
-          indexes[x]=(IndexPacket) index;
-          *q++=image->colormap[(int) index];
+          jindex=ConstrainColormapIndex(image,(unsigned long) GETJSAMPLE(*p));
+          indexes[x]=(IndexPacket) jindex;
+          *q++=image->colormap[(int) jindex];
           p++;
         }
       else
@@ -1235,9 +1240,19 @@ static Image *ReadJPEGImage(const ImageInfo *image_info,
 */
 ModuleExport unsigned long RegisterJPEGImage(void)
 {
+  char
+    version[MaxTextExtent];
+
   MagickInfo
     *entry;
 
+  static const char
+    *description = "Joint Photographic Experts Group JFIF format";
+
+  *version='\0';
+#if defined(JPEG_LIB_VERSION)
+  (void) FormatMagickString(version,MaxTextExtent,"%d",JPEG_LIB_VERSION);
+#endif
   entry=SetMagickInfo("JPEG");
   entry->thread_support=NoThreadSupport;
 #if defined(MAGICKCORE_JPEG_DELEGATE)
@@ -1246,17 +1261,9 @@ ModuleExport unsigned long RegisterJPEGImage(void)
 #endif
   entry->magick=(IsImageFormatHandler *) IsJPEG;
   entry->adjoin=MagickFalse;
-  entry->description=ConstantString(
-    "Joint Photographic Experts Group JFIF format");
-#if defined(JPEG_LIB_VERSION)
-  {
-    char
-      version[MaxTextExtent];
-
-    (void) FormatMagickString(version,MaxTextExtent,"%d",JPEG_LIB_VERSION);
+  entry->description=ConstantString(description);
+  if (*version != '\0')
     entry->version=ConstantString(version);
-  }
-#endif
   entry->module=ConstantString("JPEG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("JPG");
@@ -1266,8 +1273,9 @@ ModuleExport unsigned long RegisterJPEGImage(void)
   entry->encoder=(EncodeImageHandler *) WriteJPEGImage;
 #endif
   entry->adjoin=MagickFalse;
-  entry->description=ConstantString(
-    "Joint Photographic Experts Group JFIF format");
+  entry->description=ConstantString(description);
+  if (*version != '\0')
+    entry->version=ConstantString(version);
   entry->module=ConstantString("JPEG");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("PJPEG");
@@ -1277,8 +1285,9 @@ ModuleExport unsigned long RegisterJPEGImage(void)
   entry->encoder=(EncodeImageHandler *) WriteJPEGImage;
 #endif
   entry->adjoin=MagickFalse;
-  entry->description=ConstantString(
-    "Progessive Joint Photographic Experts Group JFIF");
+  entry->description=ConstantString(description);
+  if (*version != '\0')
+    entry->version=ConstantString(version);
   entry->module=ConstantString("JPEG");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
@@ -1399,6 +1408,9 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
   const StringInfo
     *profile;
 
+  MagickBooleanType
+    iptc;
+
   register long
     i;
 
@@ -1414,6 +1426,7 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
   /*
     Save image profile as a APP marker.
   */
+  iptc=MagickFalse;
   custom_profile=AcquireStringInfo(65535L);
   ResetImageProfileIterator(image);
   for (name=GetNextImageProfile(image); name != (const char *) NULL; )
@@ -1446,7 +1459,8 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
             custom_profile),(unsigned int) (length+tag_length));
         }
       }
-    if ((LocaleCompare(name,"IPTC") == 0) || (LocaleCompare(name,"8BIM") == 0))
+    if (((LocaleCompare(name,"IPTC") == 0) ||
+        (LocaleCompare(name,"8BIM") == 0)) && (iptc == MagickFalse))
       {
         register unsigned char
           *p;
@@ -1454,6 +1468,7 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
         unsigned long
           roundup;
 
+        iptc=MagickTrue;
         p=GetStringInfoDatum(custom_profile);
         if (LocaleNCompare((char *) GetStringInfoDatum(profile),"8BIM",4) == 0)
           {
@@ -1482,7 +1497,7 @@ static void WriteProfile(j_compress_ptr jpeg_info,Image *image)
       }
     if (LocaleCompare(name,"XMP") == 0)
       {
-        StringInfo      
+        StringInfo
           *xmp_profile;
 
         /*
@@ -1592,21 +1607,11 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  MagickSizeType
-    length;
-
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
-    *p;
-
   register JSAMPLE
     *q;
 
   register long
-    i,
-    x;
+    i;
 
   struct jpeg_compress_struct
     jpeg_info;
@@ -1648,6 +1653,9 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
   jpeg_info.client_data=(void *) &error_manager;
   jpeg_create_compress(&jpeg_info);
   JPEGDestinationManager(&jpeg_info,image);
+  if ((image->columns != (unsigned int) image->columns) ||
+      (image->rows != (unsigned int) image->rows))
+    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   jpeg_info.image_width=(unsigned int) image->columns;
   jpeg_info.image_height=(unsigned int) image->rows;
   jpeg_info.input_components=3;
@@ -1709,53 +1717,60 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       if (image->units == PixelsPerCentimeterResolution)
         jpeg_info.density_unit=(UINT8) 2;
     }
-    jpeg_info.dct_method=JDCT_DEFAULT;
-    option=GetImageOption(image_info,"jpeg:dct-method");
-    if (option != (const char *) NULL)
-      switch (*option)
-      {
-        case 'D':
-        case 'd':
-        {
-          if (LocaleCompare(option,"default") == 0)
-            jpeg_info.dct_method=JDCT_DEFAULT;
-          break;
-        }
-        case 'F':
-        case 'f':
-        {
-          if (LocaleCompare(option,"fastest") == 0)
-            jpeg_info.dct_method=JDCT_FASTEST;
-          if (LocaleCompare(option,"float") == 0)
-            jpeg_info.dct_method=JDCT_FLOAT;
-          break;
-        }
-        case 'I':
-        case 'i':
-        {
-          if (LocaleCompare(option,"ifast") == 0)
-            jpeg_info.dct_method=JDCT_IFAST;
-          if (LocaleCompare(option,"islow") == 0)
-            jpeg_info.dct_method=JDCT_ISLOW;
-          break;
-        }
-      }
-  length=(MagickSizeType) jpeg_info.input_components*image->columns*
-    image->rows*sizeof(JSAMPLE);
-  if (length == (MagickSizeType) ((size_t) length))
+  option=GetImageOption(image_info,"jpeg:dct-method");
+  if (option != (const char *) NULL)
+    switch (*option)
     {
-      /*
-        Perform optimization only if available memory resources permit it.
-      */
-      status=AcquireMagickResource(MemoryResource,length);
-      if (status != MagickFalse)
+      case 'D':
+      case 'd':
+      {
+        if (LocaleCompare(option,"default") == 0)
+          jpeg_info.dct_method=JDCT_DEFAULT;
+        break;
+      }
+      case 'F':
+      case 'f':
+      {
+        if (LocaleCompare(option,"fastest") == 0)
+          jpeg_info.dct_method=JDCT_FASTEST;
+        if (LocaleCompare(option,"float") == 0)
+          jpeg_info.dct_method=JDCT_FLOAT;
+        break;
+      }
+      case 'I':
+      case 'i':
+      {
+        if (LocaleCompare(option,"ifast") == 0)
+          jpeg_info.dct_method=JDCT_IFAST;
+        if (LocaleCompare(option,"islow") == 0)
+          jpeg_info.dct_method=JDCT_ISLOW;
+        break;
+      }
+    }
+  option=GetImageOption(image_info,"jpeg:optimize-coding");
+  if (option != (const char *) NULL)
+    {
+      jpeg_info.optimize_coding=MagickFalse;
+      if (IsMagickTrue(option) != MagickFalse)
+        jpeg_info.optimize_coding=MagickTrue;
+    }
+  else
+    {
+      MagickSizeType
+        length;
+
+      length=(MagickSizeType) jpeg_info.input_components*image->columns*
+        image->rows*sizeof(JSAMPLE);
+      if (length == (MagickSizeType) ((size_t) length))
         {
-          option=GetImageOption(image_info,"jpeg:optimize-coding");
-          if ((option == (const char *) NULL) ||
-              (LocaleCompare(option,"true") == 0))
+          /*
+            Perform optimization only if available memory resources permit it.
+          */
+          status=AcquireMagickResource(MemoryResource,length);
+          if (status != MagickFalse)
             jpeg_info.optimize_coding=MagickTrue;
+          RelinquishMagickResource(MemoryResource,length);
         }
-      RelinquishMagickResource(MemoryResource,length);
     }
 #if (JPEG_LIB_VERSION >= 61) && defined(C_PROGRESSIVE_SUPPORTED)
   if ((LocaleCompare(image_info->magick,"PJPEG") == 0) ||
@@ -1779,7 +1794,7 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       (image->quality <= 100))
     {
       if (image->quality == UndefinedCompressionQuality)
-        jpeg_set_quality(&jpeg_info,85,MagickTrue);
+        jpeg_set_quality(&jpeg_info,92,MagickTrue);
       else
         jpeg_set_quality(&jpeg_info,(int) image->quality,MagickTrue);
       if (image->debug != MagickFalse)
@@ -2020,6 +2035,12 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       if (jpeg_info.in_color_space == JCS_GRAYSCALE)
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *p;
+
+          register long
+            x;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
@@ -2039,6 +2060,12 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
             (jpeg_info.in_color_space == JCS_YCbCr))
           for (y=0; y < (long) image->rows; y++)
           {
+            register const PixelPacket
+              *p;
+
+            register long
+              x;
+
             p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
@@ -2058,6 +2085,15 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
         else
           for (y=0; y < (long) image->rows; y++)
           {
+            register const IndexPacket
+              *indexes;
+
+            register const PixelPacket
+              *p;
+
+            register long
+              x;
+
             p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
             if (p == (const PixelPacket *) NULL)
               break;
@@ -2084,6 +2120,12 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
     if (jpeg_info.in_color_space == JCS_GRAYSCALE)
       for (y=0; y < (long) image->rows; y++)
       {
+        register const PixelPacket
+          *p;
+
+        register long
+          x;
+
         p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
         if (p == (const PixelPacket *) NULL)
           break;
@@ -2102,6 +2144,12 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
           (jpeg_info.in_color_space == JCS_YCbCr))
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *p;
+
+          register long
+            x;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
@@ -2120,6 +2168,15 @@ static MagickBooleanType WriteJPEGImage(const ImageInfo *image_info,
       else
         for (y=0; y < (long) image->rows; y++)
         {
+          register const IndexPacket
+            *indexes;
+
+          register const PixelPacket
+            *p;
+
+          register long
+            x;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
