@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -100,8 +100,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -399,9 +398,6 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
   LinkedListInfo
     *profiles;
 
-  MagickOffsetType
-    offset;
-
   long
     y;
 
@@ -423,15 +419,8 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
   QuantumType
     quantum_type;
 
-  register IndexPacket
-    *indexes;
-
   register long
-    i,
-    x;
-
-  register PixelPacket
-    *q;
+    i;
 
   size_t
     length,
@@ -543,7 +532,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                 break;
               if (c == (int) '=')
                 break;
-              if ((size_t) (p-keyword) < MaxTextExtent)
+              if ((size_t) (p-keyword) < (MaxTextExtent-1))
                 *p++=(char) c;
               c=ReadBlobByte(image);
             } while (c != EOF);
@@ -620,8 +609,14 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               {
                 if (LocaleCompare(keyword,"class") == 0)
                   {
-                    image->storage_class=(ClassType) ParseMagickOption(
-                      MagickClassOptions,MagickFalse,options);
+                    long
+                      storage_class;
+
+                    storage_class=ParseMagickOption(MagickClassOptions,
+                      MagickFalse,options);
+                    if (storage_class < 0)
+                      break;
+                    image->storage_class=(ClassType) storage_class;
                     break;
                   }
                 if (LocaleCompare(keyword,"colors") == 0)
@@ -631,14 +626,26 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                   }
                 if (LocaleCompare(keyword,"colorspace") == 0)
                   {
-                    image->colorspace=(ColorspaceType) ParseMagickOption(
-                      MagickColorspaceOptions,MagickFalse,options);
+                    long
+                      colorspace;
+
+                    colorspace=ParseMagickOption(MagickColorspaceOptions,
+                      MagickFalse,options);
+                    if (colorspace < 0)
+                      break;
+                    image->colorspace=(ColorspaceType) colorspace;
                     break;
                   }
                 if (LocaleCompare(keyword,"compression") == 0)
                   {
-                    image->compression=(CompressionType) ParseMagickOption(
-                      MagickCompressOptions,MagickFalse,options);
+                    long
+                      compression;
+
+                    compression=ParseMagickOption(MagickCompressOptions,
+                      MagickFalse,options);
+                    if (compression < 0)
+                      break;
+                    image->compression=(CompressionType) compression;
                     break;
                   }
                 if (LocaleCompare(keyword,"columns") == 0)
@@ -664,8 +671,14 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                   }
                 if (LocaleCompare(keyword,"dispose") == 0)
                   {
-                    image->dispose=(DisposeType) ParseMagickOption(
-                      MagickDisposeOptions,MagickFalse,options);
+                    long
+                      dispose;
+
+                    dispose=ParseMagickOption(MagickDisposeOptions,MagickFalse,
+                      options);
+                    if (dispose < 0)
+                      break;
+                    image->dispose=(DisposeType) dispose;
                     break;
                   }
                 (void) SetImageProperty(image,keyword,options);
@@ -676,8 +689,14 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               {
                 if (LocaleCompare(keyword,"endian") == 0)
                   {
-                    image->endian=(EndianType) ParseMagickOption(
-                      MagickEndianOptions,MagickFalse,options);
+                    long
+                      endian;
+
+                    endian=ParseMagickOption(MagickEndianOptions,MagickFalse,
+                      options);
+                    if (endian < 0)
+                      break;
+                    image->endian=(EndianType) endian;
                     break;
                   }
                 (void) SetImageProperty(image,keyword,options);
@@ -691,6 +710,18 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                     image->gamma=atof(options);
                     break;
                   }
+                if (LocaleCompare(keyword,"gravity") == 0)
+                  {
+                    long
+                      gravity;
+
+                    gravity=ParseMagickOption(MagickGravityOptions,MagickFalse,
+                      options);
+                    if (gravity < 0)
+                      break;
+                    image->gravity=(GravityType) gravity;
+                    break;
+                  }
                 if (LocaleCompare(keyword,"green-primary") == 0)
                   {
                     flags=ParseGeometry(options,&geometry_info);
@@ -725,8 +756,14 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               {
                 if (LocaleCompare(keyword,"matte") == 0)
                   {
-                    image->matte=(MagickBooleanType) ParseMagickOption(
-                      MagickBooleanOptions,MagickFalse,options);
+                    long
+                      matte;
+
+                    matte=ParseMagickOption(MagickBooleanOptions,MagickFalse,
+                      options);
+                    if (matte < 0)
+                      break;
+                    image->matte=(MagickBooleanType) matte;
                     break;
                   }
                 if (LocaleCompare(keyword,"matte-color") == 0)
@@ -748,14 +785,26 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               {
                 if (LocaleCompare(keyword,"opaque") == 0)
                   {
-                    image->matte=(MagickBooleanType) ParseMagickOption(
-                      MagickBooleanOptions,MagickFalse,options);
+                    long
+                      matte;
+
+                    matte=ParseMagickOption(MagickBooleanOptions,MagickFalse,
+                      options);
+                    if (matte < 0)
+                      break;
+                    image->matte=(MagickBooleanType) matte;
                     break;
                   }
                 if (LocaleCompare(keyword,"orientation") == 0)
                   {
-                    image->orientation=(OrientationType) ParseMagickOption(
-                      MagickOrientationOptions,MagickFalse,options);
+                    long
+                      orientation;
+
+                    orientation=ParseMagickOption(MagickOrientationOptions,
+                      MagickFalse,options);
+                    if (orientation < 0)
+                      break;
+                    image->orientation=(OrientationType) orientation;
                     break;
                   }
                 (void) SetImageProperty(image,keyword,options);
@@ -801,8 +850,14 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                   }
                 if (LocaleCompare(keyword,"quantum-format") == 0)
                   {
-                    quantum_format=(QuantumFormatType) ParseMagickOption(
-                      MagickQuantumFormatOptions,MagickFalse,options);
+                    long
+                      format;
+
+                    format=ParseMagickOption(MagickQuantumFormatOptions,
+                      MagickFalse,options);
+                    if (format < 0)
+                      break;
+                    quantum_format=(QuantumFormatType) format;
                     break;
                   }
                 (void) SetImageProperty(image,keyword,options);
@@ -823,8 +878,14 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                   }
                 if (LocaleCompare(keyword,"rendering-intent") == 0)
                   {
-                    image->rendering_intent=(RenderingIntent) ParseMagickOption(
-                      MagickIntentOptions,MagickFalse,options);
+                    long
+                      rendering_intent;
+
+                    rendering_intent=ParseMagickOption(MagickIntentOptions,
+                      MagickFalse,options);
+                    if (rendering_intent < 0)
+                      break;
+                    image->rendering_intent=(RenderingIntent) rendering_intent;
                     break;
                   }
                 if (LocaleCompare(keyword,"resolution") == 0)
@@ -875,8 +936,14 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                   }
                 if (LocaleCompare(keyword,"type") == 0)
                   {
-                    image->type=(ImageType) ParseMagickOption(MagickTypeOptions,
-                      MagickFalse,options);
+                    long
+                      type;
+
+                    type=ParseMagickOption(MagickTypeOptions,MagickFalse,
+                      options);
+                    if (type < 0)
+                      break;
+                    image->type=(ImageType) type;
                     break;
                   }
                 (void) SetImageProperty(image,keyword,options);
@@ -887,8 +954,14 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               {
                 if (LocaleCompare(keyword,"units") == 0)
                   {
-                    image->units=(ResolutionType) ParseMagickOption(
-                      MagickResolutionOptions,MagickFalse,options);
+                    long
+                      units;
+
+                    units=ParseMagickOption(MagickResolutionOptions,MagickFalse,
+                      options);
+                    if (units < 0)
+                      break;
+                    image->units=(ResolutionType) units;
                     break;
                   }
                 (void) SetImageProperty(image,keyword,options);
@@ -1090,14 +1163,15 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
     /*
       Allocate image pixels.
     */
-    if (SetImageExtent(image,0,0) == MagickFalse)
-      {
-        InheritException(exception,&image->exception);
-        return(DestroyImageList(image));
-      }
     quantum_info=AcquireQuantumInfo(image_info,image);
+    if (quantum_info == (QuantumInfo *) NULL)
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
     if (quantum_format != UndefinedQuantumFormat)
-      (void) SetQuantumFormat(image,quantum_info,quantum_format);
+      {
+        status=SetQuantumFormat(image,quantum_info,quantum_format);
+        if (status == MagickFalse)
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      }
     packet_size=(size_t) (quantum_info->depth/8);
     if (image->storage_class == DirectClass)
       packet_size=(size_t) (3*quantum_info->depth/8);
@@ -1143,6 +1217,15 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
     length=0;
     for (y=0; y < (long) image->rows; y++)
     {
+      register IndexPacket
+        *__restrict indexes;
+
+      register long
+        x;
+
+      register PixelPacket
+        *__restrict q;
+
       q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         break;
@@ -1183,6 +1266,9 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
             {
               if (version == 0)
                 {
+                  MagickOffsetType
+                    offset;
+
                   offset=SeekBlob(image,-((MagickOffsetType) zip_info.avail_in),
                     SEEK_CUR);
                   if (offset < 0)
@@ -1193,7 +1279,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               if (code >= 0)
                 status=MagickTrue;
             }
-          (void) ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+          (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
             quantum_type,pixels,exception);
           break;
         }
@@ -1232,6 +1318,9 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
             {
               if (version == 0)
                 {
+                  MagickOffsetType
+                    offset;
+
                   offset=SeekBlob(image,-((MagickOffsetType)
                     bzip_info.avail_in),SEEK_CUR);
                   if (offset < 0)
@@ -1242,7 +1331,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               if (code >= 0)
                 status=MagickTrue;
             }
-          (void) ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+          (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
             quantum_type,pixels,exception);
           break;
         }
@@ -1273,7 +1362,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
         default:
         {
           count=ReadBlob(image,packet_size*image->columns,pixels);
-          (void) ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+          (void) ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
             quantum_type,pixels,exception);
           break;
         }
@@ -1621,15 +1710,8 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
   QuantumType
     quantum_type;
 
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
-    *p;
-
   register long
-    i,
-    x;
+    i;
 
   size_t
     length,
@@ -1667,32 +1749,40 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
     image->depth=image->depth <= 8 ? 8UL : image->depth <= 16 ? 16UL :
       image->depth <= 32 ? 32UL : 64UL;
     quantum_info=AcquireQuantumInfo(image_info,image);
-    if ((image->storage_class != PseudoClass) &&
+    if (quantum_info == (QuantumInfo *) NULL)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    if ((image->storage_class != PseudoClass) && (image->depth >= 32) &&
         (quantum_info->format == UndefinedQuantumFormat) &&
         (IsHighDynamicRangeImage(image,&image->exception) != MagickFalse))
-      (void) SetQuantumFormat(image,quantum_info,FloatingPointQuantumFormat);
+      {
+        status=SetQuantumFormat(image,quantum_info,FloatingPointQuantumFormat);
+        if (status == MagickFalse)
+          ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+      }
     if ((image->storage_class == PseudoClass) &&
         (image->colors > (unsigned long) (GetQuantumRange(image->depth)+1)))
       (void) SetImageStorageClass(image,DirectClass);
     if (IsGrayImage(image,&image->exception) != MagickFalse)
       {
         image->storage_class=DirectClass;
-        image->colorspace=GRAYColorspace;
+        (void) SetImageColorspace(image,GRAYColorspace);
       }
-    compression=NoCompression;
-    switch (image_info->compression)
+    compression=image->compression;
+    if (image_info->compression != UndefinedCompression)
+      compression=image_info->compression;
+    switch (compression)
     {
-#if defined(MAGICKCORE_ZLIB_DELEGATE)
+#if !defined(MAGICKCORE_ZLIB_DELEGATE)
       case LZWCompression:
-      case ZipCompression: compression=ZipCompression; break;
+      case ZipCompression: compression=NoCompression; break;
 #endif
-#if defined(MAGICKCORE_BZLIB_DELEGATE)
-      case BZipCompression: compression=BZipCompression; break;
+#if !defined(MAGICKCORE_BZLIB_DELEGATE)
+      case BZipCompression: compression=NoCompression; break;
 #endif
       case RLECompression:
       {
-        if (quantum_info->format != FloatingPointQuantumFormat)
-          compression=RLECompression;
+        if (quantum_info->format == FloatingPointQuantumFormat)
+          compression=NoCompression;
         break;
       }
       default:
@@ -1819,6 +1909,12 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
             (void) WriteBlobString(image,buffer);
           }
       }
+    if (image->gravity != UndefinedGravity)
+      {
+        (void) FormatMagickString(buffer,MaxTextExtent,"gravity=%s\n",
+          MagickOptionToMnemonic(MagickGravityOptions,image->gravity));
+        (void) WriteBlobString(image,buffer);
+      }
     if (image->dispose != UndefinedDispose)
       {
         (void) FormatMagickString(buffer,MaxTextExtent,"dispose=%s\n",
@@ -2027,6 +2123,15 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
     status=MagickTrue;
     for (y=0; y < (long) image->rows; y++)
     {
+      register const IndexPacket
+        *__restrict indexes;
+
+      register const PixelPacket
+        *__restrict p;
+
+      register long
+        x;
+
       p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
@@ -2051,7 +2156,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
             }
           zip_info.next_in=pixels;
           zip_info.avail_in=(uInt) (packet_size*image->columns);
-          (void) ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          (void) ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             quantum_type,pixels,&image->exception);
           do
           {
@@ -2106,7 +2211,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
             }
           bzip_info.next_in=(char *) pixels;
           bzip_info.avail_in=(unsigned int) (packet_size*image->columns);
-          (void) ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+          (void) ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
             quantum_type,pixels,&image->exception);
           do
           {
@@ -2178,8 +2283,8 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
         }
         default:
         {
-          (void) ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
-            quantum_type,pixels,&image->exception);
+          (void) ExportQuantumPixels(image,(const CacheView *) NULL,
+            quantum_info,quantum_type,pixels,&image->exception);
           (void) WriteBlob(image,packet_size*image->columns,pixels);
           break;
         }
