@@ -326,7 +326,6 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
       "  currentfile buffer readline pop",
       "  token pop /compression exch def pop",
       "  class 0 gt { PseudoClassImage } { DirectClassImage } ifelse",
-      "  grestore",
       (char *) NULL
     };
 
@@ -449,15 +448,16 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
     */
     delta.x=DefaultResolution;
     delta.y=DefaultResolution;
-    flags=ParseGeometry(PSDensityGeometry,&geometry_info);
-    resolution.x=geometry_info.rho;
-    resolution.y=geometry_info.sigma;
-    if ((flags & SigmaValue) == 0)
-      resolution.y=resolution.x;
-    if (image->x_resolution != 0.0)
-      resolution.x=image->x_resolution;
-    if (image->y_resolution != 0.0)
-      resolution.y=image->y_resolution;
+    resolution.x=image->x_resolution;
+    resolution.y=image->y_resolution;
+    if ((resolution.x == 0.0) || (resolution.y == 0.0))
+      {
+        flags=ParseGeometry(PSDensityGeometry,&geometry_info);
+        resolution.x=geometry_info.rho;
+        resolution.y=geometry_info.sigma;
+        if ((flags & SigmaValue) == 0)
+          resolution.y=resolution.x;
+      }
     if (image_info->density != (char *) NULL)
       {
         flags=ParseGeometry(image_info->density,&geometry_info);
@@ -522,9 +522,7 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
           image->filename);
         (void) WriteBlobString(image,buffer);
         timer=time((time_t *) NULL);
-        (void) localtime(&timer);
-        (void) CopyMagickString(date,ctime(&timer),MaxTextExtent);
-        date[strlen(date)-1]='\0';
+        (void) FormatMagickTime(timer,MaxTextExtent,date);
         (void) FormatMagickString(buffer,MaxTextExtent,
           "%%%%CreationDate: (%s)\n",date);
         (void) WriteBlobString(image,buffer);
@@ -704,17 +702,18 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
           {
             if (LocaleCompare(CCITTParam,"0") == 0)
               {
-                (void) HuffmanEncodeImage(image_info,image);
+                (void) HuffmanEncodeImage(image_info,image,image);
                 break;
               }
 #if defined(MAGICKCORE_TIFF_DELEGATE)
-            (void) Huffman2DEncodeImage(image_info,image);
+            (void) Huffman2DEncodeImage(image_info,image,image);
 #endif
             break;
           }
           case JPEGCompression:
           {
-            status=InjectImageBlob(image_info,image,"jpeg");
+            status=InjectImageBlob(image_info,image,image,"jpeg",
+              &image->exception);
             if (status == MagickFalse)
               ThrowWriterException(CoderError,image->exception.reason);
             break;
@@ -807,7 +806,8 @@ static MagickBooleanType WritePS2Image(const ImageInfo *image_info,Image *image)
           {
             case JPEGCompression:
             {
-              status=InjectImageBlob(image_info,image,"jpeg");
+              status=InjectImageBlob(image_info,image,image,"jpeg",
+                &image->exception);
               if (status == MagickFalse)
                 ThrowWriterException(CoderError,image->exception.reason);
               break;
