@@ -166,7 +166,7 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
   /*
     Create virtual canvas to support cropping (i.e. image.gray[100x100+10+20]).
   */
-  canvas_image=CloneImage(image,image->extract_info.width,1,MagickTrue,
+  canvas_image=CloneImage(image,image->extract_info.width,1,MagickFalse,
     exception);
   (void) SetImageVirtualPixelMethod(canvas_image,BlackVirtualPixelMethod);
   quantum_type=GrayQuantum;
@@ -206,7 +206,7 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
       q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         break;
-      length=ImportQuantumPixels(canvas_image,(ViewInfo *) NULL,quantum_info,
+      length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,quantum_info,
         quantum_type,pixels,exception);
       if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
         break;
@@ -569,7 +569,7 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
       p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
       if (p == (const PixelPacket *) NULL)
         break;
-      length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
+      length=ExportQuantumPixels(image,(const CacheView *) NULL,quantum_info,
         quantum_type,pixels,&image->exception);
       count=WriteBlob(image,length,pixels);
       if (count != (ssize_t) length)
