@@ -99,61 +99,61 @@ typedef struct _ExifInfo
 static const ExifInfo
   exif_info[] = {
     { EXIFTAG_EXPOSURETIME, TIFF_RATIONAL, "exif:ExposureTime" },
-    { EXIFTAG_FNUMBER, TIFF_RATIONAL, "exif:FNumber" }, 
-    { EXIFTAG_EXPOSUREPROGRAM, TIFF_SHORT, "exif:ExposureProgram" }, 
-    { EXIFTAG_SPECTRALSENSITIVITY, TIFF_ASCII, "exif:SpectralSensitivity" }, 
+    { EXIFTAG_FNUMBER, TIFF_RATIONAL, "exif:FNumber" },
+    { EXIFTAG_EXPOSUREPROGRAM, TIFF_SHORT, "exif:ExposureProgram" },
+    { EXIFTAG_SPECTRALSENSITIVITY, TIFF_ASCII, "exif:SpectralSensitivity" },
     /* { EXIFTAG_ISOSPEEDRATINGS, TIFF_LONG, "exif:ISOSpeedRatings" }, */
-    { EXIFTAG_OECF, TIFF_UNDEFINED, "exif:OptoelectricConversionFactor" }, 
-    { EXIFTAG_EXIFVERSION, TIFF_UNDEFINED, "exif:ExifVersion" }, 
-    { EXIFTAG_DATETIMEORIGINAL, TIFF_ASCII, "exif:DateTimeOriginal" }, 
-    { EXIFTAG_DATETIMEDIGITIZED, TIFF_ASCII, "exif:DateTimeDigitized" }, 
-    { EXIFTAG_COMPONENTSCONFIGURATION, TIFF_UNDEFINED, "exif:ComponentsConfiguration" }, 
-    { EXIFTAG_COMPRESSEDBITSPERPIXEL, TIFF_RATIONAL, "exif:CompressedBitsPerPixel" }, 
-    { EXIFTAG_SHUTTERSPEEDVALUE, TIFF_SRATIONAL, "exif:ShutterSpeedValue" }, 
-    { EXIFTAG_APERTUREVALUE, TIFF_RATIONAL, "exif:ApertureValue" }, 
-    { EXIFTAG_BRIGHTNESSVALUE, TIFF_SRATIONAL, "exif:BrightnessValue" }, 
-    { EXIFTAG_EXPOSUREBIASVALUE, TIFF_SRATIONAL, "exif:ExposureBiasValue" }, 
-    { EXIFTAG_MAXAPERTUREVALUE, TIFF_RATIONAL, "exif:MaxApertureValue" }, 
-    { EXIFTAG_SUBJECTDISTANCE, TIFF_RATIONAL, "exif:SubjectDistance" }, 
-    { EXIFTAG_METERINGMODE, TIFF_SHORT, "exif:MeteringMode" }, 
-    { EXIFTAG_LIGHTSOURCE, TIFF_SHORT, "exif:LightSource" }, 
-    { EXIFTAG_FLASH, TIFF_SHORT, "exif:Flash" }, 
-    { EXIFTAG_FOCALLENGTH, TIFF_RATIONAL, "exif:FocalLength" }, 
-    { EXIFTAG_SUBJECTAREA, TIFF_SHORT, "exif:SubjectArea" }, 
-    { EXIFTAG_MAKERNOTE, TIFF_UNDEFINED, "exif:MakerNote" }, 
-    { EXIFTAG_USERCOMMENT, TIFF_UNDEFINED, "exif:UserComment" }, 
-    { EXIFTAG_SUBSECTIME, TIFF_ASCII, "exif:SubSecTime" }, 
-    { EXIFTAG_SUBSECTIMEORIGINAL, TIFF_ASCII, "exif:SubSecTimeOriginal" }, 
-    { EXIFTAG_SUBSECTIMEDIGITIZED, TIFF_ASCII, "exif:SubSecTimeDigitized" }, 
-    { EXIFTAG_FLASHPIXVERSION, TIFF_UNDEFINED, "exif:FlashpixVersion" }, 
-    { EXIFTAG_PIXELXDIMENSION, TIFF_LONG, "exif:PixelXDimension" }, 
-    { EXIFTAG_PIXELXDIMENSION, TIFF_SHORT, "exif:PixelXDimension" }, 
-    { EXIFTAG_PIXELYDIMENSION, TIFF_LONG, "exif:PixelYDimension" }, 
-    { EXIFTAG_PIXELYDIMENSION, TIFF_SHORT, "exif:PixelYDimension" }, 
-    { EXIFTAG_RELATEDSOUNDFILE, TIFF_ASCII, "exif:RelatedSoundFile" }, 
-    { EXIFTAG_FLASHENERGY, TIFF_RATIONAL, "exif:FlashEnergy" }, 
-    { EXIFTAG_SPATIALFREQUENCYRESPONSE, TIFF_UNDEFINED, "exif:SpatialFrequencyResponse" }, 
-    { EXIFTAG_FOCALPLANEXRESOLUTION, TIFF_RATIONAL, "exif:FocalPlaneXResolution" }, 
-    { EXIFTAG_FOCALPLANEYRESOLUTION, TIFF_RATIONAL, "exif:FocalPlaneYResolution" }, 
-    { EXIFTAG_FOCALPLANERESOLUTIONUNIT, TIFF_SHORT, "exif:FocalPlaneResolutionUnit" }, 
-    { EXIFTAG_SUBJECTLOCATION, TIFF_SHORT, "exif:SubjectLocation" }, 
-    { EXIFTAG_EXPOSUREINDEX, TIFF_RATIONAL, "exif:ExposureIndex" }, 
-    { EXIFTAG_SENSINGMETHOD, TIFF_SHORT, "exif:SensingMethod" }, 
-    { EXIFTAG_FILESOURCE, TIFF_UNDEFINED, "exif:FileSource" }, 
-    { EXIFTAG_SCENETYPE, TIFF_UNDEFINED, "exif:SceneType" }, 
-    { EXIFTAG_CFAPATTERN, TIFF_UNDEFINED, "exif:CFAPattern" }, 
-    { EXIFTAG_CUSTOMRENDERED, TIFF_SHORT, "exif:CustomRendered" }, 
-    { EXIFTAG_EXPOSUREMODE, TIFF_SHORT, "exif:ExposureMode" }, 
-    { EXIFTAG_WHITEBALANCE, TIFF_SHORT, "exif:WhiteBalance" }, 
-    { EXIFTAG_DIGITALZOOMRATIO, TIFF_RATIONAL, "exif:DigitalZoomRatio" }, 
-    { EXIFTAG_FOCALLENGTHIN35MMFILM, TIFF_SHORT, "exif:FocalLengthIn35mmFilm" }, 
-    { EXIFTAG_SCENECAPTURETYPE, TIFF_SHORT, "exif:SceneCaptureType" }, 
-    { EXIFTAG_GAINCONTROL, TIFF_RATIONAL, "exif:GainControl" }, 
-    { EXIFTAG_CONTRAST, TIFF_SHORT, "exif:Contrast" }, 
-    { EXIFTAG_SATURATION, TIFF_SHORT, "exif:Saturation" }, 
-    { EXIFTAG_SHARPNESS, TIFF_SHORT, "exif:Sharpness" }, 
-    { EXIFTAG_DEVICESETTINGDESCRIPTION, TIFF_UNDEFINED, "exif:DeviceSettingDescription" }, 
-    { EXIFTAG_SUBJECTDISTANCERANGE, TIFF_SHORT, "exif:SubjectDistanceRange" }, 
+    { EXIFTAG_OECF, TIFF_UNDEFINED, "exif:OptoelectricConversionFactor" },
+    { EXIFTAG_EXIFVERSION, TIFF_UNDEFINED, "exif:ExifVersion" },
+    { EXIFTAG_DATETIMEORIGINAL, TIFF_ASCII, "exif:DateTimeOriginal" },
+    { EXIFTAG_DATETIMEDIGITIZED, TIFF_ASCII, "exif:DateTimeDigitized" },
+    { EXIFTAG_COMPONENTSCONFIGURATION, TIFF_UNDEFINED, "exif:ComponentsConfiguration" },
+    { EXIFTAG_COMPRESSEDBITSPERPIXEL, TIFF_RATIONAL, "exif:CompressedBitsPerPixel" },
+    { EXIFTAG_SHUTTERSPEEDVALUE, TIFF_SRATIONAL, "exif:ShutterSpeedValue" },
+    { EXIFTAG_APERTUREVALUE, TIFF_RATIONAL, "exif:ApertureValue" },
+    { EXIFTAG_BRIGHTNESSVALUE, TIFF_SRATIONAL, "exif:BrightnessValue" },
+    { EXIFTAG_EXPOSUREBIASVALUE, TIFF_SRATIONAL, "exif:ExposureBiasValue" },
+    { EXIFTAG_MAXAPERTUREVALUE, TIFF_RATIONAL, "exif:MaxApertureValue" },
+    { EXIFTAG_SUBJECTDISTANCE, TIFF_RATIONAL, "exif:SubjectDistance" },
+    { EXIFTAG_METERINGMODE, TIFF_SHORT, "exif:MeteringMode" },
+    { EXIFTAG_LIGHTSOURCE, TIFF_SHORT, "exif:LightSource" },
+    { EXIFTAG_FLASH, TIFF_SHORT, "exif:Flash" },
+    { EXIFTAG_FOCALLENGTH, TIFF_RATIONAL, "exif:FocalLength" },
+    { EXIFTAG_SUBJECTAREA, TIFF_SHORT, "exif:SubjectArea" },
+    { EXIFTAG_MAKERNOTE, TIFF_UNDEFINED, "exif:MakerNote" },
+    { EXIFTAG_USERCOMMENT, TIFF_UNDEFINED, "exif:UserComment" },
+    { EXIFTAG_SUBSECTIME, TIFF_ASCII, "exif:SubSecTime" },
+    { EXIFTAG_SUBSECTIMEORIGINAL, TIFF_ASCII, "exif:SubSecTimeOriginal" },
+    { EXIFTAG_SUBSECTIMEDIGITIZED, TIFF_ASCII, "exif:SubSecTimeDigitized" },
+    { EXIFTAG_FLASHPIXVERSION, TIFF_UNDEFINED, "exif:FlashpixVersion" },
+    { EXIFTAG_PIXELXDIMENSION, TIFF_LONG, "exif:PixelXDimension" },
+    { EXIFTAG_PIXELXDIMENSION, TIFF_SHORT, "exif:PixelXDimension" },
+    { EXIFTAG_PIXELYDIMENSION, TIFF_LONG, "exif:PixelYDimension" },
+    { EXIFTAG_PIXELYDIMENSION, TIFF_SHORT, "exif:PixelYDimension" },
+    { EXIFTAG_RELATEDSOUNDFILE, TIFF_ASCII, "exif:RelatedSoundFile" },
+    { EXIFTAG_FLASHENERGY, TIFF_RATIONAL, "exif:FlashEnergy" },
+    { EXIFTAG_SPATIALFREQUENCYRESPONSE, TIFF_UNDEFINED, "exif:SpatialFrequencyResponse" },
+    { EXIFTAG_FOCALPLANEXRESOLUTION, TIFF_RATIONAL, "exif:FocalPlaneXResolution" },
+    { EXIFTAG_FOCALPLANEYRESOLUTION, TIFF_RATIONAL, "exif:FocalPlaneYResolution" },
+    { EXIFTAG_FOCALPLANERESOLUTIONUNIT, TIFF_SHORT, "exif:FocalPlaneResolutionUnit" },
+    { EXIFTAG_SUBJECTLOCATION, TIFF_SHORT, "exif:SubjectLocation" },
+    { EXIFTAG_EXPOSUREINDEX, TIFF_RATIONAL, "exif:ExposureIndex" },
+    { EXIFTAG_SENSINGMETHOD, TIFF_SHORT, "exif:SensingMethod" },
+    { EXIFTAG_FILESOURCE, TIFF_UNDEFINED, "exif:FileSource" },
+    { EXIFTAG_SCENETYPE, TIFF_UNDEFINED, "exif:SceneType" },
+    { EXIFTAG_CFAPATTERN, TIFF_UNDEFINED, "exif:CFAPattern" },
+    { EXIFTAG_CUSTOMRENDERED, TIFF_SHORT, "exif:CustomRendered" },
+    { EXIFTAG_EXPOSUREMODE, TIFF_SHORT, "exif:ExposureMode" },
+    { EXIFTAG_WHITEBALANCE, TIFF_SHORT, "exif:WhiteBalance" },
+    { EXIFTAG_DIGITALZOOMRATIO, TIFF_RATIONAL, "exif:DigitalZoomRatio" },
+    { EXIFTAG_FOCALLENGTHIN35MMFILM, TIFF_SHORT, "exif:FocalLengthIn35mmFilm" },
+    { EXIFTAG_SCENECAPTURETYPE, TIFF_SHORT, "exif:SceneCaptureType" },
+    { EXIFTAG_GAINCONTROL, TIFF_RATIONAL, "exif:GainControl" },
+    { EXIFTAG_CONTRAST, TIFF_SHORT, "exif:Contrast" },
+    { EXIFTAG_SATURATION, TIFF_SHORT, "exif:Saturation" },
+    { EXIFTAG_SHARPNESS, TIFF_SHORT, "exif:Sharpness" },
+    { EXIFTAG_DEVICESETTINGDESCRIPTION, TIFF_UNDEFINED, "exif:DeviceSettingDescription" },
+    { EXIFTAG_SUBJECTDISTANCERANGE, TIFF_SHORT, "exif:SubjectDistanceRange" },
     { EXIFTAG_IMAGEUNIQUEID, TIFF_ASCII, "exif:ImageUniqueID" },
     { 0, 0, (char *) NULL }
 };
@@ -195,8 +195,7 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
@@ -461,6 +460,9 @@ static void TIFFGetEXIFProperties(TIFF *tiff,Image *image)
       (void) SetImageProperty(image,exif_info[i].property,value);
   }
   TIFFSetDirectory(tiff,directory);
+#else
+  (void) tiff;
+  (void) image;
 #endif
 }
 
@@ -585,14 +587,11 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     quantum_type;
 
   register long
-    i,
-    x;
-
-  register PixelPacket
-    *q;
+    i;
 
   size_t
-    length;
+    length,
+    pad;
 
   TIFF
     *tiff;
@@ -630,8 +629,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     *pixels;
 
   unsigned long
-    lsb_first,
-    pad;
+    lsb_first;
 
   /*
     Open image.
@@ -696,6 +694,44 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_MINSAMPLEVALUE,&min_sample_value);
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_MAXSAMPLEVALUE,&max_sample_value);
     (void) TIFFGetFieldDefaulted(tiff,TIFFTAG_PHOTOMETRIC,&photometric);
+    switch (photometric)
+    {
+      case PHOTOMETRIC_MINISBLACK:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","min-is-black");
+        break;
+      }
+      case PHOTOMETRIC_MINISWHITE:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","min-is-white");
+        break;
+      }
+      case PHOTOMETRIC_PALETTE:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","palette");
+        break;
+      }
+      case PHOTOMETRIC_RGB:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","RGB");
+        break;
+      }
+      case PHOTOMETRIC_CIELAB:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","CIELAB");
+        break;
+      }
+      case PHOTOMETRIC_SEPARATED:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","separated");
+        break;
+      }
+      default:
+      {
+        (void) SetImageProperty(image,"tiff:photometric","unknown");
+        break;
+      }
+    }
     if (image->debug != MagickFalse)
       {
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Geometry: %ux%u",
@@ -708,52 +744,15 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           "Min sample value: %u",min_sample_value);
         (void) LogMagickEvent(CoderEvent,GetMagickModule(),
           "Max sample value: %u",max_sample_value);
-        switch (photometric)
-        {
-          case PHOTOMETRIC_MINISBLACK:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: MINISBLACK");
-            break;
-          }
-          case PHOTOMETRIC_MINISWHITE:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: MINISWHITE");
-            break;
-          }
-          case PHOTOMETRIC_PALETTE:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: PALETTE");
-            break;
-          }
-          case PHOTOMETRIC_RGB:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: RGB");
-            break;
-          }
-          case PHOTOMETRIC_CIELAB:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: CIELAB");
-            break;
-          }
-          case PHOTOMETRIC_SEPARATED:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric: SEPARATED");
-            break;
-          }
-          default:
-          {
-            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-              "Photometric interpretation: %u",photometric);
-            break;
-          }
-        }
+        (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Photometric "
+          "interpretation: %u",GetImageProperty(image,"tiff:photometric"));
       }
+    image->columns=(unsigned long) width;
+    image->rows=(unsigned long) height;
+    image->depth=(unsigned long) bits_per_sample;
+    if (image->debug != MagickFalse)
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Image depth: %lu",
+        image->depth);
     lsb_first=1;
     image->endian=MSBEndian;
     if ((int) (*(char *) &lsb_first) != 0)
@@ -845,12 +844,6 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
       case COMPRESSION_ADOBE_DEFLATE: image->compression=ZipCompression; break;
       default: image->compression=RLECompression; break;
     }
-    image->columns=width;
-    image->rows=height;
-    image->depth=(unsigned long) bits_per_sample;
-    if (image->debug != MagickFalse)
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),"Image depth: %lu",
-        image->depth);
     quantum_info=AcquireQuantumInfo(image_info,image);
     if (quantum_info == (QuantumInfo *) NULL)
       ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
@@ -1040,13 +1033,16 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           int
             status;
 
+          register PixelPacket
+            *__restrict q;
+
           status=TIFFReadPixels(tiff,bits_per_sample,0,y,(char *) pixels);
           if (status == -1)
             break;
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+          length=ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
             quantum_type,pixels,exception);
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
@@ -1090,13 +1086,16 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           int
             status;
 
+          register PixelPacket
+            *__restrict q;
+
           status=TIFFReadPixels(tiff,bits_per_sample,0,y,(char *) pixels);
           if (status == -1)
             break;
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
-          length=ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+          length=ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
             quantum_type,pixels,exception);
           if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
@@ -1116,6 +1115,10 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
         */
         for (y=0; y < (long) image->rows; y++)
         {
+          register PixelPacket
+            *__restrict q;
+
+          (void) TIFFReadPixels(tiff,bits_per_sample,0,y,(char *) pixels);
           for (i=0; i < (long) samples_per_pixel; i++)
           {
             int
@@ -1147,7 +1150,7 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
                 case 4: quantum_type=AlphaQuantum; break;
                 default: quantum_type=UndefinedQuantum; break;
               }
-            length=ImportQuantumPixels(image,(ViewInfo *) NULL,quantum_info,
+            length=ImportQuantumPixels(image,(CacheView *) NULL,quantum_info,
               quantum_type,pixels,exception);
             if (SyncAuthenticPixels(image,exception) == MagickFalse)
               break;
@@ -1173,6 +1176,12 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
         p=(unsigned char *) NULL;
         for (y=0; y < (long) image->rows; y++)
         {
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
@@ -1186,11 +1195,11 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           p=pixels+image->columns*i;
           for (x=0; x < (long) image->columns; x++)
           {
-            q->red=ScaleCharToQuantum((unsigned char) TIFFGetR(*p));
-            q->green=ScaleCharToQuantum((unsigned char) TIFFGetG(*p));
-            q->blue=ScaleCharToQuantum((unsigned char) TIFFGetB(*p));
+            q->red=ScaleCharToQuantum((unsigned char) (TIFFGetR(*p)));
+            q->green=ScaleCharToQuantum((unsigned char) (TIFFGetG(*p)));
+            q->blue=ScaleCharToQuantum((unsigned char) (TIFFGetB(*p)));
             if (image->matte != MagickFalse)
-              q->opacity=ScaleCharToQuantum((unsigned char) TIFFGetA(*p));
+              q->opacity=ScaleCharToQuantum((unsigned char) (TIFFGetA(*p)));
             p++;
             q++;
           }
@@ -1241,6 +1250,12 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
           PixelPacket
             *tile;
 
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           unsigned long
             columns_remaining,
             rows_remaining;
@@ -1337,6 +1352,12 @@ static Image *ReadTIFFImage(const ImageInfo *image_info,
         p=pixels+number_pixels-1;
         for (y=0; y < (long) image->rows; y++)
         {
+          register long
+            x;
+
+          register PixelPacket
+            *__restrict q;
+
           q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
           if (q == (PixelPacket *) NULL)
             break;
@@ -1606,7 +1627,7 @@ static MagickBooleanType WritePTIFImage(const ImageInfo *image_info,
   images=NewImageList();
   for (next=image; next != (Image *) NULL; next=GetNextImageInList(next))
   {
-    AppendImageToList(&images,CloneImage(next,0,0,MagickTrue,
+    AppendImageToList(&images,CloneImage(next,0,0,MagickFalse,
       &image->exception));
     columns=next->columns;
     rows=next->rows;
@@ -1945,6 +1966,9 @@ static void TIFFSetEXIFProperties(TIFF *tiff,Image *image)
     }
   }
   /* (void) TIFFSetField(tiff,TIFFTAG_EXIFIFD,offset); */
+#else
+  (void) tiff;
+  (void) image;
 #endif
 }
 
@@ -1981,9 +2005,6 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
   QuantumType
     quantum_type;
 
-  register const PixelPacket
-    *p;
-
   register long
     i;
 
@@ -2006,12 +2027,14 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
     endian,
     photometric;
 
+  uint32
+    rows_per_strip;
+
   unsigned char
     *pixels;
 
   unsigned long
-    lsb_first,
-    rows_per_strip;
+    lsb_first;
 
   /*
     Open TIFF file.
@@ -2065,7 +2088,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
     quantum_info=AcquireQuantumInfo(image_info,image);
     if (quantum_info == (QuantumInfo *) NULL)
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-    if ((image->storage_class != PseudoClass) &&
+    if ((image->storage_class != PseudoClass) && (image->depth >= 32) &&
         (quantum_info->format == UndefinedQuantumFormat) &&
         (IsHighDynamicRangeImage(image,&image->exception) != MagickFalse))
       {
@@ -2076,6 +2099,9 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
     if ((LocaleCompare(image_info->magick,"PTIF") == 0) &&
         (GetPreviousImageInList(image) != (Image *) NULL))
       (void) TIFFSetField(tiff,TIFFTAG_SUBFILETYPE,FILETYPE_REDUCEDIMAGE);
+    if ((image->columns != (uint32) image->columns) ||
+        (image->rows != (uint32) image->rows))
+      ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
     (void) TIFFSetField(tiff,TIFFTAG_IMAGELENGTH,(uint32) image->rows);
     (void) TIFFSetField(tiff,TIFFTAG_IMAGEWIDTH,(uint32) image->columns);
     compression=image->compression;
@@ -2086,11 +2112,13 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
       case FaxCompression:
       {
         compress_tag=COMPRESSION_CCITTFAX3;
+        SetQuantumMinIsWhite(quantum_info,MagickTrue);
         break;
       }
       case Group4Compression:
       {
         compress_tag=COMPRESSION_CCITTFAX4;
+        SetQuantumMinIsWhite(quantum_info,MagickTrue);
         break;
       }
       case JPEGCompression:
@@ -2215,7 +2243,7 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
           else
             {
               if (image->colorspace != RGBColorspace)
-                (void) SetImageColorspace(image,RGBColorspace);
+                (void) TransformImageColorspace(image,RGBColorspace);
               photometric=PHOTOMETRIC_RGB;
             }
         (void) TIFFSetField(tiff,TIFFTAG_SAMPLESPERPIXEL,3);
@@ -2529,11 +2557,14 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
               quantum_type=RGBAQuantum;
             for (y=0; y < (long) image->rows; y++)
             {
+              register const PixelPacket
+                *__restrict p;
+
               p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+              length=ExportQuantumPixels(image,(const CacheView *) NULL,
                 quantum_info,quantum_type,pixels,&image->exception);
               if (TIFFWritePixels(tiff,&tiff_info,y,0,image) == -1)
                 break;
@@ -2554,11 +2585,14 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
             */
             for (y=0; y < (long) image->rows; y++)
             {
+              register const PixelPacket
+                *__restrict p;
+
               p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+              length=ExportQuantumPixels(image,(const CacheView *) NULL,
                 quantum_info,RedQuantum,pixels,&image->exception);
               if (TIFFWritePixels(tiff,&tiff_info,y,0,image) == -1)
                 break;
@@ -2571,11 +2605,14 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
               }
             for (y=0; y < (long) image->rows; y++)
             {
+              register const PixelPacket
+                *__restrict p;
+
               p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+              length=ExportQuantumPixels(image,(const CacheView *) NULL,
                 quantum_info,GreenQuantum,pixels,&image->exception);
               if (TIFFWritePixels(tiff,&tiff_info,y,1,image) == -1)
                 break;
@@ -2588,11 +2625,14 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
               }
             for (y=0; y < (long) image->rows; y++)
             {
+              register const PixelPacket
+                *__restrict p;
+
               p=GetVirtualPixels(image,0,y,image->columns,1,
                 &image->exception);
               if (p == (const PixelPacket *) NULL)
                 break;
-              length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+              length=ExportQuantumPixels(image,(const CacheView *) NULL,
                 quantum_info,BlueQuantum,pixels,&image->exception);
               if (TIFFWritePixels(tiff,&tiff_info,y,2,image) == -1)
                 break;
@@ -2606,11 +2646,14 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
             if (image->matte != MagickFalse)
               for (y=0; y < (long) image->rows; y++)
               {
+              register const PixelPacket
+                *__restrict p;
+
                 p=GetVirtualPixels(image,0,y,image->columns,1,
                   &image->exception);
                 if (p == (const PixelPacket *) NULL)
                   break;
-                length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+                length=ExportQuantumPixels(image,(const CacheView *) NULL,
                   quantum_info,AlphaQuantum,pixels,&image->exception);
                 if (TIFFWritePixels(tiff,&tiff_info,y,3,image) == -1)
                   break;
@@ -2635,13 +2678,16 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
         if (image->matte != MagickFalse)
           quantum_type=CMYKAQuantum;
         if (image->colorspace != CMYKColorspace)
-          (void) SetImageColorspace(image,CMYKColorspace);
+          (void) TransformImageColorspace(image,CMYKColorspace);
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
             quantum_info,quantum_type,pixels,&image->exception);
           if (TIFFWritePixels(tiff,&tiff_info,y,0,image) == -1)
             break;
@@ -2705,10 +2751,13 @@ static MagickBooleanType WriteTIFFImage(const ImageInfo *image_info,
              quantum_type=GrayQuantum;
         for (y=0; y < (long) image->rows; y++)
         {
+          register const PixelPacket
+            *__restrict p;
+
           p=GetVirtualPixels(image,0,y,image->columns,1,&image->exception);
           if (p == (const PixelPacket *) NULL)
             break;
-          length=ExportQuantumPixels(image,(const ViewInfo *) NULL,
+          length=ExportQuantumPixels(image,(const CacheView *) NULL,
             quantum_info,quantum_type,pixels,&image->exception);
           if (TIFFWritePixels(tiff,&tiff_info,y,0,image) == -1)
             break;
