@@ -65,6 +65,7 @@
 #include "magick/resource_.h"
 #include "magick/statistic.h"
 #include "magick/string_.h"
+#include "magick/token-private.h"
 #include "magick/transform.h"
 #include "magick/type.h"
 #include "magick/utility.h"
@@ -269,7 +270,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
         offset.x=(geometry.width == 0 ? 1.0 : -1.0)*annotate_info->affine.tx+
           geometry.width+i*annotate_info->affine.ry*height-
           annotate_info->affine.sx*(metrics.width+metrics.bounds.x1)+
-          annotate_info->affine.ry*(metrics.ascent+metrics.descent);
+          annotate_info->affine.ry*(metrics.ascent+metrics.descent)-1.0;
         offset.y=(geometry.height == 0 ? -1.0 : 1.0)*annotate_info->affine.ty+i*
           annotate_info->affine.sy*height+annotate_info->affine.sy*
           metrics.ascent-annotate_info->affine.rx*(metrics.width-
@@ -308,7 +309,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
           geometry.width+i*annotate_info->affine.ry*height-
           annotate_info->affine.sx*(metrics.width+metrics.bounds.x1)+
           annotate_info->affine.ry*(metrics.ascent+metrics.descent-
-          (number_lines-1.0)*height)/2.0;
+          (number_lines-1.0)*height)/2.0-1.0;
         offset.y=(geometry.height == 0 ? -1.0 : 1.0)*annotate_info->affine.ty+
           geometry.height/2.0+i*annotate_info->affine.sy*height-
           annotate_info->affine.rx*(metrics.width+metrics.bounds.x1)+
@@ -343,7 +344,7 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
         offset.x=(geometry.width == 0 ? 1.0 : -1.0)*annotate_info->affine.tx+
           geometry.width+i*annotate_info->affine.ry*height-
           annotate_info->affine.sx*(metrics.width+metrics.bounds.x1)-
-          annotate_info->affine.ry*(number_lines-1.0)*height;
+          annotate_info->affine.ry*(number_lines-1.0)*height-1.0;
         offset.y=(geometry.height == 0 ? 1.0 : -1.0)*annotate_info->affine.ty+
           geometry.height+i*annotate_info->affine.sy*height-
           annotate_info->affine.rx*(metrics.width+metrics.bounds.x1)-
@@ -441,297 +442,6 @@ MagickExport MagickBooleanType AnnotateImage(Image *image,
   return(status);
 }
 
-#if defined(MAGICKCORE_FREETYPE_DELEGATE)
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-+   E n c o d e S J I S                                                       %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  EncodeSJIS() converts an ASCII text string to 2-bytes per character code
-%  (like UCS-2).  Returns the translated codes and the character count.
-%  Characters under 0x7f are just copied, characters over 0x80 are tied with
-%  the next character.
-%
-%  Katsutoshi Shibuya contributed this method.
-%
-%  The format of the EncodeSJIS function is:
-%
-%      encoding=EncodeSJIS(const Image *image,const char *text,size_t count)
-%
-%  A description of each parameter follows:
-%
-%    o image: the image.
-%
-%    o text: the text.
-%
-%    o count: return the number of characters generated by the encoding.
-%
-*/
-
-static int GetOneCharacter(const unsigned char *text,size_t *length)
-{
-  int
-    c;
-
-  if (*length < 1)
-    return(-1);
-  c=text[0];
-  if ((c & 0x80) == 0)
-    {
-      *length=1;
-      return((int) c);
-    }
-  if (*length < 2)
-    {
-      *length=0;
-      return(-1);
-    }
-  *length=2;
-  c=((int) (text[0]) << 8);
-  c|=text[1];
-  return((int) c);
-}
-
-static unsigned long *EncodeSJIS(const char *text,size_t *count)
-{
-  int
-    c;
-
-  register const char
-    *p;
-
-  register unsigned long
-    *q;
-
-  size_t
-    length;
-
-  unsigned long
-    *encoding;
-
-  *count=0;
-  if ((text == (char *) NULL) || (*text == '\0'))
-    return((unsigned long *) NULL);
-  length=strlen(text);
-  if (~length < MaxTextExtent)
-    return((unsigned long *) NULL);
-  length+=MaxTextExtent;
-  encoding=(unsigned long *) AcquireQuantumMemory(length,sizeof(*encoding));
-  if (encoding == (unsigned long *) NULL)
-    return((unsigned long *) NULL);
-  q=encoding;
-  for (p=text; *p != '\0'; p+=length)
-  {
-    length=strlen(p);
-    c=GetOneCharacter((const unsigned char *) p,&length);
-    if (c < 0)
-      {
-        q=encoding;
-        for (p=text; *p != '\0'; p++)
-          *q++=(unsigned char) *p;
-        break;
-      }
-    *q=(unsigned long) c;
-    q++;
-  }
-  *count=q-encoding;
-  return(encoding);
-}
-#endif
-
-#if defined(MAGICKCORE_FREETYPE_DELEGATE)
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-+   E n c o d e T e x t                                                       %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  EncodeText() converts an ASCII text string to wide text and returns the
-%  translation and the character count.
-%
-%  The format of the EncodeText function is:
-%
-%      encoding=EncodeText(const char *text,size_t count)
-%
-%  A description of each parameter follows:
-%
-%    o text: the text.
-%
-%    o count: return the number of characters generated by the encoding.
-%
-*/
-static unsigned long *EncodeText(const char *text,size_t *count)
-{
-  register const char
-    *p;
-
-  register unsigned long
-    *q;
-
-  size_t
-    length;
-
-  unsigned long
-    *encoding;
-
-  *count=0;
-  if ((text == (char *) NULL) || (*text == '\0'))
-    return((unsigned long *) NULL);
-  length=strlen(text);
-  if (~length < MaxTextExtent)
-    return((unsigned long *) NULL);
-  length+=MaxTextExtent;
-  encoding=(unsigned long *) AcquireQuantumMemory(length,sizeof(*encoding));
-  if (encoding == (unsigned long *) NULL)
-    return((unsigned long *) NULL);
-  q=encoding;
-  for (p=text; *p != '\0'; p++)
-    *q++=(unsigned char) *p;
-  *count=q-encoding;
-  return(encoding);
-}
-#endif
-
-#if defined(MAGICKCORE_FREETYPE_DELEGATE)
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-+   E n c o d e U n i c o d e                                                 %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  EncodeUnicode() converts an ASCII text string to Unicode and returns the
-%  Unicode translation and the character count.  Characters under 0x7f are
-%  just copied, characters over 0x80 are tied with the next character.
-%
-%  The format of the EncodeUnicode function is:
-%
-%      unsigned short *EncodeUnicode(const unsigned char *text,size_t count)
-%
-%  A description of each parameter follows:
-%
-%    o unicode:  EncodeUnicode() returns a pointer to an unsigned short array
-%      representing the encoded version of the ASCII string.
-%
-%    o text: the text.
-%
-%    o count: return the number of characters generated by the encoding.
-%
-*/
-
-static long GetUnicodeCharacter(const unsigned char *text,size_t *length)
-{
-  unsigned long
-    c;
-
-  if (*length < 1)
-    return(-1);
-  c=text[0];
-  if ((c & 0x80) == 0)
-    {
-      *length=1;
-      return((long) c);
-    }
-  if ((*length < 2) || ((text[1] & 0xc0) != 0x80))
-    {
-      *length=0;
-      return(-1);
-    }
-  if ((c & 0xe0) != 0xe0)
-    {
-      *length=2;
-      c=(text[0] & 0x1f) << 6;
-      c|=text[1] & 0x3f;
-      return((long) c);
-    }
-  if ((*length < 3) || ((text[2] & 0xc0) != 0x80))
-    {
-      *length=0;
-      return(-1);
-    }
-  if ((c & 0xf0) != 0xf0)
-    {
-      *length=3;
-      c=(text[0] & 0xf) << 12;
-      c|=(text[1] & 0x3f) << 6;
-      c|=text[2] & 0x3f;
-      return((long) c);
-    }
-  if ((*length < 4) || ((c & 0xf8) != 0xf0) || ((text[3] & 0xc0) != 0x80))
-    {
-      *length=0;
-      return(-1);
-    }
-  *length=4;
-  c=(text[0] & 0x7) << 18;
-  c|=(text[1] & 0x3f) << 12;
-  c|=(text[2] & 0x3f) << 6;
-  c|=text[3] & 0x3f;
-  return((long) c);
-}
-
-static unsigned long *EncodeUnicode(const char *text,size_t *count)
-{
-  long
-    c;
-
-  register const char
-    *p;
-
-  register unsigned long
-    *q;
-
-  size_t
-    length;
-
-  unsigned long
-    *unicode;
-
-  *count=0;
-  if ((text == (char *) NULL) || (*text == '\0'))
-    return((unsigned long *) NULL);
-  length=strlen(text);
-  if (~length < MaxTextExtent)
-    return((unsigned long *) NULL);
-  length+=MaxTextExtent;
-  unicode=(unsigned long *) AcquireQuantumMemory(length,sizeof(*unicode));
-  if (unicode == (unsigned long *) NULL)
-    return((unsigned long *) NULL);
-  q=unicode;
-  for (p=text; *p != '\0'; p+=length)
-  {
-    length=strlen(p);
-    c=GetUnicodeCharacter((const unsigned char *) p,&length);
-    if (c < 0)
-      {
-        q=unicode;
-        for (p=text; *p != '\0'; p++)
-          *q++=(unsigned char) *p;
-        break;
-      }
-    *q=(unsigned long) c;
-    q++;
-  }
-  *count=q-unicode;
-  return(unicode);
-}
-#endif
-
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -748,20 +458,22 @@ static unsigned long *EncodeUnicode(const char *text,size_t *count)
 %
 %  The format of the FormatMagickCaption method is:
 %
-%      long FormatMagickCaption(Image *image,DrawInfo *draw_info,char *caption,
-%        TypeMetric *metrics)
+%      long FormatMagickCaption(Image *image,DrawInfo *draw_info,
+%        TypeMetric *metrics,char **caption)
 %
 %  A description of each parameter follows.
 %
 %    o image:  The image.
 %
+%    o caption: the caption.
+%
 %    o draw_info: the draw info.
 %
 %    o metrics: Return the font metrics in this structure.
 %
 */
 MagickExport long FormatMagickCaption(Image *image,DrawInfo *draw_info,
-  char *caption,TypeMetric *metrics)
+  TypeMetric *metrics,char **caption)
 {
   MagickBooleanType
     status;
@@ -779,36 +491,58 @@ MagickExport long FormatMagickCaption(Image *image,DrawInfo *draw_info,
 
   q=draw_info->text;
   s=(char *) NULL;
-  for (p=caption; *p != '\0'; p++)
+  for (p=(*caption); GetUTFCode(p) != 0; p+=GetUTFOctets(p))
   {
-    if (isspace((int) ((unsigned char) *p)) != 0)
+    if (IsUTFSpace(GetUTFCode(p)) != MagickFalse)
       s=p;
-    *q++=(*p);
+    for (i=0; i < (long) GetUTFOctets(p); i++)
+      *q++=(*(p+i));
     *q='\0';
     status=GetTypeMetrics(image,draw_info,metrics);
     if (status == MagickFalse)
       break;
     width=(unsigned long) (metrics->width+0.5);
-    if (*p != '\n')
+    if (GetUTFCode(p) != '\n')
       if (width <= image->columns)
         continue;
     if (s == (char *) NULL)
       {
         s=p;
-        while ((isspace((int) ((unsigned char) *s)) == 0) && (*s != '\0'))
-          s++;
+        while ((IsUTFSpace(GetUTFCode(s)) == MagickFalse) &&
+               (GetUTFCode(s) != 0))
+          s+=GetUTFOctets(s);
       }
-    if (*s != '\0')
+    if (GetUTFCode(s) != 0)
       {
         *s='\n';
         p=s;
-        s=(char *) NULL;
       }
+    else
+      {
+        char
+          *target;
+
+        long
+          n;
+
+        /*
+          No convenient line breaks-- insert newline.
+        */
+        target=AcquireString(*caption);
+        n=p-(*caption);
+        CopyMagickString(target,*caption,n+1);
+        ConcatenateMagickString(target,"\n",strlen(*caption)+1);
+        ConcatenateMagickString(target,p,strlen(*caption)+2);
+        (void) DestroyString(*caption);
+        *caption=target;
+        p=(*caption)+n;
+      }
+    s=(char *) NULL;
     q=draw_info->text;
   }
   i=0;
-  for (p=caption; *p != '\0'; p++)
-    if (*p == '\n')
+  for (p=(*caption); GetUTFCode(p) != 0; p+=GetUTFOctets(p))
+    if (GetUTFCode(p) == '\n')
       i++;
   return(i);
 }
@@ -1057,11 +791,19 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
   if (draw_info->font != (char *) NULL)
     {
       if (*draw_info->font == '@')
-        return(RenderFreetype(image,draw_info,draw_info->encoding,offset,metrics));
+        {
+          status=RenderFreetype(image,draw_info,draw_info->encoding,offset,
+            metrics);
+          return(status);
+        }
       if (*draw_info->font == '-')
         return(RenderX11(image,draw_info,offset,metrics));
       if (IsPathAccessible(draw_info->font) != MagickFalse)
-        return(RenderFreetype(image,draw_info,draw_info->encoding,offset,metrics));
+        {
+          status=RenderFreetype(image,draw_info,draw_info->encoding,offset,
+            metrics);
+          return(status);
+        }
       type_info=GetTypeInfo(draw_info->font,&image->exception);
       if (type_info == (const TypeInfo *) NULL)
         (void) ThrowMagickException(&image->exception,GetMagickModule(),
@@ -1087,8 +829,7 @@ static MagickBooleanType RenderType(Image *image,const DrawInfo *draw_info,
       draw_info->stretch,draw_info->weight,&image->exception);
   if (type_info == (const TypeInfo *) NULL)
     {
-      status=RenderFreetype(image,draw_info,draw_info->encoding,offset,
-        metrics);
+      status=RenderFreetype(image,draw_info,draw_info->encoding,offset,metrics);
       return(status);
     }
   annotate_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
@@ -1260,17 +1001,15 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     last_glyph;
 
   long
+    code,
     y;
 
   PointInfo
     point,
     resolution;
 
-  register long
-    i;
-
-  size_t
-    length;
+  register char
+    *p;
 
   static FT_Outline_Funcs
     OutlineMethods =
@@ -1282,9 +1021,6 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       0, 0
     };
 
-  unsigned long
-    *text;
-
   /*
     Initialize Truetype library.
   */
@@ -1294,14 +1030,15 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       image->filename);
   args.flags=FT_OPEN_PATHNAME;
   if (draw_info->font == (char *) NULL)
-    args.pathname=(char *) "helvetica";
+    args.pathname=ConstantString("helvetica");
   else
     if (*draw_info->font != '@')
-      args.pathname=draw_info->font;
+      args.pathname=ConstantString(draw_info->font);
     else
-      args.pathname=draw_info->font+1;
+      args.pathname=ConstantString(draw_info->font+1);
   face=(FT_Face) NULL;
   status=FT_Open_Face(library,&args,draw_info->face,&face);
+  args.pathname=DestroyString(args.pathname);
   if (status != 0)
     {
       (void) FT_Done_FreeType(library);
@@ -1397,53 +1134,6 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       (void) FT_Done_FreeType(library);
       return(MagickTrue);
     }
-  /*
-    Convert text to 2-byte format as prescribed by the encoding.
-  */
-  switch (encoding_type)
-  {
-    case ft_encoding_sjis:
-    {
-      text=EncodeSJIS(draw_info->text,&length);
-      break;
-    }
-    case ft_encoding_unicode:
-    {
-      text=EncodeUnicode(draw_info->text,&length);
-      break;
-    }
-    default:
-    {
-      if (draw_info->encoding != (char *) NULL)
-        {
-          if (LocaleCompare(draw_info->encoding,"SJIS") == 0)
-            {
-              text=EncodeSJIS(draw_info->text,&length);
-              break;
-            }
-          if ((LocaleCompare(draw_info->encoding,"UTF-8") == 0) ||
-              (encoding_type != ft_encoding_none))
-            {
-              text=EncodeUnicode(draw_info->text,&length);
-              break;
-            }
-        }
-      if (encoding_type == ft_encoding_unicode)
-        {
-          text=EncodeUnicode(draw_info->text,&length);
-          break;
-        }
-      text=EncodeText(draw_info->text,&length);
-      break;
-    }
-  }
-  if (text == (unsigned long *) NULL)
-    {
-      (void) FT_Done_Face(face);
-      (void) FT_Done_FreeType(library);
-      ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-        image->filename);
-    }
   /*
     Compute bounding box.
   */
@@ -1486,9 +1176,10 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     }
   point.x=0.0;
   point.y=0.0;
-  for (i=0; i < (long) length; i++)
+  code=0;
+  for (p=draw_info->text; GetUTFCode(p) != 0; p+=GetUTFOctets(p))
   {
-    glyph.id=FT_Get_Char_Index(face,text[i]);
+    glyph.id=FT_Get_Char_Index(face,GetUTFCode(p));
     if (glyph.id == 0)
       glyph.id=FT_Get_Char_Index(face,'?');
     if ((glyph.id != 0) && (last_glyph.id != 0))
@@ -1518,13 +1209,13 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       &bounds);
     if (status != 0)
       continue;
-    if ((i == 0) || (bounds.xMin < metrics->bounds.x1))
+    if ((p == draw_info->text) || (bounds.xMin < metrics->bounds.x1))
       metrics->bounds.x1=bounds.xMin;
-    if ((i == 0) || (bounds.yMin < metrics->bounds.y1))
+    if ((p == draw_info->text) || (bounds.yMin < metrics->bounds.y1))
       metrics->bounds.y1=bounds.yMin;
-    if ((i == 0) || (bounds.xMax > metrics->bounds.x2))
+    if ((p == draw_info->text) || (bounds.xMax > metrics->bounds.x2))
       metrics->bounds.x2=bounds.xMax;
-    if ((i == 0) || (bounds.yMax > metrics->bounds.y2))
+    if ((p == draw_info->text) || (bounds.yMax > metrics->bounds.y2))
       metrics->bounds.y2=bounds.yMax;
     if (draw_info->render != MagickFalse)
       if ((draw_info->stroke.opacity != TransparentOpacity) ||
@@ -1555,7 +1246,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
         MagickBooleanType
           status;
 
-        ViewInfo
+        CacheView
           *image_view;
 
         /*
@@ -1587,7 +1278,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
             x;
 
           register PixelPacket
-            *q;
+            *__restrict q;
 
           register unsigned char
             *p;
@@ -1598,9 +1289,15 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
           y_offset=(long) (point.y+y+0.5);
           if ((y_offset < 0) || (y_offset >= (long) image->rows))
             continue;
-          q=GetCacheViewAuthenticPixels(image_view,x_offset,y_offset,
-            bitmap->bitmap.width,1,exception);
-          active=q != (PixelPacket *) NULL ? MagickTrue : MagickFalse;
+          q=(PixelPacket *) NULL;
+          if ((x_offset < 0) || (x_offset >= (long) image->columns))
+            active=MagickFalse;
+          else
+            {
+              q=GetCacheViewAuthenticPixels(image_view,x_offset,y_offset,
+                bitmap->bitmap.width,1,exception);
+              active=q != (PixelPacket *) NULL ? MagickTrue : MagickFalse;
+            }
           p=bitmap->bitmap.buffer+y*bitmap->bitmap.width;
           for (x=0; x < (long) bitmap->bitmap.width; x++)
           {
@@ -1612,9 +1309,9 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
                 q++;
                 continue;
               }
-            fill_opacity=(MagickRealType) (*p)/255.0;
+            fill_opacity=(MagickRealType) (*p)/(bitmap->bitmap.num_grays-1);
             if (draw_info->text_antialias == MagickFalse)
-              fill_opacity=fill_opacity > 0.5 ? 1.0 : 0.0;
+              fill_opacity=fill_opacity >= 0.5 ? 1.0 : 0.0;
             if (active == MagickFalse)
               q=GetCacheViewAuthenticPixels(image_view,x_offset,y_offset,1,1,
                 exception);
@@ -1645,8 +1342,9 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
       }
     if ((bitmap->left+bitmap->bitmap.width) > metrics->width)
       metrics->width=bitmap->left+bitmap->bitmap.width;
-    if ((draw_info->interword_spacing != 0.0) && (isspace(text[i]) != 0) &&
-        (i != 0) && (isspace(text[i-1]) == 0))
+    if ((draw_info->interword_spacing != 0.0) &&
+        (IsUTFSpace(GetUTFCode(p)) != MagickFalse) &&
+        (IsUTFSpace(code) == MagickFalse))
       origin.x+=64.0*draw_info->interword_spacing;
     else
       origin.x+=face->glyph->advance.x;
@@ -1655,6 +1353,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
     if (last_glyph.id != 0)
       FT_Done_Glyph(last_glyph.image);
     last_glyph=glyph;
+    code=GetUTFCode(p);
   }
   if (last_glyph.id != 0)
     FT_Done_Glyph(last_glyph.image);
@@ -1666,6 +1365,7 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
           /*
             Draw text stroke.
           */
+          annotate_info->linejoin=RoundJoin;
           annotate_info->affine.tx=offset->x;
           annotate_info->affine.ty=offset->y;
           (void) ConcatenateString(&annotate_info->primitive,"'");
@@ -1709,7 +1409,6 @@ static MagickBooleanType RenderFreetype(Image *image,const DrawInfo *draw_info,
   /*
     Relinquish resources.
   */
-  text=(unsigned long *) RelinquishMagickMemory(text);
   annotate_info=DestroyDrawInfo(annotate_info);
   (void) FT_Done_Face(face);
   (void) FT_Done_FreeType(library);
@@ -1827,11 +1526,7 @@ static MagickBooleanType RenderPostscript(Image *image,
     resolution;
 
   register long
-    i,
-    x;
-
-  register PixelPacket
-    *q;
+    i;
 
   /*
     Render label with a Postscript font.
@@ -1975,7 +1670,7 @@ static MagickBooleanType RenderPostscript(Image *image,
       PixelPacket
         fill_color;
 
-      ViewInfo
+      CacheView
         *annotate_view;
 
       /*
@@ -1990,6 +1685,12 @@ static MagickBooleanType RenderPostscript(Image *image,
       annotate_view=AcquireCacheView(annotate_image);
       for (y=0; y < (long) annotate_image->rows; y++)
       {
+        register long
+          x;
+
+        register PixelPacket
+          *__restrict q;
+
         q=GetCacheViewAuthenticPixels(annotate_view,0,y,annotate_image->columns,
           1,exception);
         if (q == (PixelPacket *) NULL)
@@ -2241,6 +1942,9 @@ static MagickBooleanType RenderX11(Image *image,const DrawInfo *draw_info,
 static MagickBooleanType RenderX11(Image *image,const DrawInfo *draw_info,
   const PointInfo *offset,TypeMetric *metrics)
 {
+  (void) draw_info;
+  (void) offset;
+  (void) metrics;
   (void) ThrowMagickException(&image->exception,GetMagickModule(),
     MissingDelegateError,"DelegateLibrarySupportNotBuiltIn","`%s' (X11)",
     image->filename);
