@@ -42,6 +42,7 @@
 #include "magick/studio.h"
 #include "magick/property.h"
 #include "magick/cache.h"
+#include "magick/cache-private.h"
 #include "magick/cache-view.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
@@ -178,7 +179,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
     *y_map,
     *z_map;
 
-  ViewInfo
+  CacheView
     *image_view;
 
   assert(image != (Image *) NULL);
@@ -198,11 +199,12 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
       break;
     default:
     {
-      (void) SetImageColorspace(image,image->colorspace);
+      (void) TransformImageColorspace(image,image->colorspace);
       break;
     }
   }
-  image->colorspace=colorspace;
+  if (SetImageColorspace(image,colorspace) == MagickFalse)
+    return(MagickFalse);
   status=MagickTrue;
   progress=0;
   exception=(&image->exception);
@@ -230,7 +232,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -283,13 +285,13 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           pixel;
 
         register IndexPacket
-          *indexes;
+          *__restrict indexes;
 
         register long
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -345,7 +347,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -401,7 +403,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -457,7 +459,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -516,7 +518,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -608,7 +610,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1016,7 +1018,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         register unsigned long
           blue,
@@ -1057,7 +1059,7 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
               proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical
+  #pragma omp critical (MagickCore_RGBTransformImage)
 #endif
             proceed=SetImageProgress(image,RGBTransformImageTag,progress++,
               image->rows);
@@ -1105,6 +1107,8 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
   z_map=(TransformPacket *) RelinquishMagickMemory(z_map);
   y_map=(TransformPacket *) RelinquishMagickMemory(y_map);
   x_map=(TransformPacket *) RelinquishMagickMemory(x_map);
+  if (SetImageColorspace(image,colorspace) == MagickFalse)
+    return(MagickFalse);
   return(status);
 }
 
@@ -1135,6 +1139,45 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
 */
 MagickExport MagickBooleanType SetImageColorspace(Image *image,
   const ColorspaceType colorspace)
+{
+  Cache
+    cache;
+
+  if (image->colorspace == colorspace)
+    return(MagickTrue);
+  image->colorspace=colorspace;
+  cache=GetImagePixelCache(image,MagickTrue,&image->exception);
+  image->colorspace=colorspace;  /* GRAY colorspace might get reset to RGB */
+  return(cache == (Cache) NULL ? MagickFalse: MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   T r a n s f o r m I m a g e C o l o r s p a c e                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  TransformImageColorspace() transforms an image colorspace.
+%
+%  The format of the TransformImageColorspace method is:
+%
+%      MagickBooleanType TransformImageColorspace(Image *image,
+%        const ColorspaceType colorspace)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o colorspace: the colorspace.
+%
+*/
+MagickExport MagickBooleanType TransformImageColorspace(Image *image,
+  const ColorspaceType colorspace)
 {
   MagickBooleanType
     status;
@@ -1145,7 +1188,8 @@ MagickExport MagickBooleanType SetImageColorspace(Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (colorspace == UndefinedColorspace)
     {
-      image->colorspace=UndefinedColorspace;
+      if (SetImageColorspace(image,colorspace) == MagickFalse)
+        return(MagickFalse);
       return(MagickTrue);
     }
   if (image->colorspace == colorspace)
@@ -1322,7 +1366,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
     *x_map,
     *z_map;
 
-  ViewInfo
+  CacheView
     *image_view;
 
   assert(image != (Image *) NULL);
@@ -1371,7 +1415,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1394,7 +1438,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           status=MagickFalse;
       }
       image_view=DestroyCacheView(image_view);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     case CMYKColorspace:
@@ -1426,13 +1471,13 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           pixel;
 
         register IndexPacket
-          *indexes;
+          *__restrict indexes;
 
         register long
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1457,7 +1502,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           status=MagickFalse;
       }
       image_view=DestroyCacheView(image_view);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     case HSBColorspace:
@@ -1490,7 +1536,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1514,7 +1560,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           status=MagickFalse;
       }
       image_view=DestroyCacheView(image_view);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     case HSLColorspace:
@@ -1547,7 +1594,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1571,7 +1618,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           status=MagickFalse;
       }
       image_view=DestroyCacheView(image_view);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     case HWBColorspace:
@@ -1604,7 +1652,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1628,7 +1676,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           status=MagickFalse;
       }
       image_view=DestroyCacheView(image_view);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     case LabColorspace:
@@ -1664,7 +1713,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1692,7 +1741,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           status=MagickFalse;
       }
       image_view=DestroyCacheView(image_view);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     case LogColorspace:
@@ -1756,7 +1806,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1780,7 +1830,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       }
       image_view=DestroyCacheView(image_view);
       logmap=(Quantum *) RelinquishMagickMemory(logmap);
-      image->colorspace=RGBColorspace;
+      if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+        return(MagickFalse);
       return(status);
     }
     default:
@@ -2125,7 +2176,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -2202,7 +2253,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
               proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical
+  #pragma omp critical (MagickCore_TransformRGBImage)
 #endif
             proceed=SetImageProgress(image,TransformRGBImageTag,progress++,
               image->rows);
@@ -2293,6 +2344,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
   z_map=(TransformPacket *) RelinquishMagickMemory(z_map);
   y_map=(TransformPacket *) RelinquishMagickMemory(y_map);
   x_map=(TransformPacket *) RelinquishMagickMemory(x_map);
-  image->colorspace=RGBColorspace;
+  if (SetImageColorspace(image,RGBColorspace) == MagickFalse)
+    return(MagickFalse);
   return(MagickTrue);
 }
