@@ -17,7 +17,7 @@
 %                               December 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -57,6 +57,8 @@
 #include "magick/list.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
+#include "magick/monitor.h"
+#include "magick/monitor-private.h"
 #include "magick/option.h"
 #include "magick/pixel-private.h"
 #include "magick/resource_.h"
@@ -133,10 +135,10 @@ MagickExport Image *CompareImageChannels(Image *image,
     lowlight,
     zero;
 
-  ViewInfo
-    **highlight_view,
-    **image_view,
-    **reconstruct_view;
+  CacheView
+    *highlight_view,
+    *image_view,
+    *reconstruct_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -192,11 +194,11 @@ MagickExport Image *CompareImageChannels(Image *image,
   */
   status=MagickTrue;
   GetMagickPixelPacket(image,&zero);
-  image_view=AcquireCacheViewThreadSet(image);
-  reconstruct_view=AcquireCacheViewThreadSet(reconstruct_image);
-  highlight_view=AcquireCacheViewThreadSet(highlight_image);
+  image_view=AcquireCacheView(image);
+  reconstruct_view=AcquireCacheView(reconstruct_image);
+  highlight_view=AcquireCacheView(highlight_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(status)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
@@ -208,41 +210,38 @@ MagickExport Image *CompareImageChannels(Image *image,
       reconstruct_pixel;
 
     register const IndexPacket
-      *indexes,
-      *reconstruct_indexes;
+      *__restrict indexes,
+      *__restrict reconstruct_indexes;
 
     register const PixelPacket
-      *p,
-      *q;
+      *__restrict p,
+      *__restrict q;
 
     register IndexPacket
-      *highlight_indexes;
+      *__restrict highlight_indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
-      *r;
+      *__restrict r;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],0,y,image->columns,1,
-      exception);
-    q=GetCacheViewVirtualPixels(reconstruct_view[id],0,y,
-      reconstruct_image->columns,1,exception);
-    r=QueueCacheViewAuthenticPixels(highlight_view[id],0,y,
-      highlight_image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,reconstruct_image->columns,
+      1,exception);
+    r=QueueCacheViewAuthenticPixels(highlight_view,0,y,highlight_image->columns,
+      1,exception);
     if ((p == (const PixelPacket *) NULL) ||
         (q == (const PixelPacket *) NULL) || (r == (PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexes(image_view[id]);
-    reconstruct_indexes=GetCacheViewVirtualIndexes(reconstruct_view[id]);
-    highlight_indexes=GetCacheViewAuthenticIndexes(highlight_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
+    highlight_indexes=GetCacheViewAuthenticIndexQueue(highlight_view);
     pixel=zero;
     reconstruct_pixel=zero;
     for (x=0; x < (long) image->columns; x++)
@@ -284,13 +283,13 @@ MagickExport Image *CompareImageChannels(Image *image,
       q++;
       r++;
     }
-    sync=SyncCacheViewAuthenticPixels(highlight_view[id],exception);
+    sync=SyncCacheViewAuthenticPixels(highlight_view,exception);
     if (sync == MagickFalse)
       status=MagickFalse;
   }
-  highlight_view=DestroyCacheViewThreadSet(highlight_view);
-  reconstruct_view=DestroyCacheViewThreadSet(reconstruct_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  highlight_view=DestroyCacheView(highlight_view);
+  reconstruct_view=DestroyCacheView(reconstruct_view);
+  image_view=DestroyCacheView(image_view);
   (void) CompositeImage(difference_image,image->compose,highlight_image,0,0);
   highlight_image=DestroyImage(highlight_image);
   if (status == MagickFalse)
@@ -359,19 +358,19 @@ static MagickBooleanType GetAbsoluteError(const Image *image,
   MagickPixelPacket
     zero;
 
-  ViewInfo
-    **image_view,
-    **reconstruct_view;
+  CacheView
+    *image_view,
+    *reconstruct_view;
 
   /*
     Compute the absolute difference in pixels between two images.
   */
   status=MagickTrue;
   GetMagickPixelPacket(image,&zero);
-  image_view=AcquireCacheViewThreadSet(image);
-  reconstruct_view=AcquireCacheViewThreadSet(reconstruct_image);
+  image_view=AcquireCacheView(image);
+  reconstruct_view=AcquireCacheView(reconstruct_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(status)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
@@ -383,31 +382,29 @@ static MagickBooleanType GetAbsoluteError(const Image *image,
       reconstruct_pixel;
 
     register const IndexPacket
-      *indexes,
-      *reconstruct_indexes;
+      *__restrict indexes,
+      *__restrict reconstruct_indexes;
 
     register const PixelPacket
-      *p,
-      *q;
+      *__restrict p,
+      *__restrict q;
 
     register long
       i,
-      id,
       x;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],0,y,image->columns,1,exception);
-    q=GetCacheViewVirtualPixels(reconstruct_view[id],0,y,
-      reconstruct_image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,reconstruct_image->columns,
+      1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexes(image_view[id]);
-    reconstruct_indexes=GetCacheViewVirtualIndexes(reconstruct_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
     pixel=zero;
     reconstruct_pixel=pixel;
     (void) ResetMagickMemory(channel_distortion,0,sizeof(channel_distortion));
@@ -436,13 +433,13 @@ static MagickBooleanType GetAbsoluteError(const Image *image,
       q++;
     }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-    #pragma omp critical
+  #pragma omp critical (MagickCore_GetAbsoluteError)
 #endif
     for (i=0; i <= (long) AllChannels; i++)
       distortion[i]+=channel_distortion[i];
   }
-  reconstruct_view=DestroyCacheViewThreadSet(reconstruct_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  reconstruct_view=DestroyCacheView(reconstruct_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -472,25 +469,24 @@ static MagickBooleanType GetMeanAbsoluteError(const Image *image,
   const Image *reconstruct_image,const ChannelType channel,
   double *distortion,ExceptionInfo *exception)
 {
-  double
-    scale;
-
   long
     y;
 
   MagickBooleanType
     status;
 
-  ViewInfo
-    **image_view,
-    **reconstruct_view;
+  register long
+    i;
+
+  CacheView
+    *image_view,
+    *reconstruct_view;
 
   status=MagickTrue;
-  scale=1.0/((double) image->columns*image->rows);
-  image_view=AcquireCacheViewThreadSet(image);
-  reconstruct_view=AcquireCacheViewThreadSet(reconstruct_image);
+  image_view=AcquireCacheView(image);
+  reconstruct_view=AcquireCacheView(reconstruct_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(status)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
@@ -498,31 +494,29 @@ static MagickBooleanType GetMeanAbsoluteError(const Image *image,
       channel_distortion[AllChannels+1];
 
     register const IndexPacket
-      *indexes,
-      *reconstruct_indexes;
+      *__restrict indexes,
+      *__restrict reconstruct_indexes;
 
     register const PixelPacket
-      *p,
-      *q;
+      *__restrict p,
+      *__restrict q;
 
     register long
       i,
-      id,
       x;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],0,y,image->columns,1,exception);
-    q=GetCacheViewVirtualPixels(reconstruct_view[id],0,y,
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,
       reconstruct_image->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexes(image_view[id]);
-    reconstruct_indexes=GetCacheViewVirtualIndexes(reconstruct_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
     (void) ResetMagickMemory(channel_distortion,0,sizeof(channel_distortion));
     for (x=0; x < (long) image->columns; x++)
     {
@@ -531,47 +525,50 @@ static MagickBooleanType GetMeanAbsoluteError(const Image *image,
 
       if ((channel & RedChannel) != 0)
         {
-          distance=fabs(p->red-(double) q->red);
-          channel_distortion[RedChannel]+=scale*distance;
-          channel_distortion[AllChannels]+=scale*distance;
+          distance=QuantumScale*fabs(p->red-(double) q->red);
+          channel_distortion[RedChannel]+=distance;
+          channel_distortion[AllChannels]+=distance;
         }
       if ((channel & GreenChannel) != 0)
         {
-          distance=fabs(p->green-(double) q->green);
-          channel_distortion[GreenChannel]+=scale*distance;
-          channel_distortion[AllChannels]+=scale*distance;
+          distance=QuantumScale*fabs(p->green-(double) q->green);
+          channel_distortion[GreenChannel]+=distance;
+          channel_distortion[AllChannels]+=distance;
         }
       if ((channel & BlueChannel) != 0)
         {
-          distance=fabs(p->blue-(double) q->blue);
-          channel_distortion[BlueChannel]+=scale*distance;
-          channel_distortion[AllChannels]+=scale*distance;
+          distance=QuantumScale*fabs(p->blue-(double) q->blue);
+          channel_distortion[BlueChannel]+=distance;
+          channel_distortion[AllChannels]+=distance;
         }
       if (((channel & OpacityChannel) != 0) &&
           (image->matte != MagickFalse))
         {
-          distance=fabs(p->opacity-(double) q->opacity);
-          channel_distortion[OpacityChannel]+=scale*distance;
-          channel_distortion[AllChannels]+=scale*distance;
+          distance=QuantumScale*fabs(p->opacity-(double) q->opacity);
+          channel_distortion[OpacityChannel]+=distance;
+          channel_distortion[AllChannels]+=distance;
         }
       if (((channel & IndexChannel) != 0) &&
           (image->colorspace == CMYKColorspace))
         {
-          distance=fabs(indexes[x]-(double) reconstruct_indexes[x]);
-          channel_distortion[BlackChannel]+=scale*distance;
-          channel_distortion[AllChannels]+=scale*distance;
+          distance=QuantumScale*fabs(indexes[x]-(double)
+            reconstruct_indexes[x]);
+          channel_distortion[BlackChannel]+=distance;
+          channel_distortion[AllChannels]+=distance;
         }
       p++;
       q++;
     }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-    #pragma omp critical
+  #pragma omp critical (MagickCore_GetMeanAbsoluteError)
 #endif
     for (i=0; i <= (long) AllChannels; i++)
       distortion[i]+=channel_distortion[i];
   }
-  reconstruct_view=DestroyCacheViewThreadSet(reconstruct_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  reconstruct_view=DestroyCacheView(reconstruct_view);
+  image_view=DestroyCacheView(image_view);
+  for (i=0; i <= (long) AllChannels; i++)
+    distortion[i]/=((double) image->columns*image->rows);
   distortion[AllChannels]/=(double) GetNumberChannels(image,channel);
   return(status);
 }
@@ -593,7 +590,7 @@ static MagickBooleanType GetMeanErrorPerPixel(Image *image,
     maximum_error,
     mean_error;
 
-  ViewInfo
+  CacheView
     *image_view,
     *reconstruct_view;
 
@@ -608,26 +605,26 @@ static MagickBooleanType GetMeanErrorPerPixel(Image *image,
   for (y=0; y < (long) image->rows; y++)
   {
     register const IndexPacket
-      *indexes,
-      *reconstruct_indexes;
+      *__restrict indexes,
+      *__restrict reconstruct_indexes;
 
     register const PixelPacket
-      *p,
-      *q;
+      *__restrict p,
+      *__restrict q;
 
     register long
       x;
 
     p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
-    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,reconstruct_image->columns,1,
-      exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,reconstruct_image->columns,
+      1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
       {
         status=MagickFalse;
         break;
       }
-    indexes=GetCacheViewVirtualIndexes(image_view);
-    reconstruct_indexes=GetCacheViewVirtualIndexes(reconstruct_view);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
     for (x=0; x < (long) image->columns; x++)
     {
       MagickRealType
@@ -709,25 +706,24 @@ static MagickBooleanType GetMeanSquaredError(const Image *image,
   const Image *reconstruct_image,const ChannelType channel,
   double *distortion,ExceptionInfo *exception)
 {
-  double
-    scale;
-
   long
     y;
 
   MagickBooleanType
     status;
 
-  ViewInfo
-    **image_view,
-    **reconstruct_view;
+  register long
+    i;
+
+  CacheView
+    *image_view,
+    *reconstruct_view;
 
   status=MagickTrue;
-  scale=QuantumScale/((double) image->columns*image->rows);
-  image_view=AcquireCacheViewThreadSet(image);
-  reconstruct_view=AcquireCacheViewThreadSet(reconstruct_image);
+  image_view=AcquireCacheView(image);
+  reconstruct_view=AcquireCacheView(reconstruct_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(status)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
@@ -735,31 +731,29 @@ static MagickBooleanType GetMeanSquaredError(const Image *image,
       channel_distortion[AllChannels+1];
 
     register const IndexPacket
-      *indexes,
-      *reconstruct_indexes;
+      *__restrict indexes,
+      *__restrict reconstruct_indexes;
 
     register const PixelPacket
-      *p,
-      *q;
+      *__restrict p,
+      *__restrict q;
 
     register long
       i,
-      id,
       x;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],0,y,image->columns,1,exception);
-    q=GetCacheViewVirtualPixels(reconstruct_view[id],0,y,
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,
       reconstruct_image->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexes(image_view[id]);
-    reconstruct_indexes=GetCacheViewVirtualIndexes(reconstruct_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
     (void) ResetMagickMemory(channel_distortion,0,sizeof(channel_distortion));
     for (x=0; x < (long) image->columns; x++)
     {
@@ -768,48 +762,51 @@ static MagickBooleanType GetMeanSquaredError(const Image *image,
 
       if ((channel & RedChannel) != 0)
         {
-          distance=(p->red-(MagickRealType) q->red);
-          channel_distortion[RedChannel]+=scale*distance*distance;
-          channel_distortion[AllChannels]+=scale*distance*distance;
+          distance=QuantumScale*(p->red-(MagickRealType) q->red);
+          channel_distortion[RedChannel]+=distance*distance;
+          channel_distortion[AllChannels]+=distance*distance;
         }
       if ((channel & GreenChannel) != 0)
         {
-          distance=(p->green-(MagickRealType) q->green);
-          channel_distortion[GreenChannel]+=scale*distance*distance;
-          channel_distortion[AllChannels]+=scale*distance*distance;
+          distance=QuantumScale*(p->green-(MagickRealType) q->green);
+          channel_distortion[GreenChannel]+=distance*distance;
+          channel_distortion[AllChannels]+=distance*distance;
         }
       if ((channel & BlueChannel) != 0)
         {
-          distance=(p->blue-(MagickRealType) q->blue);
-          channel_distortion[BlueChannel]+=scale*distance*distance;
-          channel_distortion[AllChannels]+=scale*distance*distance;
+          distance=QuantumScale*(p->blue-(MagickRealType) q->blue);
+          channel_distortion[BlueChannel]+=distance*distance;
+          channel_distortion[AllChannels]+=distance*distance;
         }
       if (((channel & OpacityChannel) != 0) &&
           (image->matte != MagickFalse))
         {
-          distance=(p->opacity-(MagickRealType) q->opacity);
-          channel_distortion[OpacityChannel]+=scale*distance*distance;
-          channel_distortion[AllChannels]+=scale*distance*distance;
+          distance=QuantumScale*(p->opacity-(MagickRealType) q->opacity);
+          channel_distortion[OpacityChannel]+=distance*distance;
+          channel_distortion[AllChannels]+=distance*distance;
         }
       if (((channel & IndexChannel) != 0) &&
           (image->colorspace == CMYKColorspace) &&
           (reconstruct_image->colorspace == CMYKColorspace))
         {
-          distance=(indexes[x]-(MagickRealType) reconstruct_indexes[x]);
-          channel_distortion[BlackChannel]+=scale*distance*distance;
-          channel_distortion[AllChannels]+=scale*distance*distance;
+          distance=QuantumScale*(indexes[x]-(MagickRealType)
+            reconstruct_indexes[x]);
+          channel_distortion[BlackChannel]+=distance*distance;
+          channel_distortion[AllChannels]+=distance*distance;
         }
       p++;
       q++;
     }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-    #pragma omp critical
+  #pragma omp critical (MagickCore_GetMeanSquaredError)
 #endif
     for (i=0; i <= (long) AllChannels; i++)
       distortion[i]+=channel_distortion[i];
   }
-  reconstruct_view=DestroyCacheViewThreadSet(reconstruct_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  reconstruct_view=DestroyCacheView(reconstruct_view);
+  image_view=DestroyCacheView(image_view);
+  for (i=0; i <= (long) AllChannels; i++)
+    distortion[i]/=((double) image->columns*image->rows);
   distortion[AllChannels]/=(double) GetNumberChannels(image,channel);
   return(status);
 }
@@ -824,15 +821,15 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
   MagickBooleanType
     status;
 
-  ViewInfo
-    **image_view,
-    **reconstruct_view;
+  CacheView
+    *image_view,
+    *reconstruct_view;
 
   status=MagickTrue;
-  image_view=AcquireCacheViewThreadSet(image);
-  reconstruct_view=AcquireCacheViewThreadSet(reconstruct_image);
+  image_view=AcquireCacheView(image);
+  reconstruct_view=AcquireCacheView(reconstruct_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(status)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
@@ -840,31 +837,29 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
       channel_distortion[AllChannels+1];
 
     register const IndexPacket
-      *indexes,
-      *reconstruct_indexes;
+      *__restrict indexes,
+      *__restrict reconstruct_indexes;
 
     register const PixelPacket
-      *p,
-      *q;
+      *__restrict p,
+      *__restrict q;
 
     register long
       i,
-      id,
       x;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],0,y,image->columns,1,exception);
-    q=GetCacheViewVirtualPixels(reconstruct_view[id],0,y,
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,
       reconstruct_image->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexes(image_view[id]);
-    reconstruct_indexes=GetCacheViewVirtualIndexes(reconstruct_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
     (void) ResetMagickMemory(channel_distortion,0,sizeof(channel_distortion));
     for (x=0; x < (long) image->columns; x++)
     {
@@ -873,7 +868,7 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
 
       if ((channel & RedChannel) != 0)
         {
-          distance=fabs(p->red-(double) q->red);
+          distance=QuantumScale*fabs(p->red-(double) q->red);
           if (distance > channel_distortion[RedChannel])
             channel_distortion[RedChannel]=distance;
           if (distance > channel_distortion[AllChannels])
@@ -881,7 +876,7 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
         }
       if ((channel & GreenChannel) != 0)
         {
-          distance=fabs(p->green-(double) q->green);
+          distance=QuantumScale*fabs(p->green-(double) q->green);
           if (distance > channel_distortion[GreenChannel])
             channel_distortion[GreenChannel]=distance;
           if (distance > channel_distortion[AllChannels])
@@ -889,7 +884,7 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
         }
       if ((channel & BlueChannel) != 0)
         {
-          distance=fabs(p->blue-(double) q->blue);
+          distance=QuantumScale*fabs(p->blue-(double) q->blue);
           if (distance > channel_distortion[BlueChannel])
             channel_distortion[BlueChannel]=distance;
           if (distance > channel_distortion[AllChannels])
@@ -898,7 +893,7 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
       if (((channel & OpacityChannel) != 0) &&
           (image->matte != MagickFalse))
         {
-          distance=fabs(p->opacity-(double) q->opacity);
+          distance=QuantumScale*fabs(p->opacity-(double) q->opacity);
           if (distance > channel_distortion[OpacityChannel])
             channel_distortion[OpacityChannel]=distance;
           if (distance > channel_distortion[AllChannels])
@@ -908,7 +903,8 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
           (image->colorspace == CMYKColorspace) &&
           (reconstruct_image->colorspace == CMYKColorspace))
         {
-          distance=fabs(indexes[x]-(double) reconstruct_indexes[x]);
+          distance=QuantumScale*fabs(indexes[x]-(double)
+            reconstruct_indexes[x]);
           if (distance > channel_distortion[BlackChannel])
             channel_distortion[BlackChannel]=distance;
           if (distance > channel_distortion[AllChannels])
@@ -918,14 +914,14 @@ static MagickBooleanType GetPeakAbsoluteError(const Image *image,
       q++;
     }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-    #pragma omp critical
+  #pragma omp critical (MagickCore_GetPeakAbsoluteError)
 #endif
     for (i=0; i <= (long) AllChannels; i++)
       if (channel_distortion[i] > distortion[i])
         distortion[i]=channel_distortion[i];
   }
-  reconstruct_view=DestroyCacheViewThreadSet(reconstruct_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  reconstruct_view=DestroyCacheView(reconstruct_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -939,23 +935,23 @@ static MagickBooleanType GetPeakSignalToNoiseRatio(const Image *image,
   status=GetMeanSquaredError(image,reconstruct_image,channel,distortion,
     exception);
   if ((channel & RedChannel) != 0)
-    distortion[RedChannel]=20.0*log10((double) 1.0/sqrt(QuantumScale*
+    distortion[RedChannel]=20.0*log10((double) 1.0/sqrt(
       distortion[RedChannel]));
   if ((channel & GreenChannel) != 0)
-    distortion[GreenChannel]=20.0*log10((double) 1.0/sqrt(QuantumScale*
+    distortion[GreenChannel]=20.0*log10((double) 1.0/sqrt(
       distortion[GreenChannel]));
   if ((channel & BlueChannel) != 0)
-    distortion[BlueChannel]=20.0*log10((double) 1.0/sqrt(QuantumScale*
+    distortion[BlueChannel]=20.0*log10((double) 1.0/sqrt(
       distortion[BlueChannel]));
   if (((channel & OpacityChannel) != 0) &&
       (image->matte != MagickFalse))
-    distortion[OpacityChannel]=20.0*log10((double) 1.0/sqrt(QuantumScale*
+    distortion[OpacityChannel]=20.0*log10((double) 1.0/sqrt(
       distortion[OpacityChannel]));
   if (((channel & IndexChannel) != 0) &&
       (image->colorspace == CMYKColorspace))
-    distortion[BlackChannel]=20.0*log10((double) 1.0/sqrt(QuantumScale*
+    distortion[BlackChannel]=20.0*log10((double) 1.0/sqrt(
       distortion[BlackChannel]));
-  distortion[AllChannels]=20.0*log10((double) 1.0/sqrt(QuantumScale*
+  distortion[AllChannels]=20.0*log10((double) 1.0/sqrt(
     distortion[AllChannels]));
   return(status);
 }
@@ -970,18 +966,18 @@ static MagickBooleanType GetRootMeanSquaredError(const Image *image,
   status=GetMeanSquaredError(image,reconstruct_image,channel,distortion,
     exception);
   if ((channel & RedChannel) != 0)
-    distortion[RedChannel]=sqrt(QuantumRange*distortion[RedChannel]);
+    distortion[RedChannel]=sqrt(distortion[RedChannel]);
   if ((channel & GreenChannel) != 0)
-    distortion[GreenChannel]=sqrt(QuantumRange*distortion[GreenChannel]);
+    distortion[GreenChannel]=sqrt(distortion[GreenChannel]);
   if ((channel & BlueChannel) != 0)
-    distortion[BlueChannel]=sqrt(QuantumRange*distortion[BlueChannel]);
+    distortion[BlueChannel]=sqrt(distortion[BlueChannel]);
   if (((channel & OpacityChannel) != 0) &&
       (image->matte != MagickFalse))
-    distortion[OpacityChannel]=sqrt(QuantumRange*distortion[OpacityChannel]);
+    distortion[OpacityChannel]=sqrt(distortion[OpacityChannel]);
   if (((channel & IndexChannel) != 0) &&
       (image->colorspace == CMYKColorspace))
-    distortion[BlackChannel]=sqrt(QuantumRange*distortion[BlackChannel]);
-  distortion[AllChannels]=sqrt(QuantumRange*distortion[AllChannels]);
+    distortion[BlackChannel]=sqrt(distortion[BlackChannel]);
+  distortion[AllChannels]=sqrt(distortion[AllChannels]);
   return(status);
 }
 
@@ -1241,6 +1237,9 @@ MagickExport double *GetImageChannelDistortions(Image *image,
 MagickExport MagickBooleanType IsImagesEqual(Image *image,
   const Image *reconstruct_image)
 {
+  ExceptionInfo
+    *exception;
+
   long
     y;
 
@@ -1253,7 +1252,7 @@ MagickExport MagickBooleanType IsImagesEqual(Image *image,
     mean_error,
     mean_error_per_pixel;
 
-  ViewInfo
+  CacheView
     *image_view,
     *reconstruct_view;
 
@@ -1268,28 +1267,29 @@ MagickExport MagickBooleanType IsImagesEqual(Image *image,
   maximum_error=0.0;
   mean_error_per_pixel=0.0;
   mean_error=0.0;
+  exception=(&image->exception);
   image_view=AcquireCacheView(image);
   reconstruct_view=AcquireCacheView(reconstruct_image);
   for (y=0; y < (long) image->rows; y++)
   {
     register const IndexPacket
-      *indexes,
-      *reconstruct_indexes;
+      *__restrict indexes,
+      *__restrict reconstruct_indexes;
 
     register const PixelPacket
-      *p,
-      *q;
+      *__restrict p,
+      *__restrict q;
 
     register long
       x;
 
-    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,&image->exception);
-    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,reconstruct_image->columns,1,
-      &image->exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reconstruct_view,0,y,reconstruct_image->columns,
+      1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
       break;
-    indexes=GetCacheViewVirtualIndexes(image_view);
-    reconstruct_indexes=GetCacheViewVirtualIndexes(reconstruct_view);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reconstruct_indexes=GetCacheViewVirtualIndexQueue(reconstruct_view);
     for (x=0; x < (long) image->columns; x++)
     {
       MagickRealType
@@ -1345,3 +1345,218 @@ MagickExport MagickBooleanType IsImagesEqual(Image *image,
   status=image->error.mean_error_per_pixel == 0.0 ? MagickTrue : MagickFalse;
   return(status);
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   S i m i l a r i t y I m a g e                                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  SimilarityImage() compares the reference image of the image and returns the
+%  best match offset.  In addition, it returns a similarity image such that an
+%  exact match location is completely white and if none of the pixels match,
+%  black, otherwise some gray level in-between.
+%
+%  The format of the SimilarityImageImage method is:
+%
+%      Image *SimilarityImage(const Image *image,const Image *reference,
+%        RectangleInfo *offset,double *similarity,ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o reference: find an area of the image that closely resembles this image.
+%
+%    o the best match offset of the reference image within the image.
+%
+%    o similarity: the computed similarity between the images.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+
+static double GetSimilarityMetric(const Image *image,const Image *reference,
+  const long x_offset,const long y_offset,ExceptionInfo *exception)
+{
+  double
+    similarity;
+
+  long
+    y;
+
+  CacheView
+    *image_view,
+    *reference_view;
+
+  /*
+    Compute the similarity in pixels between two images.
+  */
+  similarity=0.0;
+  image_view=AcquireCacheView(image);
+  reference_view=AcquireCacheView(reference);
+  for (y=0; y < (long) reference->rows; y++)
+  {
+    register const IndexPacket
+      *__restrict indexes,
+      *__restrict reference_indexes;
+
+    register const PixelPacket
+      *__restrict p,
+      *__restrict q;
+
+    register long
+      x;
+
+    p=GetCacheViewVirtualPixels(image_view,x_offset,y_offset+y,
+      reference->columns,1,exception);
+    q=GetCacheViewVirtualPixels(reference_view,0,y,reference->columns,1,
+      exception);
+    if ((p == (const PixelPacket *) NULL) || (q == (const PixelPacket *) NULL))
+      continue;
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    reference_indexes=GetCacheViewVirtualIndexQueue(reference_view);
+    for (x=0; x < (long) reference->columns; x++)
+    {
+      MagickRealType
+        distance;
+
+      distance=QuantumScale*(p->red-(MagickRealType) q->red);
+      similarity+=distance*distance;
+      distance=QuantumScale*(p->green-(MagickRealType) q->green);
+      similarity+=distance*distance;
+      distance=QuantumScale*(p->blue-(MagickRealType) q->blue);
+      similarity+=distance*distance;
+      if ((image->matte != MagickFalse) && (reference->matte != MagickFalse))
+        {
+          distance=QuantumScale*(p->opacity-(MagickRealType) q->opacity);
+          similarity+=distance*distance;
+        }
+      if ((image->colorspace == CMYKColorspace) &&
+          (reference->colorspace == CMYKColorspace))
+        {
+          distance=QuantumScale*(indexes[x]-(MagickRealType)
+            reference_indexes[x]);
+          similarity+=distance*distance;
+        }
+      p++;
+      q++;
+    }
+  }
+  reference_view=DestroyCacheView(reference_view);
+  image_view=DestroyCacheView(image_view);
+  similarity/=((double) reference->columns*reference->rows);
+  similarity/=(double) GetNumberChannels(reference,AllChannels);
+  return(sqrt(similarity));
+}
+
+MagickExport Image *SimilarityImage(Image *image,const Image *reference,
+  RectangleInfo *offset,double *similarity_metric,ExceptionInfo *exception)
+{
+#define SimilarityImageTag  "Similarity/Image"
+
+  long
+    progress,
+    y;
+
+  Image
+    *similarity_image;
+
+  MagickBooleanType
+    status;
+
+  CacheView
+    *similarity_view;
+
+  assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  assert(offset != (RectangleInfo *) NULL);
+  SetGeometry(reference,offset);
+  *similarity_metric=1.0;
+  if ((reference->columns > image->columns) ||
+      (reference->rows > image->rows))
+    return((Image *) NULL);
+  similarity_image=CloneImage(image,image->columns-reference->columns+1,
+    image->rows-reference->rows+1,MagickTrue,exception);
+  if (similarity_image == (Image *) NULL)
+    return((Image *) NULL);
+  if (SetImageStorageClass(similarity_image,DirectClass) == MagickFalse)
+    {
+      InheritException(exception,&similarity_image->exception);
+      similarity_image=DestroyImage(similarity_image);
+      return((Image *) NULL);
+    }
+  /*
+    Measure similarity of reference image against image.
+  */
+  status=MagickTrue;
+  progress=0;
+  similarity_view=AcquireCacheView(similarity_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
+  for (y=0; y < (long) (image->rows-reference->rows+1); y++)
+  {
+    double
+      similarity;
+
+    register long
+      x;
+
+    register PixelPacket
+      *__restrict q;
+
+    if (status == MagickFalse)
+      continue;
+    q=GetCacheViewAuthenticPixels(similarity_view,0,y,
+      similarity_image->columns,1,exception);
+    if (q == (const PixelPacket *) NULL)
+      {
+        status=MagickFalse;
+        continue;
+      }
+    for (x=0; x < (long) (image->columns-reference->columns+1); x++)
+    {
+      similarity=GetSimilarityMetric(image,reference,x,y,exception);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical (MagickCore_SimilarityImage)
+#endif
+      if (similarity < *similarity_metric)
+        {
+          *similarity_metric=similarity;
+          offset->x=x;
+          offset->y=y;
+        }
+      q->red=RoundToQuantum(QuantumRange-QuantumRange*similarity);
+      q->green=q->red;
+      q->blue=q->red;
+      q++;
+    }
+    if (SyncCacheViewAuthenticPixels(similarity_view,exception) == MagickFalse)
+      status=MagickFalse;
+    if (image->progress_monitor != (MagickProgressMonitor) NULL)
+      {
+        MagickBooleanType
+          proceed;
+
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical (MagickCore_SimilarityImage)
+#endif
+        proceed=SetImageProgress(image,SimilarityImageTag,progress++,
+          image->rows);
+        if (proceed == MagickFalse)
+          status=MagickFalse;
+      }
+  }
+  similarity_view=DestroyCacheView(similarity_view);
+  return(similarity_image);
+}
