@@ -17,7 +17,7 @@
 %                                   July 1992                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -181,22 +181,16 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     shadow,
     trough;
 
-  register IndexPacket
-    *frame_indexes;
-
   register long
     x;
 
-  register PixelPacket
-    *q;
-
   unsigned long
     bevel_width,
     height,
     width;
 
-  ViewInfo
-    **frame_view;
+  CacheView
+    *frame_view;
 
   /*
     Check frame geometry.
@@ -206,6 +200,8 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(frame_info != (FrameInfo *) NULL);
+  if ((frame_info->outer_bevel < 0) || (frame_info->inner_bevel < 0))
+    ThrowImageException(OptionError,"FrameIsLessThanImageSize");
   bevel_width=(unsigned long) (frame_info->outer_bevel+frame_info->inner_bevel);
   width=frame_info->width-frame_info->x-bevel_width;
   height=frame_info->height-frame_info->y-bevel_width;
@@ -284,135 +280,149 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     }
   status=MagickTrue;
   progress=0;
-  frame_view=AcquireCacheViewThreadSet(frame_image);
+  frame_view=AcquireCacheView(frame_image);
   height=(unsigned long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
     frame_info->inner_bevel);
-  q=QueueCacheViewAuthenticPixels(frame_view[0],0,0,frame_image->columns,height,
-    exception);
-  frame_indexes=GetCacheViewAuthenticIndexes(frame_view[0]);
-  if (q != (PixelPacket *) NULL)
+  if (height != 0)
     {
+      register IndexPacket
+        *__restrict frame_indexes;
+
+      register long
+        x;
+
+      register PixelPacket
+        *__restrict q;
+
       /*
         Draw top of ornamental border.
       */
-      for (y=0; y < (long) frame_info->outer_bevel; y++)
-      {
-        for (x=0; x < (long) (frame_image->columns-y); x++)
+      q=QueueCacheViewAuthenticPixels(frame_view,0,0,frame_image->columns,
+        height,exception);
+      frame_indexes=GetCacheViewAuthenticIndexQueue(frame_view);
+      if (q != (PixelPacket *) NULL)
         {
-          if (x < y)
-            SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          else
-            SetPixelPacket(frame_image,&accentuate,q,frame_indexes);
-          q++;
-          frame_indexes++;
+          /*
+            Draw top of ornamental border.
+          */
+          for (y=0; y < (long) frame_info->outer_bevel; y++)
+          {
+            for (x=0; x < (long) (frame_image->columns-y); x++)
+            {
+              if (x < y)
+                SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              else
+                SetPixelPacket(frame_image,&accentuate,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for ( ; x < (long) frame_image->columns; x++)
+            {
+              SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+          }
+          for (y=0; y < (long) (frame_info->y-bevel_width); y++)
+          {
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            width=frame_image->columns-2*frame_info->outer_bevel;
+            for (x=0; x < (long) width; x++)
+            {
+              SetPixelPacket(frame_image,&matte,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+          }
+          for (y=0; y < (long) frame_info->inner_bevel; y++)
+          {
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < (long) (frame_info->x-bevel_width); x++)
+            {
+              SetPixelPacket(frame_image,&matte,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            width=image->columns+((unsigned long) frame_info->inner_bevel << 1)-
+              y;
+            for (x=0; x < (long) width; x++)
+            {
+              if (x < y)
+                SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              else
+                SetPixelPacket(frame_image,&trough,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for ( ; x < (long) (image->columns+2*frame_info->inner_bevel); x++)
+            {
+              SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            width=frame_info->width-frame_info->x-image->columns-bevel_width;
+            for (x=0; x < (long) width; x++)
+            {
+              SetPixelPacket(frame_image,&matte,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+          }
+          (void) SyncCacheViewAuthenticPixels(frame_view,exception);
         }
-        for ( ; x < (long) frame_image->columns; x++)
-        {
-          SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-      }
-      for (y=0; y < (long) (frame_info->y-bevel_width); y++)
-      {
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        width=frame_image->columns-2*frame_info->outer_bevel;
-        for (x=0; x < (long) width; x++)
-        {
-          SetPixelPacket(frame_image,&matte,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-      }
-      for (y=0; y < (long) frame_info->inner_bevel; y++)
-      {
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for (x=0; x < (long) (frame_info->x-bevel_width); x++)
-        {
-          SetPixelPacket(frame_image,&matte,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        width=image->columns+((unsigned long) frame_info->inner_bevel << 1)-y;
-        for (x=0; x < (long) width; x++)
-        {
-          if (x < y)
-            SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          else
-            SetPixelPacket(frame_image,&trough,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for ( ; x < (long) (image->columns+2*frame_info->inner_bevel); x++)
-        {
-          SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        width=frame_info->width-frame_info->x-image->columns-bevel_width;
-        for (x=0; x < (long) width; x++)
-        {
-          SetPixelPacket(frame_image,&matte,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-      }
-      (void) SyncCacheViewAuthenticPixels(frame_view[0],exception);
     }
   /*
     Draw sides of ornamental border.
   */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,8) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
-      *frame_indexes;
+      *__restrict frame_indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     /*
       Initialize scanline with matte color.
     */
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=QueueCacheViewAuthenticPixels(frame_view[id],0,frame_info->y+y,
+    q=QueueCacheViewAuthenticPixels(frame_view,0,frame_info->y+y,
       frame_image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    frame_indexes=GetCacheViewAuthenticIndexes(frame_view[id]);
+    frame_indexes=GetCacheViewAuthenticIndexQueue(frame_view);
     for (x=0; x < (long) frame_info->outer_bevel; x++)
     {
       SetPixelPacket(frame_image,&highlight,q,frame_indexes);
@@ -459,7 +469,7 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
       q++;
       frame_indexes++;
     }
-    if (SyncCacheViewAuthenticPixels(frame_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(frame_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -467,7 +477,7 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_FrameImage)
 #endif
         proceed=SetImageProgress(image,FrameImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
@@ -476,101 +486,116 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   }
   height=(unsigned long) (frame_info->inner_bevel+frame_info->height-
     frame_info->y-image->rows-bevel_width+frame_info->outer_bevel);
-  q=QueueCacheViewAuthenticPixels(frame_view[0],0,(long) (frame_image->rows-
-    height),frame_image->columns,height,exception);
-  if (q != (PixelPacket *) NULL)
+  if (height != 0)
     {
+      register IndexPacket
+        *__restrict frame_indexes;
+
+      register long
+        x;
+
+      register PixelPacket
+        *__restrict q;
+
       /*
         Draw bottom of ornamental border.
       */
-      frame_indexes=GetCacheViewAuthenticIndexQueue(frame_view[0]);
-      for (y=frame_info->inner_bevel-1; y >= 0; y--)
-      {
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
+      q=QueueCacheViewAuthenticPixels(frame_view,0,(long) (frame_image->rows-
+        height),frame_image->columns,height,exception);
+      if (q != (PixelPacket *) NULL)
         {
-          SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          q++;
-          frame_indexes++;
+          /*
+            Draw bottom of ornamental border.
+          */
+          frame_indexes=GetCacheViewAuthenticIndexQueue(frame_view);
+          for (y=frame_info->inner_bevel-1; y >= 0; y--)
+          {
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < (long) (frame_info->x-bevel_width); x++)
+            {
+              SetPixelPacket(frame_image,&matte,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < y; x++)
+            {
+              SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for ( ; x < (long) (image->columns+2*frame_info->inner_bevel); x++)
+            {
+              if (x >= (long) (image->columns+2*frame_info->inner_bevel-y))
+                SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              else
+                SetPixelPacket(frame_image,&accentuate,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            width=frame_info->width-frame_info->x-image->columns-bevel_width;
+            for (x=0; x < (long) width; x++)
+            {
+              SetPixelPacket(frame_image,&matte,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+          }
+          height=frame_info->height-frame_info->y-image->rows-bevel_width;
+          for (y=0; y < (long) height; y++)
+          {
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            width=frame_image->columns-2*frame_info->outer_bevel;
+            for (x=0; x < (long) width; x++)
+            {
+              SetPixelPacket(frame_image,&matte,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for (x=0; x < (long) frame_info->outer_bevel; x++)
+            {
+              SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+          }
+          for (y=frame_info->outer_bevel-1; y >= 0; y--)
+          {
+            for (x=0; x < y; x++)
+            {
+              SetPixelPacket(frame_image,&highlight,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+            for ( ; x < (long) frame_image->columns; x++)
+            {
+              if (x >= (long) (frame_image->columns-y))
+                SetPixelPacket(frame_image,&shadow,q,frame_indexes);
+              else
+                SetPixelPacket(frame_image,&trough,q,frame_indexes);
+              q++;
+              frame_indexes++;
+            }
+          }
+          (void) SyncCacheViewAuthenticPixels(frame_view,exception);
         }
-        for (x=0; x < (long) (frame_info->x-bevel_width); x++)
-        {
-          SetPixelPacket(frame_image,&matte,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for (x=0; x < y; x++)
-        {
-          SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for ( ; x < (long) (image->columns+2*frame_info->inner_bevel); x++)
-        {
-          if (x >= (long) (image->columns+2*frame_info->inner_bevel-y))
-            SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          else
-            SetPixelPacket(frame_image,&accentuate,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        width=frame_info->width-frame_info->x-image->columns-bevel_width;
-        for (x=0; x < (long) width; x++)
-        {
-          SetPixelPacket(frame_image,&matte,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-      }
-      height=frame_info->height-frame_info->y-image->rows-bevel_width;
-      for (y=0; y < (long) height; y++)
-      {
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        width=frame_image->columns-2*frame_info->outer_bevel;
-        for (x=0; x < (long) width; x++)
-        {
-          SetPixelPacket(frame_image,&matte,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for (x=0; x < (long) frame_info->outer_bevel; x++)
-        {
-          SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-      }
-      for (y=frame_info->outer_bevel-1; y >= 0; y--)
-      {
-        for (x=0; x < y; x++)
-        {
-          SetPixelPacket(frame_image,&highlight,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-        for ( ; x < (long) frame_image->columns; x++)
-        {
-          if (x >= (long) (frame_image->columns-y))
-            SetPixelPacket(frame_image,&shadow,q,frame_indexes);
-          else
-            SetPixelPacket(frame_image,&trough,q,frame_indexes);
-          q++;
-          frame_indexes++;
-        }
-      }
-      (void) SyncCacheViewAuthenticPixels(frame_view[0],exception);
     }
-  frame_view=DestroyCacheViewThreadSet(frame_view);
+  frame_view=DestroyCacheView(frame_view);
   x=(long) (frame_info->outer_bevel+(frame_info->x-bevel_width)+
     frame_info->inner_bevel);
   y=(long) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
@@ -633,8 +658,8 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
     foreground,
     background;
 
-  ViewInfo
-    **image_view;
+  CacheView
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -660,24 +685,21 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
   status=MagickTrue;
   progress=0;
   exception=(&image->exception);
-  image_view=AcquireCacheViewThreadSet(image);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) raise_info->height; y++)
   {
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewAuthenticPixels(image_view[id],0,y,image->columns,1,
-      exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
@@ -719,7 +741,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
       q++;
     }
-    if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -732,22 +754,19 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       }
   }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=(long) raise_info->height; y < (long) (image->rows-raise_info->height); y++)
   {
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewAuthenticPixels(image_view[id],0,y,image->columns,1,
-      exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
@@ -778,7 +797,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
       q++;
     }
-    if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -791,22 +810,19 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       }
   }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=(long) (image->rows-raise_info->height); y < (long) image->rows; y++)
   {
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewAuthenticPixels(image_view[id],0,y,image->columns,1,
-      exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
@@ -845,7 +861,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
         ShadowFactor+(MagickRealType) background*(QuantumRange-ShadowFactor)));
       q++;
     }
-    if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -853,13 +869,13 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_RaiseImage)
 #endif
         proceed=SetImageProgress(image,RaiseImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
