@@ -57,6 +57,7 @@
 #include "magick/hashmap.h"
 #include "magick/list.h"
 #include "magick/memory_.h"
+#include "magick/policy.h"
 #include "magick/resource_.h"
 #include "magick/semaphore.h"
 #include "magick/string_.h"
@@ -116,11 +117,11 @@ static const char
     "  <delegate decode=\"rad\" command=\"&quot;ra_pfm&quot; &quot;%i&quot; &quot;%o&quot;\"/>"
     "  <delegate decode=\"rgba\" encode=\"rle\" mode=\"encode\" command=\"&quot;rawtorle&quot; -o &quot;%o&quot; -v &quot;%i&quot;\"/>"
     "  <delegate decode=\"scan\" command=\"&quot;scanimage&quot; -d &quot;%i&quot; &gt; &quot;%o&quot;\"/>"
-    "  <delegate encode=\"show\" stealth=\"True\" spawn=\"True\" command=\"&quot;/usr/local/bin/display&quot; -immutable -delay 0 -window-group %g -title &quot;%l of %f&quot; &quot;tmp:%i&quot;\"/>"
+    "  <delegate encode=\"show\" stealth=\"True\" spawn=\"True\" command=\"&quot;/usr/local/bin/display&quot; -immutable -delay 0 -window-group %g -title &quot;%l of %f&quot; &quot;temporary:%i&quot;\"/>"
     "  <delegate decode=\"shtml\" command=\"&quot;html2ps&quot; -U -o &quot;%o&quot; &quot;%i&quot;\"/>"
     "  <delegate decode=\"svg\" command=\"&quot;wmf2eps&quot; -o &quot;%o&quot; &quot;%i&quot;\"/>"
     "  <delegate decode=\"txt\" encode=\"ps\" mode=\"bi\" command=\"&quot;enscript&quot; -o &quot;%o&quot; &quot;%i&quot;\"/>"
-    "  <delegate encode=\"win\" stealth=\"True\" spawn=\"True\" command=\"&quot;/usr/local/bin/display&quot; -immutable -delay 0 -window-group %g -title &quot;%l of %f&quot; &quot;tmp:%i&quot;\"/>"
+    "  <delegate encode=\"win\" stealth=\"True\" spawn=\"True\" command=\"&quot;/usr/local/bin/display&quot; -immutable -delay 0 -window-group %g -title &quot;%l of %f&quot; &quot;temporary:%i&quot;\"/>"
     "  <delegate decode=\"wmf\" command=\"&quot;wmf2eps&quot; -o &quot;%o&quot; &quot;%i&quot;\"/>"
     "</delegatemap>";
 
@@ -814,13 +815,16 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   const DelegateInfo
     *delegate_info;
 
-  register long
-    i;
-
   MagickBooleanType
     status,
     temporary;
 
+  register long
+    i;
+
+  PolicyRights
+    rights;
+
   /*
     Get delegate.
   */
@@ -830,6 +834,19 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  rights=ExecutePolicyRights;
+  if (IsRightsAuthorized(DelegatePolicyDomain,rights,decode) == MagickFalse)
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
+        "NotAuthorized","`%s'",decode);
+      return(MagickFalse);
+    }
+  if (IsRightsAuthorized(DelegatePolicyDomain,rights,encode) == MagickFalse)
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),PolicyError,
+        "NotAuthorized","`%s'",encode);
+      return(MagickFalse);
+    }
   temporary=(*image->filename == '\0') ? MagickTrue : MagickFalse;
   if (temporary != MagickFalse)
     if (AcquireUniqueFilename(image->filename) == MagickFalse)
@@ -1191,7 +1208,7 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
   status=MagickTrue;
   delegate_info=(DelegateInfo *) NULL;
   token=AcquireString(xml);
-  for (q=(char *) xml; *q != '\0'; )
+  for (q=(const char *) xml; *q != '\0'; )
   {
     /*
       Interpret XML.
@@ -1308,14 +1325,15 @@ static MagickBooleanType LoadDelegateList(const char *xml,const char *filename,
                   path[MaxTextExtent];
 
                 NTGhostscriptEXE(path,MaxTextExtent);
-                SubstituteString((char **) &commands,"@PSDelegate@",path);
-                SubstituteString((char **) &commands,"\\","/");
+                (void) SubstituteString((char **) &commands,"@PSDelegate@",
+                  path);
+                (void) SubstituteString((char **) &commands,"\\","/");
               }
 #endif
-            SubstituteString((char **) &commands,"&amp;","\"");
-            SubstituteString((char **) &commands,"&quot;","\"");
-            SubstituteString((char **) &commands,"&gt;",">");
-            SubstituteString((char **) &commands,"&lt;","<");
+            (void) SubstituteString((char **) &commands,"&amp;","&");
+            (void) SubstituteString((char **) &commands,"&quot;","\"");
+            (void) SubstituteString((char **) &commands,"&gt;",">");
+            (void) SubstituteString((char **) &commands,"&lt;","<");
             delegate_info->commands=commands;
             break;
           }
@@ -1442,7 +1460,7 @@ static MagickBooleanType LoadDelegateLists(const char *filename,
     option=(const StringInfo *) GetNextValueInLinkedList(options);
   }
   options=DestroyConfigureOptions(options);
-  if ((delegate_list == (LinkedListInfo *) NULL) || 
+  if ((delegate_list == (LinkedListInfo *) NULL) ||
       (IsLinkedListEmpty(delegate_list) != MagickFalse))
     status|=LoadDelegateList(DelegateMap,"built-in",0,exception);
   return(status != 0 ? MagickTrue : MagickFalse);
