@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -25,15 +25,21 @@ extern "C" {
 #include <magick/color.h>
 
 #define OpaqueOpacity  ((Quantum) 0UL)
-#define TransparentOpacity  (QuantumRange)
+#define TransparentOpacity  ((Quantum) QuantumRange)
 
 typedef enum
 {
   UndefinedAlphaChannel,
   ActivateAlphaChannel,
+  BackgroundAlphaChannel,
+  CopyAlphaChannel,
   DeactivateAlphaChannel,
-  ResetAlphaChannel,
-  SetAlphaChannel
+  ExtractAlphaChannel,
+  OpaqueAlphaChannel,
+  ResetAlphaChannel,  /* deprecated */
+  SetAlphaChannel,
+  ShapeAlphaChannel,
+  TransparentAlphaChannel
 } AlphaChannelType;
 
 typedef enum
@@ -205,7 +211,7 @@ struct _Image
 
   double
     bias,
-    blur,
+    blur,  /* deprecated */
     fuzz;
 
   FilterTypes
@@ -278,7 +284,7 @@ struct _Image
   MagickBooleanType
     debug;
 
-  long
+  volatile long
     reference_count;
 
   SemaphoreInfo
@@ -316,7 +322,14 @@ struct _Image
     tile_offset;
 
   void
-    *properties;
+    *properties,
+    *artifacts;
+
+  ImageType
+    type;
+
+  MagickBooleanType
+    dither;
 };
 
 struct _ImageInfo
@@ -457,20 +470,16 @@ struct _ImageInfo
 
   void
     *profile;
+
+  MagickBooleanType
+    synchronize;
 };
 
-extern MagickExport const IndexPacket
-  *AcquireIndexes(const Image *);
-
-extern MagickExport const PixelPacket
-  *AcquireImagePixels(const Image *,const long,const long,const unsigned long,
-    const unsigned long,ExceptionInfo *);
-
 extern MagickExport ExceptionType
   CatchImageException(Image *);
 
 extern MagickExport Image
-  *AllocateImage(const ImageInfo *),
+  *AcquireImage(const ImageInfo *),
   *AppendImages(const Image *,const MagickBooleanType,ExceptionInfo *),
   *AverageImages(const Image *,ExceptionInfo *),
   *CloneImage(const Image *,const unsigned long,const unsigned long,
@@ -492,27 +501,21 @@ extern MagickExport ImageInfo
 extern MagickExport ImageType
   GetImageType(const Image *,ExceptionInfo *);
 
-extern MagickExport IndexPacket
-  *GetIndexes(const Image *);
-
-extern MagickExport long
-  InterpretImageFilename(char *,const size_t,const char *,int);
-
 extern MagickExport MagickBooleanType
-  AllocateImageColormap(Image *,const unsigned long),
+  AcquireImageColormap(Image *,const unsigned long),
   ClipImage(Image *),
-  ClipPathImage(Image *,const char *,const MagickBooleanType),
+  ClipImagePath(Image *,const char *,const MagickBooleanType),
   CycleColormapImage(Image *,const long),
-  GradientImage(Image *,const PixelPacket *,const PixelPacket *),
+  GetImageAlphaChannel(const Image *),
   IsTaintImage(const Image *),
   IsMagickConflict(const char *),
   IsHighDynamicRangeImage(const Image *,ExceptionInfo *),
   IsImageObject(const Image *),
   ListMagickInfo(FILE *,ExceptionInfo *),
   ModifyImage(Image **,ExceptionInfo *),
-  PlasmaImage(Image *,const SegmentInfo *,unsigned long,unsigned long),
   ResetImagePage(Image *,const char *),
   SeparateImageChannel(Image *,const ChannelType),
+  SetImageAlphaChannel(Image *,const AlphaChannelType),
   SetImageBackgroundColor(Image *),
   SetImageClipMask(Image *,const Image *),
   SetImageExtent(Image *,const unsigned long,const unsigned long),
@@ -524,25 +527,17 @@ extern MagickExport MagickBooleanType
   SortColormapByIntensity(Image *),
   StripImage(Image *),
   SyncImage(Image *),
-  SyncImagePixels(Image *),
   TextureImage(Image *,const Image *);
 
-extern MagickExport MagickPixelPacket
-  AcquireOneMagickPixel(const Image *,const long,const long,ExceptionInfo *);
+extern MagickExport size_t
+  InterpretImageFilename(const ImageInfo *,Image *,const char *,int,char *);
 
-extern MagickExport PixelPacket
-  AcquireOnePixel(const Image *,const long,const long,ExceptionInfo *),
-  AcquireOneVirtualPixel(const Image *,const VirtualPixelMethod,const long,
-    const long,ExceptionInfo *),
-  *GetImagePixels(Image *,const long,const long,const unsigned long,
-    const unsigned long),
-  GetOnePixel(Image *,const long,const long),
-  *GetPixels(const Image *),
-  *SetImagePixels(Image *,const long,const long,const unsigned long,
-    const unsigned long);
+extern MagickExport VirtualPixelMethod
+  GetImageVirtualPixelMethod(const Image *),
+  SetImageVirtualPixelMethod(const Image *,const VirtualPixelMethod);
 
 extern MagickExport void
-  AllocateNextImage(const ImageInfo *,Image *),
+  AcquireNextImage(const ImageInfo *,Image *),
   DestroyImagePixels(Image *),
   DisassociateImageStream(Image *),
   GetImageException(Image *,ExceptionInfo *),
