@@ -37,6 +37,7 @@ extern "C" {
 #define PrependMagickMethod(method) \
   EvaluateMagickPrefix(MAGICKCORE_NAMESPACE_PREFIX,method)
 
+#define AcquireAlignedMemory  PrependMagickMethod(AcquireAlignedMemory)
 #define AcquireCacheViewIndexes  PrependMagickMethod(AcquireCacheViewIndexes)
 #define AcquireCacheViewPixels  PrependMagickMethod(AcquireCacheViewPixels)
 #define AcquireCacheView  PrependMagickMethod(AcquireCacheView)
@@ -58,6 +59,8 @@ extern "C" {
 #define AcquireOneMagickPixel  PrependMagickMethod(AcquireOneMagickPixel)
 #define AcquireOnePixel  PrependMagickMethod(AcquireOnePixel)
 #define AcquireOneVirtualPixel  PrependMagickMethod(AcquireOneVirtualPixel)
+#define AcquirePixelCacheInfo  PrependMagickMethod(AcquirePixelCacheInfo)
+#define AcquirePixelCacheNexus  PrependMagickMethod(AcquirePixelCacheNexus)
 #define AcquirePixels  PrependMagickMethod(AcquirePixels)
 #define AcquireQuantizeInfo  PrependMagickMethod(AcquireQuantizeInfo)
 #define AcquireQuantumInfo  PrependMagickMethod(AcquireQuantumInfo)
@@ -115,6 +118,7 @@ extern "C" {
 #define BlackThresholdImage  PrependMagickMethod(BlackThresholdImage)
 #define BlobToFile  PrependMagickMethod(BlobToFile)
 #define BlobToImage  PrependMagickMethod(BlobToImage)
+#define BlueShiftImage  PrependMagickMethod(BlueShiftImage)
 #define BlurImageChannel  PrependMagickMethod(BlurImageChannel)
 #define BlurImage  PrependMagickMethod(BlurImage)
 #define BorderImage  PrependMagickMethod(BorderImage)
@@ -156,6 +160,7 @@ extern "C" {
 #define ClutImageChannel  PrependMagickMethod(ClutImageChannel)
 #define ClutImage  PrependMagickMethod(ClutImage)
 #define CoalesceImages  PrependMagickMethod(CoalesceImages)
+#define ColorDecisionListImage  PrependMagickMethod(ColorDecisionListImage)
 #define ColorFloodfillImage  PrependMagickMethod(ColorFloodfillImage)
 #define ColorizeImage  PrependMagickMethod(ColorizeImage)
 #define CombineImages  PrependMagickMethod(CombineImages)
@@ -254,6 +259,7 @@ extern "C" {
 #define DestroyPixelCacheInfo  PrependMagickMethod(DestroyPixelCacheInfo)
 #define DestroyPixelCacheNexus  PrependMagickMethod(DestroyPixelCacheNexus)
 #define DestroyPixelCacheResources  PrependMagickMethod(DestroyPixelCacheResources)
+#define DestroyPolicyList  PrependMagickMethod(DestroyPolicyList)
 #define DestroyQuantizeInfo  PrependMagickMethod(DestroyQuantizeInfo)
 #define DestroyQuantumInfo  PrependMagickMethod(DestroyQuantumInfo)
 #define DestroyRandomInfo  PrependMagickMethod(DestroyRandomInfo)
@@ -286,6 +292,8 @@ extern "C" {
 #define DrawGradientImage  PrependMagickMethod(DrawGradientImage)
 #define DrawImage  PrependMagickMethod(DrawImage)
 #define DrawPatternPath  PrependMagickMethod(DrawPatternPath)
+#define DrawPrimitive  PrependMagickMethod(DrawPrimitive)
+#define DuplicateBlob  PrependMagickMethod(DuplicateBlob)
 #define EdgeImage  PrependMagickMethod(EdgeImage)
 #define EmbossImage  PrependMagickMethod(EmbossImage)
 #define EncipherImage  PrependMagickMethod(EncipherImage)
@@ -303,6 +311,7 @@ extern "C" {
 #define ExportImagePixels  PrependMagickMethod(ExportImagePixels)
 #define ExportQuantumPixels  PrependMagickMethod(ExportQuantumPixels)
 #define ExtentImage  PrependMagickMethod(ExtentImage)
+#define ExtractSubimageFromImage  PrependMagickMethod(ExtractSubimageFromImage)
 #define FileToBlob  PrependMagickMethod(FileToBlob)
 #define FileToImage  PrependMagickMethod(FileToImage)
 #define FileToStringInfo  PrependMagickMethod(FileToStringInfo)
@@ -323,7 +332,10 @@ extern "C" {
 #define FormatMagickTime  PrependMagickMethod(FormatMagickTime)
 #define FormatStringList  PrependMagickMethod(FormatStringList)
 #define FormatString  PrependMagickMethod(FormatString)
+#define ForwardFourierTransformImage  PrependMagickMethod(ForwardFourierTransformImage)
 #define FrameImage  PrependMagickMethod(FrameImage)
+#define FunctionImageChannel  PrependMagickMethod(FunctionImageChannel)
+#define FunctionImage  PrependMagickMethod(FunctionImage)
 #define FuzzyColorCompare  PrependMagickMethod(FuzzyColorCompare)
 #define FuzzyColorMatch  PrependMagickMethod(FuzzyColorMatch)
 #define FuzzyOpacityCompare  PrependMagickMethod(FuzzyOpacityCompare)
@@ -331,11 +343,13 @@ extern "C" {
 #define FxEvaluateExpression  PrependMagickMethod(FxEvaluateExpression)
 #define FxImageChannel  PrependMagickMethod(FxImageChannel)
 #define FxImage  PrependMagickMethod(FxImage)
+#define FxPreprocessExpression  PrependMagickMethod(FxPreprocessExpression)
 #define GammaImageChannel  PrependMagickMethod(GammaImageChannel)
 #define GammaImage  PrependMagickMethod(GammaImage)
 #define GaussianBlurImageChannel  PrependMagickMethod(GaussianBlurImageChannel)
 #define GaussianBlurImage  PrependMagickMethod(GaussianBlurImage)
 #define GaussJordanElimination  PrependMagickMethod(GaussJordanElimination)
+#define GenerateDifferentialNoise  PrependMagickMethod(GenerateDifferentialNoise)
 #define GetAffineMatrix  PrependMagickMethod(GetAffineMatrix)
 #define GetAuthenticIndexQueue  PrependMagickMethod(GetAuthenticIndexQueue)
 #define GetAuthenticPixelCacheNexus  PrependMagickMethod(GetAuthenticPixelCacheNexus)
@@ -401,6 +415,7 @@ extern "C" {
 #define GetImageChannelDistortion  PrependMagickMethod(GetImageChannelDistortion)
 #define GetImageChannelDistortions  PrependMagickMethod(GetImageChannelDistortions)
 #define GetImageChannelExtrema  PrependMagickMethod(GetImageChannelExtrema)
+#define GetImageChannelKurtosis  PrependMagickMethod(GetImageChannelKurtosis)
 #define GetImageChannelMean  PrependMagickMethod(GetImageChannelMean)
 #define GetImageChannelRange  PrependMagickMethod(GetImageChannelRange)
 #define GetImageChannelStatistics  PrependMagickMethod(GetImageChannelStatistics)
@@ -420,6 +435,7 @@ extern "C" {
 #define GetImageHistogram  PrependMagickMethod(GetImageHistogram)
 #define GetImageIndexInList  PrependMagickMethod(GetImageIndexInList)
 #define GetImageInfo  PrependMagickMethod(GetImageInfo)
+#define GetImageKurtosis  PrependMagickMethod(GetImageKurtosis)
 #define GetImageListIndex  PrependMagickMethod(GetImageListIndex)
 #define GetImageListLength  PrependMagickMethod(GetImageListLength)
 #define GetImageList  PrependMagickMethod(GetImageList)
@@ -428,6 +444,7 @@ extern "C" {
 #define GetImageMask  PrependMagickMethod(GetImageMask)
 #define GetImageMean  PrependMagickMethod(GetImageMean)
 #define GetImageOption  PrependMagickMethod(GetImageOption)
+#define GetImagePixelCache  PrependMagickMethod(GetImagePixelCache)
 #define GetImagePixels  PrependMagickMethod(GetImagePixels)
 #define GetImageProfile  PrependMagickMethod(GetImageProfile)
 #define GetImageProperty  PrependMagickMethod(GetImageProperty)
@@ -449,7 +466,6 @@ extern "C" {
 #define GetLocaleOptions  PrependMagickMethod(GetLocaleOptions)
 #define GetLocaleValue  PrependMagickMethod(GetLocaleValue)
 #define GetLogInfoList  PrependMagickMethod(GetLogInfoList)
-#define GetLogInfo  PrependMagickMethod(GetLogInfo)
 #define GetLogList  PrependMagickMethod(GetLogList)
 #define GetLogName  PrependMagickMethod(GetLogName)
 #define GetMagicInfoList  PrependMagickMethod(GetMagicInfoList)
@@ -468,6 +484,7 @@ extern "C" {
 #define GetMagickOptions  PrependMagickMethod(GetMagickOptions)
 #define GetMagickPackageName  PrependMagickMethod(GetMagickPackageName)
 #define GetMagickPixelPacket  PrependMagickMethod(GetMagickPixelPacket)
+#define GetMagickProperty  PrependMagickMethod(GetMagickProperty)
 #define GetMagickQuantumDepth  PrependMagickMethod(GetMagickQuantumDepth)
 #define GetMagickQuantumRange  PrependMagickMethod(GetMagickQuantumRange)
 #define GetMagickRawSupport  PrependMagickMethod(GetMagickRawSupport)
@@ -524,16 +541,16 @@ extern "C" {
 #define GetPathComponent  PrependMagickMethod(GetPathComponent)
 #define GetPathComponents  PrependMagickMethod(GetPathComponents)
 #define GetPixelCacheColorspace  PrependMagickMethod(GetPixelCacheColorspace)
-#define GetPixelCacheInfo  PrependMagickMethod(GetPixelCacheInfo)
-#define GetPixelCacheMaximumThreads  PrependMagickMethod(GetPixelCacheMaximumThreads)
 #define GetPixelCacheMethods  PrependMagickMethod(GetPixelCacheMethods)
 #define GetPixelCacheNexusExtent  PrependMagickMethod(GetPixelCacheNexusExtent)
 #define GetPixelCacheNexusIndexes  PrependMagickMethod(GetPixelCacheNexusIndexes)
 #define GetPixelCacheNexusPixels  PrependMagickMethod(GetPixelCacheNexusPixels)
 #define GetPixelCacheStorageClass  PrependMagickMethod(GetPixelCacheStorageClass)
-#define GetPixelCacheThreadId  PrependMagickMethod(GetPixelCacheThreadId)
 #define GetPixelCacheVirtualMethod  PrependMagickMethod(GetPixelCacheVirtualMethod)
 #define GetPixels  PrependMagickMethod(GetPixels)
+#define GetPolicyInfoList  PrependMagickMethod(GetPolicyInfoList)
+#define GetPolicyList  PrependMagickMethod(GetPolicyList)
+#define GetPolicyValue  PrependMagickMethod(GetPolicyValue)
 #define GetPreviousImageInList  PrependMagickMethod(GetPreviousImageInList)
 #define GetPreviousImage  PrependMagickMethod(GetPreviousImage)
 #define GetPseudoRandomValue  PrependMagickMethod(GetPseudoRandomValue)
@@ -567,7 +584,6 @@ extern "C" {
 #define GetValueFromSplayTree  PrependMagickMethod(GetValueFromSplayTree)
 #define GetVirtualIndexesFromNexus  PrependMagickMethod(GetVirtualIndexesFromNexus)
 #define GetVirtualIndexQueue  PrependMagickMethod(GetVirtualIndexQueue)
-#define AcquirePixelCacheNexus  PrependMagickMethod(AcquirePixelCacheNexus)
 #define GetVirtualPixelQueue  PrependMagickMethod(GetVirtualPixelQueue)
 #define GetVirtualPixelsFromNexus  PrependMagickMethod(GetVirtualPixelsFromNexus)
 #define GetVirtualPixelsNexus  PrependMagickMethod(GetVirtualPixelsNexus)
@@ -584,6 +600,8 @@ extern "C" {
 #define GlobExpression  PrependMagickMethod(GlobExpression)
 #define GradientImage  PrependMagickMethod(GradientImage)
 #define GravityAdjustGeometry  PrependMagickMethod(GravityAdjustGeometry)
+#define HaldClutImageChannel  PrependMagickMethod(HaldClutImageChannel)
+#define HaldClutImage  PrependMagickMethod(HaldClutImage)
 #define HashPointerType  PrependMagickMethod(HashPointerType)
 #define HashStringInfoType  PrependMagickMethod(HashStringInfoType)
 #define HashStringType  PrependMagickMethod(HashStringType)
@@ -615,6 +633,7 @@ extern "C" {
 #define InterpretImageAttributes  PrependMagickMethod(InterpretImageAttributes)
 #define InterpretImageFilename  PrependMagickMethod(InterpretImageFilename)
 #define InterpretImageProperties  PrependMagickMethod(InterpretImageProperties)
+#define InverseFourierTransformImage  PrependMagickMethod(InverseFourierTransformImage)
 #define InvokeDelegate  PrependMagickMethod(InvokeDelegate)
 #define InvokeDynamicImageFilter  PrependMagickMethod(InvokeDynamicImageFilter)
 #define IsBlobExempt  PrependMagickMethod(IsBlobExempt)
@@ -642,6 +661,7 @@ extern "C" {
 #define IsOpaqueImage  PrependMagickMethod(IsOpaqueImage)
 #define IsPaletteImage  PrependMagickMethod(IsPaletteImage)
 #define IsPathAccessible  PrependMagickMethod(IsPathAccessible)
+#define IsRightsAuthorized  PrependMagickMethod(IsRightsAuthorized)
 #define IsSceneGeometry  PrependMagickMethod(IsSceneGeometry)
 #define IsSubimage  PrependMagickMethod(IsSubimage)
 #define IsTaintImage  PrependMagickMethod(IsTaintImage)
@@ -668,6 +688,7 @@ extern "C" {
 #define ListMagickResourceInfo  PrependMagickMethod(ListMagickResourceInfo)
 #define ListMimeInfo  PrependMagickMethod(ListMimeInfo)
 #define ListModuleInfo  PrependMagickMethod(ListModuleInfo)
+#define ListPolicyInfo  PrependMagickMethod(ListPolicyInfo)
 #define ListThresholdMapFile  PrependMagickMethod(ListThresholdMapFile)
 #define ListThresholdMaps  PrependMagickMethod(ListThresholdMaps)
 #define ListTypeInfo  PrependMagickMethod(ListTypeInfo)
@@ -683,11 +704,15 @@ extern "C" {
 #define LZWEncodeImage  PrependMagickMethod(LZWEncodeImage)
 #define MagickCoreGenesis  PrependMagickMethod(MagickCoreGenesis)
 #define MagickCoreTerminus  PrependMagickMethod(MagickCoreTerminus)
+#define MagickCreateThreadKey  PrependMagickMethod(MagickCreateThreadKey)
+#define MagickDeleteThreadKey  PrependMagickMethod(MagickDeleteThreadKey)
 #define MagickError  PrependMagickMethod(MagickError)
 #define MagickFatalError  PrependMagickMethod(MagickFatalError)
+#define MagickGetThreadValue  PrependMagickMethod(MagickGetThreadValue)
 #define MagickIncarnate  PrependMagickMethod(MagickIncarnate)
 #define MagickMonitor  PrependMagickMethod(MagickMonitor)
 #define MagickOptionToMnemonic  PrependMagickMethod(MagickOptionToMnemonic)
+#define MagickSetThreadValue  PrependMagickMethod(MagickSetThreadValue)
 #define MagickToMime  PrependMagickMethod(MagickToMime)
 #define MagickWarning  PrependMagickMethod(MagickWarning)
 #define MagnifyImage  PrependMagickMethod(MagnifyImage)
@@ -749,12 +774,16 @@ extern "C" {
 #define ParseMagickOption  PrependMagickMethod(ParseMagickOption)
 #define ParseMetaGeometry  PrependMagickMethod(ParseMetaGeometry)
 #define ParsePageGeometry  PrependMagickMethod(ParsePageGeometry)
+#define ParseRegionGeometry  PrependMagickMethod(ParseRegionGeometry)
 #define ParseSizeGeometry  PrependMagickMethod(ParseSizeGeometry)
+#define PasskeyDecipherImage  PrependMagickMethod(PasskeyDecipherImage)
+#define PasskeyEncipherImage  PrependMagickMethod(PasskeyEncipherImage)
 #define PersistPixelCache  PrependMagickMethod(PersistPixelCache)
 #define PingBlob  PrependMagickMethod(PingBlob)
 #define PingImage  PrependMagickMethod(PingImage)
 #define PingImages  PrependMagickMethod(PingImages)
 #define PlasmaImage  PrependMagickMethod(PlasmaImage)
+#define PlasmaImageProxy  PrependMagickMethod(PlasmaImageProxy)
 #define PolaroidImage  PrependMagickMethod(PolaroidImage)
 #define PopImageList  PrependMagickMethod(PopImageList)
 #define PopImagePixels  PrependMagickMethod(PopImagePixels)
@@ -763,6 +792,7 @@ extern "C" {
 #define PrependImageToList  PrependMagickMethod(PrependImageToList)
 #define PreviewImage  PrependMagickMethod(PreviewImage)
 #define PrintStringInfo  PrependMagickMethod(PrintStringInfo)
+#define process_message  PrependMagickMethod(process_message)
 #define ProfileImage  PrependMagickMethod(ProfileImage)
 #define PruneTagFromXMLTree  PrependMagickMethod(PruneTagFromXMLTree)
 #define PushImageList  PrependMagickMethod(PushImageList)
@@ -811,6 +841,7 @@ extern "C" {
 #define RegisterAVSImage  PrependMagickMethod(RegisterAVSImage)
 #define RegisterBMPImage  PrependMagickMethod(RegisterBMPImage)
 #define RegisterBRAILLEImage  PrependMagickMethod(RegisterBRAILLEImage)
+#define RegisterCALSImage  PrependMagickMethod(RegisterCALSImage)
 #define RegisterCAPTIONImage  PrependMagickMethod(RegisterCAPTIONImage)
 #define RegisterCINImage  PrependMagickMethod(RegisterCINImage)
 #define RegisterCIPImage  PrependMagickMethod(RegisterCIPImage)
@@ -820,18 +851,20 @@ extern "C" {
 #define RegisterDCMImage  PrependMagickMethod(RegisterDCMImage)
 #define RegisterDDSImage  PrependMagickMethod(RegisterDDSImage)
 #define RegisterDIBImage  PrependMagickMethod(RegisterDIBImage)
+#define RegisterDJVUImage  PrependMagickMethod(RegisterDJVUImage)
 #define RegisterDNGImage  PrependMagickMethod(RegisterDNGImage)
 #define RegisterDOTImage  PrependMagickMethod(RegisterDOTImage)
 #define RegisterDPSImage  PrependMagickMethod(RegisterDPSImage)
 #define RegisterDPXImage  PrependMagickMethod(RegisterDPXImage)
 #define RegisterEPTImage  PrependMagickMethod(RegisterEPTImage)
-#define RegisterEXRImage  PrependMagickMethod(RegisterEXRImage)
 #define RegisterFAXImage  PrependMagickMethod(RegisterFAXImage)
 #define RegisterFITSImage  PrependMagickMethod(RegisterFITSImage)
 #define RegisterGIFImage  PrependMagickMethod(RegisterGIFImage)
 #define RegisterGRADIENTImage  PrependMagickMethod(RegisterGRADIENTImage)
 #define RegisterGRAYImage  PrependMagickMethod(RegisterGRAYImage)
+#define RegisterHALDImage  PrependMagickMethod(RegisterHALDImage)
 #define RegisterHISTOGRAMImage  PrependMagickMethod(RegisterHISTOGRAMImage)
+#define RegisterHRZImage  PrependMagickMethod(RegisterHRZImage)
 #define RegisterHTMLImage  PrependMagickMethod(RegisterHTMLImage)
 #define RegisterICONImage  PrependMagickMethod(RegisterICONImage)
 #define RegisterINFOImage  PrependMagickMethod(RegisterINFOImage)
@@ -910,6 +943,7 @@ extern "C" {
 #define RegisterXWDImage  PrependMagickMethod(RegisterXWDImage)
 #define RegisterYCBCRImage  PrependMagickMethod(RegisterYCBCRImage)
 #define RegisterYUVImage  PrependMagickMethod(RegisterYUVImage)
+#define RelinquishAlignedMemory  PrependMagickMethod(RelinquishAlignedMemory)
 #define RelinquishMagickMatrix  PrependMagickMethod(RelinquishMagickMatrix)
 #define RelinquishMagickMemory  PrependMagickMethod(RelinquishMagickMemory)
 #define RelinquishMagickResource  PrependMagickMethod(RelinquishMagickResource)
@@ -963,6 +997,8 @@ extern "C" {
 #define SeedPseudoRandomGenerator  PrependMagickMethod(SeedPseudoRandomGenerator)
 #define SeekBlob  PrependMagickMethod(SeekBlob)
 #define SegmentImage  PrependMagickMethod(SegmentImage)
+#define SelectiveBlurImageChannel  PrependMagickMethod(SelectiveBlurImageChannel)
+#define SelectiveBlurImage  PrependMagickMethod(SelectiveBlurImage)
 #define SeparateImageChannel  PrependMagickMethod(SeparateImageChannel)
 #define SeparateImages  PrependMagickMethod(SeparateImages)
 #define SepiaToneImage  PrependMagickMethod(SepiaToneImage)
@@ -1014,7 +1050,6 @@ extern "C" {
 #define SetMagickRegistry  PrependMagickMethod(SetMagickRegistry)
 #define SetMagickResourceLimit  PrependMagickMethod(SetMagickResourceLimit)
 #define SetMonitorHandler  PrependMagickMethod(SetMonitorHandler)
-#define SetPixelCacheMaximumThreads  PrependMagickMethod(SetPixelCacheMaximumThreads)
 #define SetPixelCacheMethods  PrependMagickMethod(SetPixelCacheMethods)
 #define SetPixelCacheVirtualMethod  PrependMagickMethod(SetPixelCacheVirtualMethod)
 #define SetQuantumAlphaType  PrependMagickMethod(SetQuantumAlphaType)
@@ -1053,6 +1088,7 @@ extern "C" {
 #define SigmoidalContrastImageChannel  PrependMagickMethod(SigmoidalContrastImageChannel)
 #define SigmoidalContrastImage  PrependMagickMethod(SigmoidalContrastImage)
 #define SignatureImage  PrependMagickMethod(SignatureImage)
+#define SimilarityImage  PrependMagickMethod(SimilarityImage)
 #define SizeBlob  PrependMagickMethod(SizeBlob)
 #define SketchImage  PrependMagickMethod(SketchImage)
 #define SolarizeImage  PrependMagickMethod(SolarizeImage)
@@ -1087,10 +1123,11 @@ extern "C" {
 #define SyncCacheViewPixels  PrependMagickMethod(SyncCacheViewPixels)
 #define SyncCacheView  PrependMagickMethod(SyncCacheView)
 #define SyncImageList  PrependMagickMethod(SyncImageList)
-#define GetImagePixelCache  PrependMagickMethod(GetImagePixelCache)
 #define SyncImagePixels  PrependMagickMethod(SyncImagePixels)
 #define SyncImage  PrependMagickMethod(SyncImage)
 #define SyncImageProfiles  PrependMagickMethod(SyncImageProfiles)
+#define SyncImageSettings  PrependMagickMethod(SyncImageSettings)
+#define SyncImagesSettings  PrependMagickMethod(SyncImagesSettings)
 #define SyncNextImageInList  PrependMagickMethod(SyncNextImageInList)
 #define SystemCommand  PrependMagickMethod(SystemCommand)
 #define TellBlob  PrependMagickMethod(TellBlob)
@@ -1106,6 +1143,7 @@ extern "C" {
 #define Tokenizer  PrependMagickMethod(Tokenizer)
 #define TransformColorspace  PrependMagickMethod(TransformColorspace)
 #define TransformHSL  PrependMagickMethod(TransformHSL)
+#define TransformImageColorspace  PrependMagickMethod(TransformImageColorspace)
 #define TransformImage  PrependMagickMethod(TransformImage)
 #define TransformImages  PrependMagickMethod(TransformImages)
 #define TransformRGBImage  PrependMagickMethod(TransformRGBImage)
@@ -1124,6 +1162,7 @@ extern "C" {
 #define UnregisterAVSImage  PrependMagickMethod(UnregisterAVSImage)
 #define UnregisterBMPImage  PrependMagickMethod(UnregisterBMPImage)
 #define UnregisterBRAILLEImage  PrependMagickMethod(UnregisterBRAILLEImage)
+#define UnregisterCALSImage  PrependMagickMethod(UnregisterCALSImage)
 #define UnregisterCAPTIONImage  PrependMagickMethod(UnregisterCAPTIONImage)
 #define UnregisterCINImage  PrependMagickMethod(UnregisterCINImage)
 #define UnregisterCIPImage  PrependMagickMethod(UnregisterCIPImage)
@@ -1133,18 +1172,20 @@ extern "C" {
 #define UnregisterDCMImage  PrependMagickMethod(UnregisterDCMImage)
 #define UnregisterDDSImage  PrependMagickMethod(UnregisterDDSImage)
 #define UnregisterDIBImage  PrependMagickMethod(UnregisterDIBImage)
+#define UnregisterDJVUImage  PrependMagickMethod(UnregisterDJVUImage)
 #define UnregisterDNGImage  PrependMagickMethod(UnregisterDNGImage)
 #define UnregisterDOTImage  PrependMagickMethod(UnregisterDOTImage)
 #define UnregisterDPSImage  PrependMagickMethod(UnregisterDPSImage)
 #define UnregisterDPXImage  PrependMagickMethod(UnregisterDPXImage)
 #define UnregisterEPTImage  PrependMagickMethod(UnregisterEPTImage)
-#define UnregisterEXRImage  PrependMagickMethod(UnregisterEXRImage)
 #define UnregisterFAXImage  PrependMagickMethod(UnregisterFAXImage)
 #define UnregisterFITSImage  PrependMagickMethod(UnregisterFITSImage)
 #define UnregisterGIFImage  PrependMagickMethod(UnregisterGIFImage)
 #define UnregisterGRADIENTImage  PrependMagickMethod(UnregisterGRADIENTImage)
 #define UnregisterGRAYImage  PrependMagickMethod(UnregisterGRAYImage)
+#define UnregisterHALDImage  PrependMagickMethod(UnregisterHALDImage)
 #define UnregisterHISTOGRAMImage  PrependMagickMethod(UnregisterHISTOGRAMImage)
+#define UnregisterHRZImage  PrependMagickMethod(UnregisterHRZImage)
 #define UnregisterHTMLImage  PrependMagickMethod(UnregisterHTMLImage)
 #define UnregisterICONImage  PrependMagickMethod(UnregisterICONImage)
 #define UnregisterINFOImage  PrependMagickMethod(UnregisterINFOImage)
