@@ -62,6 +62,7 @@
 #include "magick/monitor.h"
 #include "magick/montage.h"
 #include "magick/option.h"
+#include "magick/policy.h"
 #include "magick/property.h"
 #include "magick/quantize.h"
 #include "magick/quantum.h"
@@ -90,6 +91,7 @@ static const OptionInfo
   {
     { "Undefined", (long) UndefinedAlphaChannel, MagickTrue },
     { "Activate", (long) ActivateAlphaChannel, MagickFalse },
+    { "Background", (long) BackgroundAlphaChannel, MagickFalse },
     { "Copy", (long) CopyAlphaChannel, MagickFalse },
     { "Deactivate", (long) DeactivateAlphaChannel, MagickFalse },
     { "Extract", (long) ExtractAlphaChannel, MagickFalse },
@@ -98,7 +100,7 @@ static const OptionInfo
     { "Opaque", (long) OpaqueAlphaChannel, MagickFalse },
     { "Set", (long) SetAlphaChannel, MagickFalse },
     { "Shape", (long) ShapeAlphaChannel, MagickFalse },
-    { "Reset", (long) SetAlphaChannel, MagickTrue }, /* depreciated */
+    { "Reset", (long) SetAlphaChannel, MagickTrue }, /* deprecated */
     { "Transparent", (long) TransparentAlphaChannel, MagickFalse },
     { (char *) NULL, (long) UndefinedAlphaChannel, MagickFalse }
   },
@@ -189,6 +191,8 @@ static const OptionInfo
     { "-blend", 1L, MagickFalse },
     { "+blue-primary", 0L, MagickFalse },
     { "-blue-primary", 1L, MagickFalse },
+    { "+blue-shift", 1L, MagickFalse },
+    { "-blue-shift", 1L, MagickFalse },
     { "+blur", 0L, MagickFalse },
     { "-blur", 1L, MagickFalse },
     { "+border", 0L, MagickFalse },
@@ -201,6 +205,8 @@ static const OptionInfo
     { "-box", 1L, MagickFalse },
     { "+cache", 0L, MagickFalse },
     { "-cache", 1L, MagickFalse },
+    { "+cdl", 1L, MagickFalse },
+    { "-cdl", 1L, MagickFalse },
     { "+channel", 0L, MagickFalse },
     { "-channel", 1L, MagickFalse },
     { "+charcoal", 0L, MagickFalse },
@@ -341,6 +347,8 @@ static const OptionInfo
     { "-gravity", 1L, MagickFalse },
     { "+green-primary", 0L, MagickFalse },
     { "-green-primary", 1L, MagickFalse },
+    { "+hald-clut", 0L, MagickFalse },
+    { "-hald-clut", 0L, MagickFalse },
     { "+help", 0L, MagickFalse },
     { "-help", 0L, MagickFalse },
     { "+highlight-color", 1L, MagickFalse },
@@ -659,6 +667,7 @@ static const OptionInfo
     { "Add", (long) AddCompositeOp, MagickFalse },
     { "Atop", (long) AtopCompositeOp, MagickFalse },
     { "Blend", (long) BlendCompositeOp, MagickFalse },
+    { "Blur", (long) BlurCompositeOp, MagickFalse },
     { "Bumpmap", (long) BumpmapCompositeOp, MagickFalse },
     { "ChangeMask", (long) ChangeMaskCompositeOp, MagickFalse },
     { "Clear", (long) ClearCompositeOp, MagickFalse },
@@ -680,6 +689,7 @@ static const OptionInfo
     { "Difference", (long) DifferenceCompositeOp, MagickFalse },
     { "Displace", (long) DisplaceCompositeOp, MagickFalse },
     { "Dissolve", (long) DissolveCompositeOp, MagickFalse },
+    { "Distort", (long) DistortCompositeOp, MagickFalse },
     { "DstAtop", (long) DstAtopCompositeOp, MagickFalse },
     { "DstIn", (long) DstInCompositeOp, MagickFalse },
     { "DstOut", (long) DstOutCompositeOp, MagickFalse },
@@ -690,8 +700,11 @@ static const OptionInfo
     { "Hue", (long) HueCompositeOp, MagickFalse },
     { "In", (long) InCompositeOp, MagickFalse },
     { "Lighten", (long) LightenCompositeOp, MagickFalse },
+    { "LinearBurn", (long) LinearBurnCompositeOp, MagickFalse },
+    { "LinearDodge", (long) LinearDodgeCompositeOp, MagickFalse },
     { "LinearLight", (long) LinearLightCompositeOp, MagickFalse },
     { "Luminize", (long) LuminizeCompositeOp, MagickFalse },
+    { "Mathematics", (long) MathematicsCompositeOp, MagickFalse },
     { "Minus", (long) MinusCompositeOp, MagickFalse },
     { "Modulate", (long) ModulateCompositeOp, MagickFalse },
     { "Multiply", (long) MultiplyCompositeOp, MagickFalse },
@@ -699,6 +712,8 @@ static const OptionInfo
     { "Out", (long) OutCompositeOp, MagickFalse },
     { "Overlay", (long) OverlayCompositeOp, MagickFalse },
     { "Over", (long) OverCompositeOp, MagickFalse },
+    { "PegtopLight", (long) PegtopLightCompositeOp, MagickFalse },
+    { "PinLight", (long) PinLightCompositeOp, MagickFalse },
     { "Plus", (long) PlusCompositeOp, MagickFalse },
     { "Replace", (long) ReplaceCompositeOp, MagickFalse },
     { "Saturate", (long) SaturateCompositeOp, MagickFalse },
@@ -711,7 +726,8 @@ static const OptionInfo
     { "SrcOver", (long) SrcOverCompositeOp, MagickFalse },
     { "Src", (long) SrcCompositeOp, MagickFalse },
     { "Subtract", (long) SubtractCompositeOp, MagickFalse },
-    { "Threshold", (long) ThresholdCompositeOp, MagickFalse },
+    { "Threshold", (long) ThresholdCompositeOp, MagickTrue }, /* depreciate */
+    { "VividLight", (long) VividLightCompositeOp, MagickFalse },
     { "Xor", (long) XorCompositeOp, MagickFalse },
     { (char *) NULL, (long) UndefinedCompositeOp, MagickFalse }
   },
@@ -801,8 +817,10 @@ static const OptionInfo
     { "SRT", (long) ScaleRotateTranslateDistortion, MagickFalse },
     { "Perspective", (long) PerspectiveDistortion, MagickFalse },
     { "PerspectiveProjection", (long) PerspectiveProjectionDistortion, MagickFalse },
-    { "Bilinear", (long) BilinearDistortion, MagickFalse },
-    { "Polynomial", (long) PolynomialDistortion, MagickTrue }, /* not ready */
+    { "Bilinear", (long) BilinearForwardDistortion, MagickTrue },
+    { "BilinearForward", (long) BilinearForwardDistortion, MagickFalse },
+    { "BilinearReverse", (long) BilinearReverseDistortion, MagickFalse },
+    { "Polynomial", (long) PolynomialDistortion, MagickFalse },
     { "Arc", (long) ArcDistortion, MagickFalse },
     { "Polar", (long) PolarDistortion, MagickFalse },
     { "DePolar", (long) DePolarDistortion, MagickFalse },
@@ -830,7 +848,10 @@ static const OptionInfo
   {
     { "Undefined", (long) UndefinedEvaluateOperator, MagickTrue },
     { "Add", (long) AddEvaluateOperator, MagickFalse },
+    { "AddModulus", (long) AddModulusEvaluateOperator, MagickFalse },
     { "And", (long) AndEvaluateOperator, MagickFalse },
+    { "Cos", (long) CosineEvaluateOperator, MagickFalse },
+    { "Cosine", (long) CosineEvaluateOperator, MagickFalse },
     { "Divide", (long) DivideEvaluateOperator, MagickFalse },
     { "GaussianNoise", (long) GaussianNoiseEvaluateOperator, MagickFalse },
     { "ImpulseNoise", (long) ImpulseNoiseEvaluateOperator, MagickFalse },
@@ -846,6 +867,8 @@ static const OptionInfo
     { "Pow", (long) PowEvaluateOperator, MagickFalse },
     { "RightShift", (long) RightShiftEvaluateOperator, MagickFalse },
     { "Set", (long) SetEvaluateOperator, MagickFalse },
+    { "Sin", (long) SineEvaluateOperator, MagickFalse },
+    { "Sine", (long) SineEvaluateOperator, MagickFalse },
     { "Subtract", (long) SubtractEvaluateOperator, MagickFalse },
     { "Threshold", (long) ThresholdEvaluateOperator, MagickFalse },
     { "ThresholdBlack", (long) ThresholdBlackEvaluateOperator, MagickFalse },
@@ -887,6 +910,15 @@ static const OptionInfo
     { "Welsh", (long) WelshFilter, MagickFalse },
     { (char *) NULL, (long) UndefinedFilter, MagickFalse }
   },
+  FunctionOptions[] =
+  {
+    { "Undefined", (long) UndefinedFunction, MagickTrue },
+    { "Polynomial", (long) PolynomialFunction, MagickFalse },
+    { "Sinusoid", (long) SinusoidFunction, MagickFalse },
+    { "ArcSin", (long) ArcsinFunction, MagickFalse },
+    { "ArcTan", (long) ArctanFunction, MagickFalse },
+    { (char *) NULL, (long) UndefinedFunction, MagickFalse }
+  },
   GravityOptions[] =
   {
     { "Undefined", (long) UndefinedGravity, MagickTrue },
@@ -919,6 +951,8 @@ static const OptionInfo
     { "delete", MagickTrue, MagickFalse },
     { "flatten", MagickTrue, MagickFalse },
     { "fx", MagickTrue, MagickFalse },
+    { "hald-clut", MagickTrue, MagickFalse },
+    { "ift", MagickTrue, MagickFalse },
     { "identify", MagickTrue, MagickFalse },
     { "insert", MagickTrue, MagickFalse },
     { "layers", MagickTrue, MagickFalse },
@@ -1007,20 +1041,23 @@ static const OptionInfo
   },
   ListOptions[] =
   {
-    { "Undefined", (long) MagickUndefinedOptions, MagickTrue },
     { "Align", (long) MagickAlignOptions, MagickFalse },
     { "Alpha", (long) MagickAlphaOptions, MagickFalse },
     { "Boolean", (long) MagickBooleanOptions, MagickFalse },
     { "Channel", (long) MagickChannelOptions, MagickFalse },
     { "Class", (long) MagickClassOptions, MagickFalse },
     { "ClipPath", (long) MagickClipPathOptions, MagickFalse },
+    { "Coder", (long) MagickCoderOptions, MagickFalse },
+    { "Color", (long) MagickColorOptions, MagickFalse },
     { "Colorspace", (long) MagickColorspaceOptions, MagickFalse },
-    { "Compose", (long) MagickComposeOptions, MagickFalse },
     { "Command", (long) MagickCommandOptions, MagickFalse },
+    { "Compose", (long) MagickComposeOptions, MagickFalse },
     { "Compress", (long) MagickCompressOptions, MagickFalse },
+    { "Configure", (long) MagickConfigureOptions, MagickFalse },
     { "DataType", (long) MagickDataTypeOptions, MagickFalse },
     { "Debug", (long) MagickDebugOptions, MagickFalse },
     { "Decoration", (long) MagickDecorateOptions, MagickFalse },
+    { "Delegate", (long) MagickDelegateOptions, MagickFalse },
     { "Dispose", (long) MagickDisposeOptions, MagickFalse },
     { "Distort", (long) MagickDistortOptions, MagickFalse },
     { "Dither", (long) MagickDitherOptions, MagickFalse },
@@ -1029,6 +1066,8 @@ static const OptionInfo
     { "FillRule", (long) MagickFillRuleOptions, MagickFalse },
     { "Filter", (long) MagickFilterOptions, MagickFalse },
     { "Font", (long) MagickFontOptions, MagickFalse },
+    { "Format", (long) MagickFormatOptions, MagickFalse },
+    { "Function", (long) MagickFunctionOptions, MagickFalse },
     { "Gravity", (long) MagickGravityOptions, MagickFalse },
     { "ImageList", (long) MagickImageListOptions, MagickFalse },
     { "Intent", (long) MagickIntentOptions, MagickFalse },
@@ -1038,34 +1077,34 @@ static const OptionInfo
     { "LineCap", (long) MagickLineCapOptions, MagickFalse },
     { "LineJoin", (long) MagickLineJoinOptions, MagickFalse },
     { "List", (long) MagickListOptions, MagickFalse },
+    { "Locale", (long) MagickLocaleOptions, MagickFalse },
     { "LogEvent", (long) MagickLogEventOptions, MagickFalse },
-    { "Metric", (long) MagickMetricOptions, MagickFalse },
+    { "Log", (long) MagickLogOptions, MagickFalse },
+    { "Magic", (long) MagickMagicOptions, MagickFalse },
     { "Method", (long) MagickMethodOptions, MagickFalse },
+    { "Metric", (long) MagickMetricOptions, MagickFalse },
     { "Mime", (long) MagickMimeOptions, MagickFalse },
     { "Mode", (long) MagickModeOptions, MagickFalse },
+    { "Module", (long) MagickModuleOptions, MagickFalse },
     { "Noise", (long) MagickNoiseOptions, MagickFalse },
     { "Orientation", (long) MagickOrientationOptions, MagickFalse },
+    { "Policy", (long) MagickPolicyOptions, MagickFalse },
+    { "PolicyDomain", (long) MagickPolicyDomainOptions, MagickFalse },
+    { "PolicyRights", (long) MagickPolicyRightsOptions, MagickFalse },
     { "Preview", (long) MagickPreviewOptions, MagickFalse },
     { "Primitive", (long) MagickPrimitiveOptions, MagickFalse },
     { "QuantumFormat", (long) MagickQuantumFormatOptions, MagickFalse },
-    { "Resolution", (long) MagickResolutionOptions, MagickFalse },
     { "Resource", (long) MagickResourceOptions, MagickFalse },
     { "SparseColor", (long) MagickSparseColorOptions, MagickFalse },
     { "Storage", (long) MagickStorageOptions, MagickFalse },
     { "Stretch", (long) MagickStretchOptions, MagickFalse },
     { "Style", (long) MagickStyleOptions, MagickFalse },
-    { "VirtualPixel", (long) MagickVirtualPixelOptions, MagickFalse },
-    { "Coder", (long) MagickCoderOptions, MagickFalse },
-    { "Color", (long) MagickColorOptions, MagickFalse },
-    { "Configure", (long) MagickConfigureOptions, MagickFalse },
-    { "Delegate", (long) MagickDelegateOptions, MagickFalse },
-    { "Format", (long) MagickFormatOptions, MagickFalse },
-    { "Locale", (long) MagickLocaleOptions, MagickFalse },
-    { "Log", (long) MagickLogOptions, MagickFalse },
-    { "Magic", (long) MagickMagicOptions, MagickFalse },
-    { "Module", (long) MagickModuleOptions, MagickFalse },
     { "Threshold", (long) MagickThresholdOptions, MagickFalse },
     { "Type", (long) MagickTypeOptions, MagickFalse },
+    { "Units", (long) MagickResolutionOptions, MagickFalse },
+    { "Undefined", (long) MagickUndefinedOptions, MagickTrue },
+    { "Validate", (long) MagickValidateOptions, MagickFalse },
+    { "VirtualPixel", (long) MagickVirtualPixelOptions, MagickFalse },
     { (char *) NULL, (long) MagickUndefinedOptions, MagickFalse }
   },
   LogEventOptions[] =
@@ -1083,6 +1122,7 @@ static const OptionInfo
     { "Locale", (long) LocaleEvent, MagickFalse },
     { "Module", (long) ModuleEvent, MagickFalse },
     { "None", (long) NoEvents, MagickFalse },
+    { "Policy", (long) PolicyEvent, MagickFalse },
     { "Resource", (long) ResourceEvent, MagickFalse },
     { "Trace", (long) TraceEvent, MagickFalse },
     { "Transform", (long) TransformEvent, MagickFalse },
@@ -1145,6 +1185,23 @@ static const OptionInfo
     { "RightBottom", (long) RightBottomOrientation, MagickFalse },
     { "LeftBottom", (long) LeftBottomOrientation, MagickFalse }
   },
+  PolicyDomainOptions[] =
+  {
+    { "Undefined", (long) UndefinedPolicyDomain, MagickTrue },
+    { "Coder", (long) CoderPolicyDomain, MagickFalse },
+    { "Delegate", (long) DelegatePolicyDomain, MagickFalse },
+    { "Filter", (long) FilterPolicyDomain, MagickFalse },
+    { "Path", (long) PathPolicyDomain, MagickFalse },
+    { "Resource", (long) ResourcePolicyDomain, MagickFalse }
+  },
+  PolicyRightsOptions[] =
+  {
+    { "Undefined", (long) UndefinedPolicyRights, MagickTrue },
+    { "None", (long) NoPolicyRights, MagickFalse },
+    { "Read", (long) ReadPolicyRights, MagickFalse },
+    { "Write", (long) WritePolicyRights, MagickFalse },
+    { "Execute", (long) ExecutePolicyRights, MagickFalse }
+  },
   PreviewOptions[] =
   {
     { "Undefined", (long) UndefinedPreview, MagickTrue },
@@ -1222,6 +1279,8 @@ static const OptionInfo
     { "File", (long) FileResource, MagickFalse },
     { "Map", (long) MapResource, MagickFalse },
     { "Memory", (long) MemoryResource, MagickFalse },
+    { "Thread", (long) ThreadResource, MagickFalse },
+    { "Time", (long) TimeResource, MagickFalse },
     { (char *) NULL, (long) UndefinedResource, MagickFalse }
   },
   SparseColorOptions[] =
@@ -1285,21 +1344,40 @@ static const OptionInfo
     { "TrueColor", (long) TrueColorType, MagickFalse },
     { (char *) NULL, (long) UndefinedType, MagickFalse }
   },
+  ValidateOptions[] =
+  {
+    { "Undefined", (long) UndefinedValidate, MagickTrue },
+    { "All", (long) AllValidate, MagickFalse },
+    { "Compare", (long) CompareValidate, MagickFalse },
+    { "Composite", (long) CompositeValidate, MagickFalse },
+    { "Convert", (long) ConvertValidate, MagickFalse },
+    { "FormatsInMemory", (long) FormatsInMemoryValidate, MagickFalse },
+    { "FormatsOnDisk", (long) FormatsOnDiskValidate, MagickFalse },
+    { "Identify", (long) IdentifyValidate, MagickFalse },
+    { "ImportExport", (long) ImportExportValidate, MagickFalse },
+    { "Montage", (long) MontageValidate, MagickFalse },
+    { "Stream", (long) StreamValidate, MagickFalse },
+    { "None", (long) NoValidate, MagickFalse },
+    { (char *) NULL, (long) UndefinedValidate, MagickFalse }
+  },
   VirtualPixelOptions[] =
   {
     { "Undefined", (long) UndefinedVirtualPixelMethod, MagickTrue },
     { "Background", (long) BackgroundVirtualPixelMethod, MagickFalse },
     { "Black", (long) BlackVirtualPixelMethod, MagickFalse },
-    { "Constant", (long) BackgroundVirtualPixelMethod, MagickTrue }, /* depreciated */
+    { "Constant", (long) BackgroundVirtualPixelMethod, MagickTrue }, /* deprecated */
+    { "CheckerTile", (long) CheckerTileVirtualPixelMethod, MagickFalse },
     { "Dither", (long) DitherVirtualPixelMethod, MagickFalse },
     { "Edge", (long) EdgeVirtualPixelMethod, MagickFalse },
     { "Gray", (long) GrayVirtualPixelMethod, MagickFalse },
     { "HorizontalTile", (long) HorizontalTileVirtualPixelMethod, MagickFalse },
+    { "HorizontalTileEdge", (long) HorizontalTileEdgeVirtualPixelMethod, MagickFalse },
     { "Mirror", (long) MirrorVirtualPixelMethod, MagickFalse },
     { "Random", (long) RandomVirtualPixelMethod, MagickFalse },
     { "Tile", (long) TileVirtualPixelMethod, MagickFalse },
     { "Transparent", (long) TransparentVirtualPixelMethod, MagickFalse },
     { "VerticalTile", (long) VerticalTileVirtualPixelMethod, MagickFalse },
+    { "VerticalTileEdge", (long) VerticalTileEdgeVirtualPixelMethod, MagickFalse },
     { "White", (long) WhiteVirtualPixelMethod, MagickFalse },
     { (char *) NULL, (long) UndefinedVirtualPixelMethod, MagickFalse }
   };
@@ -1554,6 +1632,7 @@ static const OptionInfo *GetOptionInfo(const MagickOption option)
     case MagickEvaluateOptions: return(EvaluateOptions);
     case MagickFillRuleOptions: return(FillRuleOptions);
     case MagickFilterOptions: return(FilterOptions);
+    case MagickFunctionOptions: return(FunctionOptions);
     case MagickGravityOptions: return(GravityOptions);
     case MagickImageListOptions: return(ImageListOptions);
     case MagickIntentOptions: return(IntentOptions);
@@ -1569,6 +1648,8 @@ static const OptionInfo *GetOptionInfo(const MagickOption option)
     case MagickModeOptions: return(ModeOptions);
     case MagickNoiseOptions: return(NoiseOptions);
     case MagickOrientationOptions: return(OrientationOptions);
+    case MagickPolicyDomainOptions: return(PolicyDomainOptions);
+    case MagickPolicyRightsOptions: return(PolicyRightsOptions);
     case MagickPreviewOptions: return(PreviewOptions);
     case MagickPrimitiveOptions: return(PrimitiveOptions);
     case MagickQuantumFormatOptions: return(QuantumFormatOptions);
@@ -1579,6 +1660,7 @@ static const OptionInfo *GetOptionInfo(const MagickOption option)
     case MagickStretchOptions: return(StretchOptions);
     case MagickStyleOptions: return(StyleOptions);
     case MagickTypeOptions: return(TypeOptions);
+    case MagickValidateOptions: return(ValidateOptions);
     case MagickVirtualPixelOptions: return(VirtualPixelOptions);
     default: break;
   }
@@ -1903,7 +1985,7 @@ MagickExport long ParseChannelOption(const char *channels)
 %    o option: Index to the option table to lookup
 %
 %    o list: A option other than zero permits more than one option separated by
-%      commas.
+%      a comma or pipe.
 %
 %    o options: One or more options separated by commas.
 %
@@ -1917,6 +1999,9 @@ MagickExport long ParseMagickOption(const MagickOption option,
   const OptionInfo
     *option_info;
 
+  int
+    sentinel;
+
   long
     option_types;
 
@@ -1936,16 +2021,19 @@ MagickExport long ParseMagickOption(const MagickOption option,
   if (option_info == (const OptionInfo *) NULL)
     return(-1);
   option_types=0;
-  for (p=options; p != (char *) NULL; p=strchr(p,','))
+  sentinel=',';
+  if (strchr(options,'|') != (char *) NULL)
+    sentinel='|';
+  for (p=options; p != (char *) NULL; p=strchr(p,sentinel))
   {
-    while (((isspace((int) ((unsigned char) *p)) != 0) || (*p == ',')) &&
+    while (((isspace((int) ((unsigned char) *p)) != 0) || (*p == sentinel)) &&
            (*p != '\0'))
       p++;
     negate=(*p == '!') ? MagickTrue : MagickFalse;
     if (negate != MagickFalse)
       p++;
     q=token;
-    while (((isspace((int) ((unsigned char) *p)) == 0) && (*p != ',')) &&
+    while (((isspace((int) ((unsigned char) *p)) == 0) && (*p != sentinel)) &&
            (*p != '\0'))
     {
       if ((q-token) >= MaxTextExtent)
@@ -2034,6 +2122,41 @@ MagickExport char *RemoveImageOption(ImageInfo *image_info,const char *option)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   R e s e t I m a g e O p t i o n                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  ResetImageOptions() resets the image_info option.  That is, it deletes
+%  all options associated with the image_info structure.
+%
+%  The format of the ResetImageOptions method is:
+%
+%      ResetImageOptions(ImageInfo *image_info)
+%
+%  A description of each parameter follows:
+%
+%    o image_info: the image info.
+%
+*/
+MagickExport void ResetImageOptions(const ImageInfo *image_info)
+{
+  assert(image_info != (ImageInfo *) NULL);
+  assert(image_info->signature == MagickSignature);
+  if (image_info->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      image_info->filename);
+  if (image_info->options == (void *) NULL)
+    return;
+  ResetSplayTree((SplayTreeInfo *) image_info->options);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   R e s e t I m a g e O p t i o n I t e r a t o r                           %
 %                                                                             %
 %                                                                             %
