@@ -89,7 +89,7 @@
 %
 %  The format of the ExportQuantumPixels method is:
 %
-%      size_t ExportQuantumPixels(const Image *image,const ViewInfo *cache_view,
+%      size_t ExportQuantumPixels(const Image *image,const CacheView *image_view,
 %        const QuantumInfo *quantum_info,const QuantumType quantum_type,
 %        unsigned char *pixels,ExceptionInfo *exception)
 %
@@ -97,7 +97,7 @@
 %
 %    o image: the image.
 %
-%    o cache_view: the image cache view.
+%    o image_view: the image cache view.
 %
 %    o quantum_info: the quantum info.
 %
@@ -113,11 +113,14 @@
 static inline unsigned char *PopDoublePixel(const QuantumState *quantum_state,
   const double pixel,unsigned char *pixels)
 {
+  double
+    *p;
+
   unsigned char
     quantum[8];
 
-  *((double *) quantum)=(double) (pixel*quantum_state->inverse_scale+
-    quantum_state->minimum); 
+  p=(double *) quantum;
+  *p=(double) (pixel*quantum_state->inverse_scale+quantum_state->minimum); 
   if (quantum_state->endian != LSBEndian)
     {
       *pixels++=quantum[7];
@@ -144,10 +147,14 @@ static inline unsigned char *PopDoublePixel(const QuantumState *quantum_state,
 static inline unsigned char *PopFloatPixel(const QuantumState *quantum_state,
   const float pixel,unsigned char *pixels)
 {
+  float
+    *p;
+
   unsigned char
     quantum[4];
 
-  *((float *) quantum)=(float) ((double) pixel*quantum_state->inverse_scale+
+  p=(float *) quantum;
+  *p=(float) ((double) pixel*quantum_state->inverse_scale+
     quantum_state->minimum); 
   if (quantum_state->endian != LSBEndian)
     {
@@ -226,7 +233,7 @@ static inline unsigned char *PopQuantumLongPixel(QuantumState *quantum_state,
 }
 
 MagickExport size_t ExportQuantumPixels(const Image *image,
-  const ViewInfo *cache_view,const QuantumInfo *quantum_info,
+  const CacheView *image_view,const QuantumInfo *quantum_info,
   const QuantumType quantum_type,unsigned char *pixels,ExceptionInfo *exception)
 {
   EndianType
@@ -242,7 +249,7 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
     number_pixels;
 
   QuantumAny
-    scale;
+    range;
 
   QuantumState
     quantum_state;
@@ -269,27 +276,27 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
   assert(quantum_info != (QuantumInfo *) NULL);
   assert(quantum_info->signature == MagickSignature);
   if (pixels == (unsigned char *) NULL)
-    pixels=quantum_info->pixels;
+    pixels=GetQuantumPixels(quantum_info);
   number_pixels=GetImageExtent(image);
   p=GetVirtualPixelQueue(image);
   indexes=GetVirtualIndexQueue(image);
-  if (cache_view != (ViewInfo *) NULL)
+  if (image_view != (CacheView *) NULL)
     {
-      number_pixels=GetCacheViewExtent(cache_view);
-      p=GetCacheViewVirtualPixelQueue(cache_view);
-      indexes=GetCacheViewVirtualIndexQueue(cache_view);
+      number_pixels=GetCacheViewExtent(image_view);
+      p=GetCacheViewVirtualPixelQueue(image_view);
+      indexes=GetCacheViewVirtualIndexQueue(image_view);
     }
   if (quantum_info->alpha_type == AssociatedQuantumAlpha)
     {
       register PixelPacket
-        *q;
+        *__restrict q;
 
       /*
         Associate alpha.
       */
       q=GetAuthenticPixelQueue(image);
-      if (cache_view != (ViewInfo *) NULL)
-        q=GetAuthenticPixelQueue(image);
+      if (image_view != (CacheView *) NULL)
+        q=(PixelPacket *) GetCacheViewVirtualPixelQueue(image_view);
       for (x=0; x < (long) image->columns; x++)
       {
         alpha=QuantumScale*((double) QuantumRange-q->opacity);
@@ -299,6 +306,25 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         q++;
       }
     }
+  if ((quantum_type == CbYCrQuantum) || (quantum_type == CbYCrAQuantum))
+    {
+      Quantum
+        quantum;
+
+      register PixelPacket
+        *__restrict q;
+
+      q=GetAuthenticPixelQueue(image);
+      if (image_view != (CacheView *) NULL)
+        q=GetAuthenticPixelQueue(image);
+      for (x=0; x < (long) number_pixels; x++)
+      {
+        quantum=q->red;
+        q->red=q->green;
+        q->green=quantum;
+        q++;
+      }
+    }
   x=0;
   q=pixels;
   InitializeQuantumState(quantum_info,image->endian,&quantum_state);
@@ -593,12 +619,12 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,indexes[x],q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              (Quantum) (QuantumRange-p->opacity),image->depth,scale),q);
+              (Quantum) (QuantumRange-p->opacity),range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -705,14 +731,14 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
           register unsigned short
             pixel;
 
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           if (quantum_info->pack == MagickFalse)
             {
               for (x=0; x < (long) number_pixels; x++)
               {
                 pixel=ScaleQuantumToShort(PixelIntensityToQuantum(p));
                 q=PopShortPixel(endian,(unsigned short) ScaleQuantumToAny(
-                  (Quantum) pixel,image->depth,scale),q);
+                  (Quantum) pixel,range),q);
                 p++;
                 q+=quantum_info->pad;
               }
@@ -721,7 +747,7 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              PixelIntensityToQuantum(p),image->depth,scale),q);
+              PixelIntensityToQuantum(p),range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -732,7 +758,7 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
           register unsigned short
             pixel;
 
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           if (quantum_info->pack == MagickFalse)
             {
               for (x=0; x < (long) number_pixels; x++)
@@ -747,7 +773,7 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              PixelIntensityToQuantum(p),image->depth,scale),q);
+              PixelIntensityToQuantum(p),range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -814,11 +840,11 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              PixelIntensityToQuantum(p),image->depth,scale),q);
+              PixelIntensityToQuantum(p),range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -833,28 +859,38 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
       {
         case 1:
         {
-          register unsigned char
-            pixel;
+          register Quantum
+            threshold;
 
+          register unsigned char
+            black,
+            pixel,
+            white;
+
+          black=0x00;
+          white=0x01;
+          if (quantum_info->min_is_white == MagickFalse)
+            {
+              black=0x01;
+              white=0x00;
+            }
+          threshold=(Quantum) (QuantumRange/2);
           for (x=((long) number_pixels-3); x > 0; x-=4)
           {
-            pixel=ScaleQuantumToChar(PixelIntensityToQuantum(p));
-            *q=(unsigned char) (((int) pixel != 0 ? 0x00 : 0x01) << 7);
+            *q='\0';
+            *q|=(PixelIntensityToQuantum(p) < threshold ? black : white) << 7;
             pixel=(unsigned char) (p->opacity == OpaqueOpacity ? 0x00 : 0x01);
             *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 6);
             p++;
-            pixel=ScaleQuantumToChar(PixelIntensityToQuantum(p));
-            *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 5);
+            *q|=(PixelIntensityToQuantum(p) < threshold ? black : white) << 5;
             pixel=(unsigned char) (p->opacity == OpaqueOpacity ? 0x00 : 0x01);
             *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 4);
             p++;
-            pixel=ScaleQuantumToChar(PixelIntensityToQuantum(p));
-            *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 3);
+            *q|=(PixelIntensityToQuantum(p) < threshold ? black : white) << 3;
             pixel=(unsigned char) (p->opacity == OpaqueOpacity ? 0x00 : 0x01);
             *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 2);
             p++;
-            pixel=ScaleQuantumToChar(PixelIntensityToQuantum(p));
-            *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 1);
+            *q|=(PixelIntensityToQuantum(p) < threshold ? black : white) << 1;
             pixel=(unsigned char) (p->opacity == OpaqueOpacity ? 0x00 : 0x01);
             *q|=(((int) pixel != 0 ? 0x00 : 0x01) << 0);
             p++;
@@ -865,12 +901,12 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
               *q='\0';
               for (bit=3; bit >= (long) (4-(number_pixels % 4)); bit-=2)
               {
-                pixel=ScaleQuantumToChar(PixelIntensityToQuantum(p));
-                *q|=(((int) pixel != 0 ? 0x00 : 0x01) << (unsigned char) bit);
-                pixel=(unsigned char) (p->opacity == OpaqueOpacity ?
-                  0x00 : 0x01);
-                *q|=(((int) pixel != 0 ? 0x00 : 0x01) <<
-                  (unsigned char) (bit-1));
+                *q|=(PixelIntensityToQuantum(p) < threshold ? black : white) <<
+                  bit;
+                pixel=(unsigned char) (p->opacity == OpaqueOpacity ? 0x00 :
+                  0x01);
+                *q|=(((int) pixel != 0 ? 0x00 : 0x01) << (unsigned char)
+                  (bit-1));
                 p++;
               }
               q++;
@@ -979,13 +1015,13 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              PixelIntensityToQuantum(p),image->depth,scale),q);
+              PixelIntensityToQuantum(p),range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              (Quantum) (QuantumRange-p->opacity),image->depth,scale),q);
+              (Quantum) (QuantumRange-p->opacity),range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -1066,11 +1102,11 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->red,image->depth,scale),q);
+              p->red,range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -1151,11 +1187,11 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->green,image->depth,scale),q);
+              p->green,range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -1236,11 +1272,11 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->blue,image->depth,scale),q);
+              p->blue,range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -1328,11 +1364,95 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              (Quantum) (QuantumRange-p->opacity),image->depth,scale),q);
+              (Quantum) (QuantumRange-p->opacity),range),q);
+            p++;
+            q+=quantum_info->pad;
+          }
+          break;
+        }
+      }
+      break;
+    }
+    case OpacityQuantum:
+    {
+      switch (quantum_info->depth)
+      {
+        case 8:
+        {
+          register unsigned char
+            pixel;
+
+          for (x=0; x < (long) number_pixels; x++)
+          {
+            pixel=ScaleQuantumToChar(p->opacity);
+            q=PopCharPixel(pixel,q);
+            p++;
+            q+=quantum_info->pad;
+          }
+          break;
+        }
+        case 16:
+        {
+          register unsigned short
+            pixel;
+
+          for (x=0; x < (long) number_pixels; x++)
+          {
+            pixel=ScaleQuantumToShort(p->opacity);
+            q=PopShortPixel(endian,pixel,q);
+            p++;
+            q+=quantum_info->pad;
+          }
+          break;
+        }
+        case 32:
+        {
+          register unsigned long
+            pixel;
+
+          if (quantum_info->format == FloatingPointQuantumFormat)
+            {
+              for (x=0; x < (long) number_pixels; x++)
+              {
+                q=PopFloatPixel(&quantum_state,(float) p->opacity,q);
+                p++;
+                q+=quantum_info->pad;
+              }
+              break;
+            }
+          for (x=0; x < (long) number_pixels; x++)
+          {
+            pixel=ScaleQuantumToLong(p->opacity);
+            q=PopLongPixel(endian,pixel,q);
+            p++;
+            q+=quantum_info->pad;
+          }
+          break;
+        }
+        case 64:
+        {
+          if (quantum_info->format == FloatingPointQuantumFormat)
+            {
+              for (x=0; x < (long) number_pixels; x++)
+              {
+                q=PopDoublePixel(&quantum_state,(double) p->opacity,q);
+                p++;
+                q+=quantum_info->pad;
+              }
+              break;
+            }
+        }
+        default:
+        {
+          range=GetQuantumRange(image->depth);
+          for (x=0; x < (long) number_pixels; x++)
+          {
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              p->opacity,range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -1418,11 +1538,11 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              (Quantum) indexes[x],image->depth,scale),q);
+              (Quantum) indexes[x],range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -1432,6 +1552,7 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
       break;
     }
     case RGBQuantum:
+    case CbYCrQuantum:
     {
       switch (quantum_info->depth)
       {
@@ -1452,14 +1573,14 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
           register unsigned long
             pixel;
 
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           if (quantum_info->pack == MagickFalse)
             {
               for (x=0; x < (long) number_pixels; x++)
               {
-                pixel=ScaleQuantumToAny(p->red,image->depth,scale) << 22 |
-                  ScaleQuantumToAny(p->green,image->depth,scale) <<  12 |
-                  ScaleQuantumToAny(p->blue,image->depth,scale) << 2;
+                pixel=(unsigned long) (ScaleQuantumToAny(p->red,range) << 22 |
+                  ScaleQuantumToAny(p->green,range) <<  12 |
+                  ScaleQuantumToAny(p->blue,range) << 2);
                 q=PopLongPixel(endian,pixel,q);
                 p++;
                 q+=quantum_info->pad;
@@ -1470,11 +1591,11 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
             {
               for (x=0; x < (long) number_pixels; x++)
               {
-                pixel=ScaleQuantumToAny(p->red,image->depth,scale);
+                pixel=(unsigned long) ScaleQuantumToAny(p->red,range);
                 q=PopQuantumLongPixel(&quantum_state,image->depth,pixel,q);
-                pixel=ScaleQuantumToAny(p->green,image->depth,scale);
+                pixel=(unsigned long) ScaleQuantumToAny(p->green,range);
                 q=PopQuantumLongPixel(&quantum_state,image->depth,pixel,q);
-                pixel=ScaleQuantumToAny(p->blue,image->depth,scale);
+                pixel=(unsigned long) ScaleQuantumToAny(p->blue,range);
                 q=PopQuantumLongPixel(&quantum_state,image->depth,pixel,q);
                 p++;
                 q+=quantum_info->pad;
@@ -1483,11 +1604,11 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
             }
           for (x=0; x < (long) number_pixels; x++)
           {
-            pixel=ScaleQuantumToAny(p->red,image->depth,scale);
+            pixel=(unsigned long) ScaleQuantumToAny(p->red,range);
             q=PopQuantumPixel(&quantum_state,image->depth,pixel,q);
-            pixel=ScaleQuantumToAny(p->green,image->depth,scale);
+            pixel=(unsigned long) ScaleQuantumToAny(p->green,range);
             q=PopQuantumPixel(&quantum_state,image->depth,pixel,q);
-            pixel=ScaleQuantumToAny(p->blue,image->depth,scale);
+            pixel=(unsigned long) ScaleQuantumToAny(p->blue,range);
             q=PopQuantumPixel(&quantum_state,image->depth,pixel,q);
             p++;
             q+=quantum_info->pad;
@@ -1499,7 +1620,7 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
           register unsigned long
             pixel;
 
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           if (quantum_info->pack == MagickFalse)
             {
               for (x=0; x < (long) (3*number_pixels-1); x+=2)
@@ -1509,17 +1630,17 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
                   default:
                   case 0:
                   {
-                    pixel=ScaleQuantumToAny(p->red,image->depth,scale);
+                    pixel=(unsigned long) ScaleQuantumToAny(p->red,range);
                     break;
                   }
                   case 1:
                   {
-                    pixel=ScaleQuantumToAny(p->green,image->depth,scale);
+                    pixel=(unsigned long) ScaleQuantumToAny(p->green,range);
                     break;
                   }
                   case 2:
                   {
-                    pixel=ScaleQuantumToAny(p->blue,image->depth,scale);
+                    pixel=(unsigned long) ScaleQuantumToAny(p->blue,range);
                     p++;
                     break;
                   }
@@ -1530,17 +1651,17 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
                   default:
                   case 0:
                   {
-                    pixel=ScaleQuantumToAny(p->red,image->depth,scale);
+                    pixel=(unsigned long) ScaleQuantumToAny(p->red,range);
                     break;
                   }
                   case 1:
                   {
-                    pixel=ScaleQuantumToAny(p->green,image->depth,scale);
+                    pixel=(unsigned long) ScaleQuantumToAny(p->green,range);
                     break;
                   }
                   case 2:
                   {
-                    pixel=ScaleQuantumToAny(p->blue,image->depth,scale);
+                    pixel=(unsigned long) ScaleQuantumToAny(p->blue,range);
                     p++;
                     break;
                   }
@@ -1555,17 +1676,17 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
                   default:
                   case 0:
                   {
-                    pixel=ScaleQuantumToAny(p->red,image->depth,scale);
+                    pixel=(unsigned long) ScaleQuantumToAny(p->red,range);
                     break;
                   }
                   case 1:
                   {
-                    pixel=ScaleQuantumToAny(p->green,image->depth,scale);
+                    pixel=(unsigned long) ScaleQuantumToAny(p->green,range);
                     break;
                   }
                   case 2:
                   {
-                    pixel=ScaleQuantumToAny(p->blue,image->depth,scale);
+                    pixel=(unsigned long) ScaleQuantumToAny(p->blue,range);
                     p++;
                     break;
                   }
@@ -1581,11 +1702,11 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
             {
               for (x=0; x < (long) number_pixels; x++)
               {
-                pixel=ScaleQuantumToAny(p->red,image->depth,scale);
+                pixel=(unsigned long) ScaleQuantumToAny(p->red,range);
                 q=PopQuantumLongPixel(&quantum_state,image->depth,pixel,q);
-                pixel=ScaleQuantumToAny(p->green,image->depth,scale);
+                pixel=(unsigned long) ScaleQuantumToAny(p->green,range);
                 q=PopQuantumLongPixel(&quantum_state,image->depth,pixel,q);
-                pixel=ScaleQuantumToAny(p->blue,image->depth,scale);
+                pixel=(unsigned long) ScaleQuantumToAny(p->blue,range);
                 q=PopQuantumLongPixel(&quantum_state,image->depth,pixel,q);
                 p++;
                 q+=quantum_info->pad;
@@ -1594,11 +1715,11 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
             }
           for (x=0; x < (long) number_pixels; x++)
           {
-            pixel=ScaleQuantumToAny(p->red,image->depth,scale);
+            pixel=(unsigned long) ScaleQuantumToAny(p->red,range);
             q=PopQuantumPixel(&quantum_state,image->depth,pixel,q);
-            pixel=ScaleQuantumToAny(p->green,image->depth,scale);
+            pixel=(unsigned long) ScaleQuantumToAny(p->green,range);
             q=PopQuantumPixel(&quantum_state,image->depth,pixel,q);
-            pixel=ScaleQuantumToAny(p->blue,image->depth,scale);
+            pixel=(unsigned long) ScaleQuantumToAny(p->blue,range);
             q=PopQuantumPixel(&quantum_state,image->depth,pixel,q);
             p++;
             q+=quantum_info->pad;
@@ -1670,15 +1791,15 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->red,image->depth,scale),q);
+              p->red,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->green,image->depth,scale),q);
+              p->green,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->blue,image->depth,scale),q);
+              p->blue,range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -1688,6 +1809,7 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
       break;
     }
     case RGBAQuantum:
+    case CbYCrAQuantum:
     {
       switch (quantum_info->depth)
       {
@@ -1790,17 +1912,17 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->red,image->depth,scale),q);
+              p->red,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->green,image->depth,scale),q);
+              p->green,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->blue,image->depth,scale),q);
+              p->blue,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              (Quantum) (QuantumRange-p->opacity),image->depth,scale),q);
+              (Quantum) (QuantumRange-p->opacity),range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -1904,17 +2026,17 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->red,image->depth,scale),q);
+              p->red,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->green,image->depth,scale),q);
+              p->green,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->blue,image->depth,scale),q);
+              p->blue,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->opacity,image->depth,scale),q);
+              p->opacity,range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -2024,17 +2146,17 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->red,image->depth,scale),q);
+              p->red,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->green,image->depth,scale),q);
+              p->green,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->blue,image->depth,scale),q);
+              p->blue,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              indexes[x],image->depth,scale),q);
+              indexes[x],range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -2160,19 +2282,134 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->red,image->depth,scale),q);
+              p->red,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->green,image->depth,scale),q);
+              p->green,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->blue,image->depth,scale),q);
+              p->blue,range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              indexes[x],image->depth,scale),q);
+              indexes[x],range),q);
             q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
-              p->opacity,image->depth,scale),q);
+              p->opacity,range),q);
+            p++;
+            q+=quantum_info->pad;
+          }
+          break;
+        }
+      }
+      break;
+    }
+    case CbYCrYQuantum:
+    {
+     long
+        n;
+
+      Quantum
+        cbcr[4];
+
+      register long
+        i;
+
+      register unsigned long
+        pixel;
+
+      unsigned long
+        quantum;
+
+      n=0;
+      quantum=0;
+      range=GetQuantumRange(image->depth);
+      switch (quantum_info->depth)
+      {
+        case 10:
+        {
+          if (quantum_info->pack == MagickFalse)
+            {
+              for (x=0; x < (long) number_pixels; x+=2)
+              {
+                for (i=0; i < 4; i++)
+                {
+                  switch (n % 3)
+                  {
+                    case 0:
+                    {
+                      quantum=p->red;
+                      break;
+                    }
+                    case 1:
+                    {
+                      quantum=p->green;
+                      break;
+                    }
+                    case 2:
+                    {
+                      quantum=p->blue;
+                      break;
+                    }
+                  }
+                  cbcr[i]=quantum;
+                  n++;
+                }
+                pixel=(unsigned long) ((unsigned long) (cbcr[1]) << 22 |
+                  (unsigned long) (cbcr[0]) <<  12 |
+                  (unsigned long) (cbcr[2]) << 2);
+                q=PopLongPixel(endian,pixel,q);
+                p++;
+                pixel=(unsigned long) ((unsigned long) (cbcr[3]) << 22 |
+                  (unsigned long) (cbcr[0]) <<  12 |
+                  (unsigned long) (cbcr[2]) << 2);
+                q=PopLongPixel(endian,pixel,q);
+                p++;
+                q+=quantum_info->pad;
+              }
+              break;
+            }
+          break;
+        }
+        default:
+        {
+          for (x=0; x < (long) number_pixels; x+=2)
+          {
+            for (i=0; i < 4; i++)
+            {
+              switch (n % 3)
+              {
+                case 0:
+                {
+                  quantum=p->red;
+                  break;
+                }
+                case 1:
+                {
+                  quantum=p->green;
+                  break;
+                }
+                case 2:
+                {
+                  quantum=p->blue;
+                  break;
+                }
+              }
+              cbcr[i]=quantum;
+              n++;
+            }
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              cbcr[1],range),q);
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              cbcr[0],range),q);
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              cbcr[2],range),q);
+            p++;
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              cbcr[3],range),q);
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              cbcr[0],range),q);
+            q=PopQuantumPixel(&quantum_state,image->depth,ScaleQuantumToAny(
+              cbcr[2],range),q);
             p++;
             q+=quantum_info->pad;
           }
@@ -2184,5 +2421,24 @@ MagickExport size_t ExportQuantumPixels(const Image *image,
     default:
       break;
   }
+  if ((quantum_type == CbYCrQuantum) || (quantum_type == CbYCrAQuantum))
+    {
+      Quantum
+        quantum;
+
+      register PixelPacket
+        *__restrict q;
+
+      q=GetAuthenticPixelQueue(image);
+      if (image_view != (CacheView *) NULL)
+        q=(PixelPacket *) GetCacheViewVirtualPixelQueue(image_view);
+      for (x=0; x < (long) number_pixels; x++)
+      {
+        quantum=q->red;
+        q->red=q->green;
+        q->green=quantum;
+        q++;
+      }
+    }
   return(extent);
 }
