@@ -89,7 +89,7 @@
 %
 %  The format of the ImportQuantumPixels method is:
 %
-%      size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
+%      size_t ImportQuantumPixels(Image *image,CacheView *image_view,
 %        const QuantumInfo *quantum_info,const QuantumType quantum_type,
 %        const unsigned char *pixels,ExceptionInfo *exception)
 %
@@ -122,6 +122,9 @@ static inline IndexPacket PushColormapIndex(Image *image,
 static inline const unsigned char *PushDoublePixel(
   const QuantumState *quantum_state,const unsigned char *pixels,double *pixel)
 {
+  double
+    *p;
+
   unsigned char
     quantum[8];
 
@@ -135,7 +138,8 @@ static inline const unsigned char *PushDoublePixel(
       quantum[2]=(*pixels++);
       quantum[1]=(*pixels++);
       quantum[0]=(*pixels++);
-      *pixel=(*((double *) quantum));
+      p=(double *) quantum;
+      *pixel=(*p);
       *pixel-=quantum_state->minimum;
       *pixel*=quantum_state->scale;
       return(pixels);
@@ -148,7 +152,8 @@ static inline const unsigned char *PushDoublePixel(
   quantum[5]=(*pixels++);
   quantum[6]=(*pixels++);
   quantum[7]=(*pixels++);
-  *pixel=(*((double *) quantum));
+  p=(double *) quantum;
+  *pixel=(*p);
   *pixel-=quantum_state->minimum;
   *pixel*=quantum_state->scale;
   return(pixels);
@@ -157,6 +162,9 @@ static inline const unsigned char *PushDoublePixel(
 static inline const unsigned char *PushFloatPixel(
   const QuantumState *quantum_state,const unsigned char *pixels,float *pixel)
 {
+  float
+    *p;
+
   unsigned char
     quantum[4];
 
@@ -166,7 +174,8 @@ static inline const unsigned char *PushFloatPixel(
       quantum[2]=(*pixels++);
       quantum[1]=(*pixels++);
       quantum[0]=(*pixels++);
-      *pixel=(*((float *) quantum));
+      p=(float *) quantum;
+      *pixel=(*p);
       *pixel-=quantum_state->minimum;
       *pixel*=quantum_state->scale;
       return(pixels);
@@ -175,14 +184,16 @@ static inline const unsigned char *PushFloatPixel(
   quantum[1]=(*pixels++);
   quantum[2]=(*pixels++);
   quantum[3]=(*pixels++);
-  *pixel=(*((float *) quantum));
+  p=(float *) quantum;
+  *pixel=(*p);
   *pixel-=quantum_state->minimum;
   *pixel*=quantum_state->scale;
   return(pixels);
 }
 
-static inline const unsigned char *PushQuantumPixel(QuantumState *quantum_state,
-  const unsigned long depth,const unsigned char *pixels,QuantumAny *quantum)
+static inline const unsigned char *PushQuantumPixel(
+  QuantumState *quantum_state,const unsigned long depth,
+  const unsigned char *pixels,unsigned long *quantum)
 {
   register long
     i;
@@ -239,7 +250,7 @@ static inline const unsigned char *PushQuantumLongPixel(
   return(pixels);
 }
 
-MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
+MagickExport size_t ImportQuantumPixels(Image *image,CacheView *image_view,
   const QuantumInfo *quantum_info,const QuantumType quantum_type,
   const unsigned char *pixels,ExceptionInfo *exception)
 {
@@ -253,8 +264,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
     number_pixels;
 
   QuantumAny
-    pixel,
-    scale;
+    range;
 
   QuantumState
     quantum_state;
@@ -269,11 +279,14 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
     x;
 
   register PixelPacket
-    *q;
+    *__restrict q;
 
   size_t
     extent;
 
+  unsigned long
+    pixel;
+
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
@@ -287,7 +300,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
   number_pixels=GetImageExtent(image);
   q=GetAuthenticPixelQueue(image);
   indexes=GetAuthenticIndexQueue(image);
-  if (image_view != (ViewInfo *) NULL)
+  if (image_view != (CacheView *) NULL)
     {
       number_pixels=GetCacheViewExtent(image_view);
       q=GetCacheViewAuthenticPixelQueue(image_view);
@@ -532,15 +545,14 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
           register unsigned char
             pixel;
 
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             pixel=(unsigned char) ((*p >> 4) & 0xf);
             indexes[x]=PushColormapIndex(image,pixel,&range_exception);
             *q=image->colormap[(long) indexes[x]];
             pixel=(unsigned char) ((*p) & 0xf);
-            q->opacity=(Quantum) (QuantumRange-
-              ScaleAnyToQuantum(pixel,image->depth,scale));
+            q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel,range));
             p++;
             q++;
           }
@@ -638,15 +650,14 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
             indexes[x]=PushColormapIndex(image,pixel,&range_exception);
             *q=image->colormap[(long) indexes[x]];
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel,
-              image->depth,scale));
+            q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel,range));
             p+=quantum_info->pad;
             q++;
           }
@@ -702,16 +713,16 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
           register unsigned char
             pixel;
 
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < ((long) number_pixels-1); x+=2)
           {
             pixel=(unsigned char) ((*p >> 4) & 0xf);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             q->green=q->red;
             q->blue=q->red;
             q++;
             pixel=(unsigned char) ((*p) & 0xf);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             q->green=q->red;
             q->blue=q->red;
             p++;
@@ -720,7 +731,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
           for (bit=0; bit < (long) (number_pixels % 2); bit++)
           {
             pixel=(unsigned char) (*p++ >> 4);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             q->green=q->red;
             q->blue=q->red;
             q++;
@@ -740,6 +751,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
                 q->red=(Quantum) (QuantumRange-ScaleCharToQuantum(pixel));
                 q->green=q->red;
                 q->blue=q->red;
+                q->opacity=OpaqueOpacity;
                 p+=quantum_info->pad;
                 q++;
               }
@@ -751,6 +763,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
             q->red=ScaleCharToQuantum(pixel);
             q->green=q->red;
             q->blue=q->red;
+            q->opacity=OpaqueOpacity;
             p+=quantum_info->pad;
             q++;
           }
@@ -758,7 +771,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         case 10:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           if (quantum_info->pack == MagickFalse)
             {
               if (image->endian != LSBEndian)
@@ -766,18 +779,15 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
                   for (x=0; x < (long) number_pixels/3; x++)
                   {
                     p=PushLongPixel(endian,p,&pixel);
-                    q->red=ScaleAnyToQuantum((pixel >> 0) & 0x3ff,image->depth,
-                      scale);
+                    q->red=ScaleAnyToQuantum((pixel >> 0) & 0x3ff,range);
                     q->green=q->red;
                     q->blue=q->red;
                     q++;
-                    q->red=ScaleAnyToQuantum((pixel >> 10) & 0x3ff,image->depth,
-                      scale);
+                    q->red=ScaleAnyToQuantum((pixel >> 10) & 0x3ff,range);
                     q->green=q->red;
                     q->blue=q->red;
                     q++;
-                    q->red=ScaleAnyToQuantum((pixel >> 20) & 0x3ff,image->depth,
-                      scale);
+                    q->red=ScaleAnyToQuantum((pixel >> 20) & 0x3ff,range);
                     q->green=q->red;
                     q->blue=q->red;
                     p+=quantum_info->pad;
@@ -788,18 +798,15 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
               for (x=0; x < (long) number_pixels/3; x++)
               {
                 p=PushLongPixel(endian,p,&pixel);
-                q->red=ScaleAnyToQuantum((pixel >> 22) & 0x3ff,image->depth,
-                  scale);
+                q->red=ScaleAnyToQuantum((pixel >> 22) & 0x3ff,range);
                 q->green=q->red;
                 q->blue=q->red;
                 q++;
-                q->red=ScaleAnyToQuantum((pixel >> 12) & 0x3ff,image->depth,
-                  scale);
+                q->red=ScaleAnyToQuantum((pixel >> 12) & 0x3ff,range);
                 q->green=q->red;
                 q->blue=q->red;
                 q++;
-                q->red=ScaleAnyToQuantum((pixel >> 2) & 0x3ff,image->depth,
-                  scale);
+                q->red=ScaleAnyToQuantum((pixel >> 2) & 0x3ff,range);
                 q->green=q->red;
                 q->blue=q->red;
                 p+=quantum_info->pad;
@@ -810,7 +817,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             q->green=q->red;
             q->blue=q->red;
             p+=quantum_info->pad;
@@ -820,7 +827,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         case 12:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           if (quantum_info->pack == MagickFalse)
             {
               unsigned short
@@ -829,14 +836,12 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
               for (x=0; x < (long) (number_pixels-1); x+=2)
               {
                 p=PushShortPixel(endian,p,&pixel);
-                q->red=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),image->depth,
-                  scale);
+                q->red=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),range);
                 q->green=q->red;
                 q->blue=q->red;
                 q++;
                 p=PushShortPixel(endian,p,&pixel);
-                q->red=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),image->depth,
-                  scale);
+                q->red=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),range);
                 q->green=q->red;
                 q->blue=q->red;
                 p+=quantum_info->pad;
@@ -845,8 +850,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
               for (bit=0; bit < (long) (number_pixels % 2); bit++)
               {
                 p=PushShortPixel(endian,p,&pixel);
-                q->red=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),image->depth,
-                  scale);
+                q->red=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),range);
                 q->green=q->red;
                 q->blue=q->red;
                 p+=quantum_info->pad;
@@ -859,7 +863,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             q->green=q->red;
             q->blue=q->red;
             p+=quantum_info->pad;
@@ -949,11 +953,11 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             q->green=q->red;
             q->blue=q->red;
             p+=quantum_info->pad;
@@ -1007,16 +1011,15 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
           register unsigned char
             pixel;
 
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             pixel=(unsigned char) ((*p >> 4) & 0xf);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             q->green=q->red;
             q->blue=q->red;
             pixel=(unsigned char) ((*p) & 0xf);
-            q->opacity=(Quantum) (QuantumRange-
-              ScaleAnyToQuantum(pixel,image->depth,scale));
+            q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel,range));
             p++;
             q++;
           }
@@ -1042,15 +1045,15 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         case 10:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             q->green=q->red;
             q->blue=q->red;
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->opacity=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->opacity=ScaleAnyToQuantum(pixel,range);
             p+=quantum_info->pad;
             q++;
           }
@@ -1058,15 +1061,15 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         case 12:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             q->green=q->red;
             q->blue=q->red;
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->opacity=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->opacity=ScaleAnyToQuantum(pixel,range);
             p+=quantum_info->pad;
             q++;
           }
@@ -1149,16 +1152,15 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             q->green=q->red;
             q->blue=q->red;
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel,
-              image->depth,scale));
+            q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel,range));
             p+=quantum_info->pad;
             q++;
           }
@@ -1247,11 +1249,11 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             p+=quantum_info->pad;
             q++;
           }
@@ -1340,11 +1342,11 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->green=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->green=ScaleAnyToQuantum(pixel,range);
             p+=quantum_info->pad;
             q++;
           }
@@ -1433,11 +1435,11 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->blue=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->blue=ScaleAnyToQuantum(pixel,range);
             p+=quantum_info->pad;
             q++;
           }
@@ -1525,12 +1527,11 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel,
-              image->depth,scale));
+            q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel,range));
             p+=quantum_info->pad;
             q++;
           }
@@ -1622,11 +1623,11 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            indexes[x]=ScaleAnyToQuantum(pixel,image->depth,scale);
+            indexes[x]=ScaleAnyToQuantum(pixel,range);
             p+=quantum_info->pad;
             q++;
           }
@@ -1636,6 +1637,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
       break;
     }
     case RGBQuantum:
+    case CbYCrQuantum:
     {
       switch (quantum_info->depth)
       {
@@ -1652,6 +1654,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
             q->green=ScaleCharToQuantum(pixel);
             p=PushCharPixel(p,&pixel);
             q->blue=ScaleCharToQuantum(pixel);
+            q->opacity=OpaqueOpacity;
             p+=quantum_info->pad;
             q++;
           }
@@ -1659,18 +1662,15 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         case 10:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           if (quantum_info->pack == MagickFalse)
             {
               for (x=0; x < (long) number_pixels; x++)
               {
                 p=PushLongPixel(endian,p,&pixel);
-                q->red=ScaleAnyToQuantum((pixel >> 22) & 0x3ff,image->depth,
-                  scale);
-                q->green=ScaleAnyToQuantum((pixel >> 12) & 0x3ff,image->depth,
-                  scale);
-                q->blue=ScaleAnyToQuantum((pixel >> 2) & 0x3ff,image->depth,
-                  scale);
+                q->red=ScaleAnyToQuantum((pixel >> 22) & 0x3ff,range);
+                q->green=ScaleAnyToQuantum((pixel >> 12) & 0x3ff,range);
+                q->blue=ScaleAnyToQuantum((pixel >> 2) & 0x3ff,range);
                 p+=quantum_info->pad;
                 q++;
               }
@@ -1681,11 +1681,11 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
               for (x=0; x < (long) number_pixels; x++)
               {
                 p=PushQuantumLongPixel(&quantum_state,image->depth,p,&pixel);
-                q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+                q->red=ScaleAnyToQuantum(pixel,range);
                 p=PushQuantumLongPixel(&quantum_state,image->depth,p,&pixel);
-                q->green=ScaleAnyToQuantum(pixel,image->depth,scale);
+                q->green=ScaleAnyToQuantum(pixel,range);
                 p=PushQuantumLongPixel(&quantum_state,image->depth,p,&pixel);
-                q->blue=ScaleAnyToQuantum(pixel,image->depth,scale);
+                q->blue=ScaleAnyToQuantum(pixel,range);
                 q++;
               }
               break;
@@ -1693,18 +1693,18 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->green=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->green=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->blue=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->blue=ScaleAnyToQuantum(pixel,range);
             q++;
           }
           break;
         }
         case 12:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           if (quantum_info->pack == MagickFalse)
             {
               unsigned short
@@ -1718,20 +1718,17 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
                   default:
                   case 0:
                   {
-                    q->red=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),
-                      image->depth,scale);
+                    q->red=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),range);
                     break;
                   }
                   case 1:
                   {
-                    q->green=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),
-                      image->depth,scale);
+                    q->green=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),range);
                     break;
                   }
                   case 2:
                   {
-                    q->blue=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),
-                      image->depth,scale);
+                    q->blue=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),range);
                     q++;
                     break;
                   }
@@ -1742,20 +1739,17 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
                   default:
                   case 0:
                   {
-                    q->red=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),
-                      image->depth,scale);
+                    q->red=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),range);
                     break;
                   }
                   case 1:
                   {
-                    q->green=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),
-                      image->depth,scale);
+                    q->green=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),range);
                     break;
                   }
                   case 2:
                   {
-                    q->blue=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),
-                      image->depth,scale);
+                    q->blue=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),range);
                     q++;
                     break;
                   }
@@ -1770,20 +1764,17 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
                   default:
                   case 0:
                   {
-                    q->red=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),
-                      image->depth,scale);
+                    q->red=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),range);
                     break;
                   }
                   case 1:
                   {
-                    q->green=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),
-                      image->depth,scale);
+                    q->green=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),range);
                     break;
                   }
                   case 2:
                   {
-                    q->blue=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),
-                      image->depth,scale);
+                    q->blue=ScaleAnyToQuantum((QuantumAny) (pixel >> 4),range);
                     q++;
                     break;
                   }
@@ -1799,11 +1790,11 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
               for (x=0; x < (long) number_pixels; x++)
               {
                 p=PushQuantumLongPixel(&quantum_state,image->depth,p,&pixel);
-                q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+                q->red=ScaleAnyToQuantum(pixel,range);
                 p=PushQuantumLongPixel(&quantum_state,image->depth,p,&pixel);
-                q->green=ScaleAnyToQuantum(pixel,image->depth,scale);
+                q->green=ScaleAnyToQuantum(pixel,range);
                 p=PushQuantumLongPixel(&quantum_state,image->depth,p,&pixel);
-                q->blue=ScaleAnyToQuantum(pixel,image->depth,scale);
+                q->blue=ScaleAnyToQuantum(pixel,range);
                 q++;
               }
               break;
@@ -1811,11 +1802,11 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->green=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->green=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->blue=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->blue=ScaleAnyToQuantum(pixel,range);
             q++;
           }
           break;
@@ -1897,15 +1888,15 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->green=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->green=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->blue=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->blue=ScaleAnyToQuantum(pixel,range);
             q++;
           }
           break;
@@ -1914,6 +1905,7 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
       break;
     }
     case RGBAQuantum:
+    case CbYCrAQuantum:
     {
       switch (quantum_info->depth)
       {
@@ -2093,18 +2085,17 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->green=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->green=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->blue=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->blue=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel,
-              image->depth,scale));
+            q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel,range));
             q++;
           }
           break;
@@ -2221,17 +2212,17 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->green=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->green=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->blue=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->blue=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->opacity=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->opacity=ScaleAnyToQuantum(pixel,range);
             q++;
           }
           break;
@@ -2354,17 +2345,17 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->green=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->green=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->blue=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->blue=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            indexes[x]=ScaleAnyToQuantum(pixel,image->depth,scale);
+            indexes[x]=ScaleAnyToQuantum(pixel,range);
             q++;
           }
           break;
@@ -2498,20 +2489,19 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->green=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->green=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->blue=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->blue=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            indexes[x]=ScaleAnyToQuantum(pixel,image->depth,scale);
+            indexes[x]=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel,
-              image->depth,scale));
+            q->opacity=(Quantum) (QuantumRange-ScaleAnyToQuantum(pixel,range));
             q++;
           }
           break;
@@ -2586,13 +2576,13 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
         }
         default:
         {
-          scale=GetQuantumScale(image->depth);
+          range=GetQuantumRange(image->depth);
           for (x=0; x < (long) number_pixels; x++)
           {
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->red=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->red=ScaleAnyToQuantum(pixel,range);
             p=PushQuantumPixel(&quantum_state,image->depth,p,&pixel);
-            q->green=ScaleAnyToQuantum(pixel,image->depth,scale);
+            q->green=ScaleAnyToQuantum(pixel,range);
             q++;
           }
           break;
@@ -2603,16 +2593,40 @@ MagickExport size_t ImportQuantumPixels(Image *image,ViewInfo *image_view,
     default:
       break;
   }
+  if ((quantum_type == CbYCrQuantum) || (quantum_type == CbYCrAQuantum))
+    {
+      Quantum
+        quantum;
+
+      register PixelPacket
+        *__restrict q;
+
+      q=GetAuthenticPixelQueue(image);
+      if (image_view != (CacheView *) NULL)
+        q=GetCacheViewAuthenticPixelQueue(image_view);
+      for (x=0; x < (long) number_pixels; x++)
+      {
+        quantum=q->red;
+        q->red=q->green;
+        q->green=quantum;
+        q++;
+      }
+    }
   if (quantum_info->alpha_type == DisassociatedQuantumAlpha)
     {
       MagickRealType
         alpha;
 
+      register PixelPacket
+        *__restrict q;
+
       /*
         Disassociate alpha.
       */
       q=GetAuthenticPixelQueue(image);
-      for (x=0; x < (long) image->columns; x++)
+      if (image_view != (CacheView *) NULL)
+        q=GetCacheViewAuthenticPixelQueue(image_view);
+      for (x=0; x < (long) number_pixels; x++)
       {
         alpha=QuantumScale*((MagickRealType) QuantumRange-q->opacity);
         alpha=1.0/(fabs(alpha) <= MagickEpsilon ? 1.0 : alpha);
