@@ -17,7 +17,7 @@
 %                                  July 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -110,7 +110,7 @@ MagickExport Image *AffineTransformImage(const Image *image,
     distort[6];
 
   Image
-    *affine_image;
+    *deskew_image;
 
   /*
     Affine transform image.
@@ -127,9 +127,9 @@ MagickExport Image *AffineTransformImage(const Image *image,
   distort[3]=affine_matrix->sy;
   distort[4]=affine_matrix->tx;
   distort[5]=affine_matrix->ty;
-  affine_image=DistortImage(image,AffineProjectionDistortion,6,distort,
+  deskew_image=DistortImage(image,AffineProjectionDistortion,6,distort,
     MagickTrue,exception);
-  return(affine_image);
+  return(deskew_image);
 }
 
 /*
@@ -446,7 +446,7 @@ static inline ssize_t ReadRadonCell(const RadonInfo *radon_info,
 
 #if !defined(MAGICKCORE_HAVE_PPREAD)
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical
+  #pragma omp critical (MagickCore_ReadRadonCell)
 #endif
   {
     i=(-1);
@@ -488,9 +488,9 @@ static inline ssize_t WriteRadonCell(const RadonInfo *radon_info,
   ssize_t
     count;
 
-#if !defined(MAGICKCORE_HAVE_PPWRITE)
+#if !defined(MAGICKCORE_HAVE_PWRITE)
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp critical
+  #pragma omp critical (MagickCore_WriteRadonCell)
 #endif
   {
     if (MagickSeek(radon_info->file,offset,SEEK_SET) >= 0)
@@ -499,7 +499,7 @@ static inline ssize_t WriteRadonCell(const RadonInfo *radon_info,
         count=0;
         for (i=0; i < (ssize_t) length; i+=count)
         {
-#if !defined(MAGICKCORE_HAVE_PPWRITE)
+#if !defined(MAGICKCORE_HAVE_PWRITE)
           count=write(radon_info->file,buffer+i,MagickMin(length-i,(size_t)
             SSIZE_MAX));
 #else
@@ -515,7 +515,7 @@ static inline ssize_t WriteRadonCell(const RadonInfo *radon_info,
               break;
             }
         }
-#if !defined(MAGICKCORE_HAVE_PPWRITE)
+#if !defined(MAGICKCORE_HAVE_PWRITE)
       }
   }
 #endif
@@ -626,7 +626,7 @@ static void RadonProjection(RadonInfo *source_cells,
     q=swap;
   }
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16)
+  #pragma omp parallel for
 #endif
   for (x=0; x < (long) p->width; x++)
   {
@@ -675,8 +675,8 @@ static MagickBooleanType RadonTransform(const Image *image,
   unsigned short
     bits[256];
 
-  ViewInfo
-    **image_view;
+  CacheView
+    *image_view;
 
   for (width=1; width < ((image->columns+7)/8); width<<=1) ;
   source_cells=AcquireRadonInfo(image,width,image->rows,exception);
@@ -704,18 +704,17 @@ static MagickBooleanType RadonTransform(const Image *image,
     bits[i]=(unsigned short) count;
   }
   status=MagickTrue;
-  image_view=AcquireCacheViewThreadSet(image);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(status)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register long
       i,
-      id,
       x;
 
     unsigned long
@@ -724,8 +723,7 @@ static MagickBooleanType RadonTransform(const Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],0,y,image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       {
         status=MagickFalse;
@@ -759,16 +757,15 @@ static MagickBooleanType RadonTransform(const Image *image,
   RadonProjection(source_cells,destination_cells,-1,projection);
   (void) ResetRadonCells(source_cells);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(status)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register long
       i,
-      id,
       x;
 
     unsigned long
@@ -777,8 +774,7 @@ static MagickBooleanType RadonTransform(const Image *image,
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],0,y,image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       {
         status=MagickFalse;
@@ -810,7 +806,7 @@ static MagickBooleanType RadonTransform(const Image *image,
       }
   }
   RadonProjection(source_cells,destination_cells,1,projection);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   destination_cells=DestroyRadonInfo(destination_cells);
   source_cells=DestroyRadonInfo(source_cells);
   return(MagickTrue);
@@ -828,7 +824,7 @@ static void GetImageBackgroundColor(Image *image,const long offset,
   MagickRealType
     count;
 
-  ViewInfo
+  CacheView
     *image_view;
 
   /*
@@ -842,7 +838,7 @@ static void GetImageBackgroundColor(Image *image,const long offset,
   for (y=0; y < (long) image->rows; y++)
   {
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register long
       x;
@@ -878,18 +874,20 @@ static void GetImageBackgroundColor(Image *image,const long offset,
 MagickExport Image *DeskewImage(const Image *image,const double threshold,
   ExceptionInfo *exception)
 {
+  AffineMatrix
+    affine_matrix;
+
   const char
     *artifact;
 
   double
-    degrees,
-    sum;
+    degrees;
 
   Image
     *clone_image,
     *crop_image,
-    *median_image,
-    *rotate_image;
+    *deskew_image,
+    *median_image;
 
   long
     skew;
@@ -923,7 +921,6 @@ MagickExport Image *DeskewImage(const Image *image,const double threshold,
       ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
     }
   max_projection=0;
-  sum=0.0;
   skew=0;
   for (i=0; i < (long) (2*width-1); i++)
   {
@@ -932,34 +929,44 @@ MagickExport Image *DeskewImage(const Image *image,const double threshold,
         skew=i-(long) width+1;
         max_projection=projection[i];
       }
-    sum+=projection[i];
   }
   projection=(unsigned long *) RelinquishMagickMemory(projection);
   /*
     Deskew image.
   */
+  clone_image=CloneImage(image,0,0,MagickTrue,exception);
+  if (clone_image == (Image *) NULL)
+    return((Image *) NULL);
+  (void) SetImageVirtualPixelMethod(clone_image,BackgroundVirtualPixelMethod);
   degrees=RadiansToDegrees(-atan((double) skew/width/8));
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TransformEvent,GetMagickModule(),"  Deskew angle: %g",
       degrees);
+  affine_matrix.sx=cos(DegreesToRadians(fmod((double) degrees,360.0)));
+  affine_matrix.rx=sin(DegreesToRadians(fmod((double) degrees,360.0)));
+  affine_matrix.ry=(-sin(DegreesToRadians(fmod((double) degrees,360.0))));
+  affine_matrix.sy=cos(DegreesToRadians(fmod((double) degrees,360.0)));
+  affine_matrix.tx=0.0;
+  affine_matrix.ty=0.0;
   artifact=GetImageArtifact(image,"deskew:auto-crop");
   if (artifact == (const char *) NULL)
-    return(RotateImage(image,degrees,exception));
+    {
+      deskew_image=AffineTransformImage(clone_image,&affine_matrix,exception);
+      clone_image=DestroyImage(clone_image);
+      return(deskew_image);
+    }
   /*
     Auto-crop image.
   */
-  clone_image=CloneImage(image,0,0,MagickTrue,exception);
-  if (clone_image == (Image *) NULL)
-    return((Image *) NULL);
   GetImageBackgroundColor(clone_image,atol(artifact),exception);
-  rotate_image=RotateImage(clone_image,degrees,exception);
+  deskew_image=AffineTransformImage(clone_image,&affine_matrix,exception);
   clone_image=DestroyImage(clone_image);
-  if (rotate_image == (Image *) NULL)
+  if (deskew_image == (Image *) NULL)
     return((Image *) NULL);
-  median_image=MedianFilterImage(rotate_image,0.0,exception);
+  median_image=MedianFilterImage(deskew_image,0.0,exception);
   if (median_image == (Image *) NULL)
     {
-      rotate_image=DestroyImage(rotate_image);
+      deskew_image=DestroyImage(deskew_image);
       return((Image *) NULL);
     }
   geometry=GetImageBoundingBox(median_image,exception);
@@ -967,8 +974,8 @@ MagickExport Image *DeskewImage(const Image *image,const double threshold,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TransformEvent,GetMagickModule(),"  Deskew geometry: "
       "%lux%lu%+ld%+ld",geometry.width,geometry.height,geometry.x,geometry.y);
-  crop_image=CropImage(rotate_image,&geometry,exception);
-  rotate_image=DestroyImage(rotate_image);
+  crop_image=CropImage(deskew_image,&geometry,exception);
+  deskew_image=DestroyImage(deskew_image);
   return(crop_image);
 }
 
@@ -1019,9 +1026,9 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
   RectangleInfo
     page;
 
-  ViewInfo
-    **image_view,
-    **rotate_view;
+  CacheView
+    *image_view,
+    *rotate_view;
 
   /*
     Initialize rotated image attributes.
@@ -1042,8 +1049,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  rotate_view=AcquireCacheViewThreadSet(rotate_image);
+  image_view=AcquireCacheView(image);
+  rotate_view=AcquireCacheView(rotate_image);
   switch (rotations)
   {
     case 0:
@@ -1052,7 +1059,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
         Rotate 0 degrees.
       */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-      #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,8) shared(progress,status)
 #endif
       for (y=0; y < (long) image->rows; y++)
       {
@@ -1060,41 +1067,36 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
           sync;
 
         register const IndexPacket
-          *indexes;
+          *__restrict indexes;
 
         register const PixelPacket
-          *p;
+          *__restrict p;
 
         register IndexPacket
-          *rotate_indexes;
-
-        register long
-          id;
+          *__restrict rotate_indexes;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
-        id=GetCacheViewThreadId();
-        p=GetCacheViewVirtualPixels(image_view[id],0,y,image->columns,1,
+        p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,
           exception);
-        q=QueueCacheViewAuthenticPixels(rotate_view[id],0,y,
+        q=QueueCacheViewAuthenticPixels(rotate_view,0,y,
           rotate_image->columns,1,exception);
         if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
           {
             status=MagickFalse;
             continue;
           }
-        indexes=GetCacheViewVirtualIndexes(image_view[id]);
+        indexes=GetCacheViewVirtualIndexQueue(image_view);
+        rotate_indexes=GetCacheViewAuthenticIndexQueue(rotate_view);
         (void) CopyMagickMemory(q,p,image->columns*sizeof(*p));
-        if (indexes != (IndexPacket *) NULL)
-          {
-            rotate_indexes=GetCacheViewAuthenticIndexes(rotate_view[id]);
-            (void) CopyMagickMemory(rotate_indexes,indexes,image->columns*
-              sizeof(*indexes));
-          }
-        sync=SyncCacheViewAuthenticPixels(rotate_view[id],exception);
+        if ((indexes != (IndexPacket *) NULL) &&
+            (rotate_indexes != (IndexPacket *) NULL))
+          (void) CopyMagickMemory(rotate_indexes,indexes,image->columns*
+            sizeof(*indexes));
+        sync=SyncCacheViewAuthenticPixels(rotate_view,exception);
         if (sync == MagickFalse)
           status=MagickFalse;
         if (image->progress_monitor != (MagickProgressMonitor) NULL)
@@ -1103,7 +1105,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
               proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-            #pragma omp critical
+  #pragma omp critical (MagickCore_IntegralRotateImage)
 #endif
             proceed=SetImageProgress(image,RotateImageTag,progress++,
               image->rows);
@@ -1122,33 +1124,34 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
         Rotate 90 degrees.
       */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-      #pragma omp parallel for schedule(static,TileHeight) shared(progress,status)
+  #pragma omp parallel for schedule(static,1) shared(progress,status)
 #endif
       for (tile_y=0; tile_y < (long) image->rows; tile_y+=TileHeight)
       {
         register long
-          id,
           tile_x;
 
         if (status == MagickFalse)
           continue;
-        id=GetCacheViewThreadId();
         for (tile_x=0; tile_x < (long) image->columns; tile_x+=TileWidth)
         {
           MagickBooleanType
             sync;
 
           register const IndexPacket
-            *indexes;
+            *__restrict indexes;
 
           register const PixelPacket
-            *p;
+            *__restrict p;
 
           register IndexPacket
-            *rotate_indexes;
+            *__restrict rotate_indexes;
+
+          register long
+            y;
 
           register PixelPacket
-            *q;
+            *__restrict q;
 
           unsigned long
             tile_height,
@@ -1162,36 +1165,42 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
           if ((tile_y+TileHeight) > (long) image->rows)
             tile_height=(unsigned long) (TileHeight-(tile_y+TileHeight-
               image->rows));
-          p=GetCacheViewVirtualPixels(image_view[id],tile_x,tile_y,tile_width,
+          p=GetCacheViewVirtualPixels(image_view,tile_x,tile_y,tile_width,
             tile_height,exception);
-          q=QueueCacheViewAuthenticPixels(rotate_view[id],(long)
-            rotate_image->columns-(tile_y+tile_height),tile_x,tile_height,
-            tile_width,exception);
-          if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
+          if (p == (const PixelPacket *) NULL)
             {
               status=MagickFalse;
-              continue;
+              break;
             }
-          indexes=GetCacheViewVirtualIndexes(image_view[id]);
-          rotate_indexes=GetCacheViewAuthenticIndexes(rotate_view[id]);
+          indexes=GetCacheViewVirtualIndexQueue(image_view);
           for (y=0; y < (long) tile_width; y++)
           {
             register const PixelPacket
-              *tile_pixels;
+              *__restrict tile_pixels;
 
             register long
               x;
 
+            q=QueueCacheViewAuthenticPixels(rotate_view,(long)
+              rotate_image->columns-(tile_y+tile_height),y+tile_x,tile_height,
+              1,exception);
+            if (q == (PixelPacket *) NULL)
+              {
+                status=MagickFalse;
+                break;
+              }
             tile_pixels=p+(tile_height-1)*tile_width+y;
             for (x=0; x < (long) tile_height; x++)
             {
               *q++=(*tile_pixels);
               tile_pixels-=tile_width;
             }
-            if (indexes != (IndexPacket *) NULL)
+            rotate_indexes=GetCacheViewAuthenticIndexQueue(rotate_view);
+            if ((indexes != (IndexPacket *) NULL) &&
+                (rotate_indexes != (IndexPacket *) NULL))
               {
                 register const IndexPacket
-                  *tile_indexes;
+                  *__restrict tile_indexes;
 
                 tile_indexes=indexes+(tile_height-1)*tile_width+y;
                 for (x=0; x < (long) tile_height; x++)
@@ -1200,10 +1209,10 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
                   tile_indexes-=tile_width;
                 }
               }
+            sync=SyncCacheViewAuthenticPixels(rotate_view,exception);
+            if (sync == MagickFalse)
+              status=MagickFalse;
           }
-          sync=SyncCacheViewAuthenticPixels(rotate_view[id],exception);
-          if (sync == MagickFalse)
-            status=MagickFalse;
         }
         if (image->progress_monitor != (MagickProgressMonitor) NULL)
           {
@@ -1211,7 +1220,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
               proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-            #pragma omp critical
+  #pragma omp critical (MagickCore_IntegralRotateImage)
 #endif
             proceed=SetImageProgress(image,RotateImageTag,progress+=TileHeight,
               image->rows);
@@ -1231,7 +1240,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
         Rotate 180 degrees.
       */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-      #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(static,8) shared(progress,status)
 #endif
       for (y=0; y < (long) image->rows; y++)
       {
@@ -1239,44 +1248,41 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
           sync;
 
         register const IndexPacket
-          *indexes;
+          *__restrict indexes;
 
         register const PixelPacket
-          *p;
+          *__restrict p;
 
         register IndexPacket
-          *rotate_indexes;
+          *__restrict rotate_indexes;
 
         register long
-          id,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
-        id=GetCacheViewThreadId();
-        p=GetCacheViewVirtualPixels(image_view[id],0,y,image->columns,1,
+        p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,
           exception);
-        q=QueueCacheViewAuthenticPixels(rotate_view[id],0,(long) (image->rows-
+        q=QueueCacheViewAuthenticPixels(rotate_view,0,(long) (image->rows-
           y-1),image->columns,1,exception);
         if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
           {
             status=MagickFalse;
             continue;
           }
-        indexes=GetCacheViewVirtualIndexes(image_view[id]);
+        indexes=GetCacheViewVirtualIndexQueue(image_view);
+        rotate_indexes=GetCacheViewAuthenticIndexQueue(rotate_view);
         q+=image->columns;
         for (x=0; x < (long) image->columns; x++)
           *--q=(*p++);
-        if (indexes != (IndexPacket *) NULL)
-          {
-            rotate_indexes=GetCacheViewAuthenticIndexes(rotate_view[id]);
-            for (x=0; x < (long) image->columns; x++)
-              rotate_indexes[image->columns-x-1]=indexes[x];
-          }
-        sync=SyncCacheViewAuthenticPixels(rotate_view[id],exception);
+        if ((indexes != (IndexPacket *) NULL) &&
+            (rotate_indexes != (IndexPacket *) NULL))
+          for (x=0; x < (long) image->columns; x++)
+            rotate_indexes[image->columns-x-1]=indexes[x];
+        sync=SyncCacheViewAuthenticPixels(rotate_view,exception);
         if (sync == MagickFalse)
           status=MagickFalse;
         if (image->progress_monitor != (MagickProgressMonitor) NULL)
@@ -1285,7 +1291,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
               proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-            #pragma omp critical
+  #pragma omp critical (MagickCore_IntegralRotateImage)
 #endif
             proceed=SetImageProgress(image,RotateImageTag,progress++,
               image->rows);
@@ -1308,33 +1314,34 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
         Rotate 270 degrees.
       */
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-      #pragma omp parallel for schedule(static,TileHeight) shared(progress,status)
+  #pragma omp parallel for schedule(static,1) shared(progress,status)
 #endif
       for (tile_y=0; tile_y < (long) image->rows; tile_y+=TileHeight)
       {
         register long
-          id,
           tile_x;
 
         if (status == MagickFalse)
           continue;
-        id=GetCacheViewThreadId();
         for (tile_x=0; tile_x < (long) image->columns; tile_x+=TileWidth)
         {
           MagickBooleanType
             sync;
 
           register const IndexPacket
-            *indexes;
+            *__restrict indexes;
 
           register const PixelPacket
-            *p;
+            *__restrict p;
 
           register IndexPacket
-            *rotate_indexes;
+            *__restrict rotate_indexes;
+
+          register long
+            y;
 
           register PixelPacket
-            *q;
+            *__restrict q;
 
           unsigned long
             tile_height,
@@ -1348,36 +1355,41 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
           if ((tile_y+TileHeight) > (long) image->rows)
             tile_height=(unsigned long) (TileHeight-(tile_y+TileHeight-
               image->rows));
-          p=GetCacheViewVirtualPixels(image_view[id],tile_x,tile_y,tile_width,
+          p=GetCacheViewVirtualPixels(image_view,tile_x,tile_y,tile_width,
             tile_height,exception);
-          q=QueueCacheViewAuthenticPixels(rotate_view[id],tile_y,(long)
-            rotate_image->rows-(tile_x+tile_width),tile_height,tile_width,
-            exception);
-          if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
+          if (p == (const PixelPacket *) NULL)
             {
               status=MagickFalse;
               break;
             }
-          indexes=GetCacheViewVirtualIndexes(image_view[id]);
-          rotate_indexes=GetCacheViewAuthenticIndexes(rotate_view[id]);
+          indexes=GetCacheViewVirtualIndexQueue(image_view);
           for (y=0; y < (long) tile_width; y++)
           {
             register const PixelPacket
-              *tile_pixels;
+              *__restrict tile_pixels;
 
             register long
               x;
 
+            q=QueueCacheViewAuthenticPixels(rotate_view,tile_y,(long)
+              y+rotate_image->rows-(tile_x+tile_width),tile_height,1,exception);
+            if (q == (PixelPacket *) NULL)
+              {
+                status=MagickFalse;
+                break;
+              }
             tile_pixels=p+(tile_width-1)-y;
             for (x=0; x < (long) tile_height; x++)
             {
               *q++=(*tile_pixels);
               tile_pixels+=tile_width;
             }
-            if (indexes != (IndexPacket *) NULL)
+            rotate_indexes=GetCacheViewAuthenticIndexQueue(rotate_view);
+            if ((indexes != (IndexPacket *) NULL) &&
+                (rotate_indexes != (IndexPacket *) NULL))
               {
                 register const IndexPacket
-                  *tile_indexes;
+                  *__restrict tile_indexes;
 
                 tile_indexes=indexes+(tile_width-1)-y;
                 for (x=0; x < (long) tile_height; x++)
@@ -1386,10 +1398,10 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
                   tile_indexes+=tile_width;
                 }
               }
+            sync=SyncCacheViewAuthenticPixels(rotate_view,exception);
+            if (sync == MagickFalse)
+              status=MagickFalse;
           }
-          sync=SyncCacheViewAuthenticPixels(rotate_view[id],exception);
-          if (sync == MagickFalse)
-            status=MagickFalse;
         }
         if (image->progress_monitor != (MagickProgressMonitor) NULL)
           {
@@ -1397,7 +1409,7 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
               proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-            #pragma omp critical
+  #pragma omp critical (MagickCore_IntegralRotateImage)
 #endif
             proceed=SetImageProgress(image,RotateImageTag,progress+=TileHeight,
               image->rows);
@@ -1412,8 +1424,8 @@ static Image *IntegralRotateImage(const Image *image,unsigned long rotations,
       break;
     }
   }
-  rotate_view=DestroyCacheViewThreadSet(rotate_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  rotate_view=DestroyCacheView(rotate_view);
+  image_view=DestroyCacheView(image_view);
   rotate_image->type=image->type;
   rotate_image->page=page;
   if (status == MagickFalse)
@@ -1524,8 +1536,8 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
   MagickPixelPacket
     background;
 
-  ViewInfo
-    **image_view;
+  CacheView
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -1542,16 +1554,12 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
   status=MagickTrue;
   progress=0;
   exception=(&image->exception);
-  image_view=AcquireCacheViewThreadSet(image);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress, status)
+  #pragma omp parallel for schedule(dynamic,8) shared(progress, status)
 #endif
   for (y=0; y < (long) height; y++)
   {
-    IndexPacket
-      *indexes,
-      *shear_indexes;
-
     long
       step;
 
@@ -1565,26 +1573,29 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
       displacement;
 
     register long
-      i,
-      id;
+      i;
+
+    register IndexPacket
+      *__restrict indexes,
+      *__restrict shear_indexes;
 
     register PixelPacket
-      *p,
-      *q;
+      *__restrict p,
+      *__restrict q;
 
     ShearDirection
       direction;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewAuthenticPixels(image_view[id],0,y_offset+y,image->columns,1,exception);
+    p=GetCacheViewAuthenticPixels(image_view,0,y_offset+y,image->columns,1,
+      exception);
     if (p == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     p+=x_offset;
     indexes+=x_offset;
     displacement=degrees*(MagickRealType) (y-height/2.0);
@@ -1667,7 +1678,7 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
         break;
       }
     }
-    if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -1675,14 +1686,14 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_XShearImage)
 #endif
         proceed=SetImageProgress(image,XShearImageTag,progress++,height);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -1745,8 +1756,8 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
   MagickPixelPacket
     background;
 
-  ViewInfo
-    **image_view;
+  CacheView
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -1763,16 +1774,12 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
   status=MagickTrue;
   progress=0;
   exception=(&image->exception);
-  image_view=AcquireCacheViewThreadSet(image);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress, status)
+  #pragma omp parallel for schedule(dynamic,8) shared(progress, status)
 #endif
   for (x=0; x < (long) width; x++)
   {
-    IndexPacket
-      *indexes,
-      *shear_indexes;
-
     long
       step;
 
@@ -1785,27 +1792,30 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
       area,
       displacement;
 
+    register IndexPacket
+      *__restrict indexes,
+      *__restrict shear_indexes;
+
     register long
-      i,
-      id;
+      i;
 
     register PixelPacket
-      *p,
-      *q;
+      *__restrict p,
+      *__restrict q;
 
     ShearDirection
       direction;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewAuthenticPixels(image_view[id],x_offset+x,0,1,image->rows,exception);
+    p=GetCacheViewAuthenticPixels(image_view,x_offset+x,0,1,image->rows,
+      exception);
     if (p == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     p+=y_offset;
     indexes+=y_offset;
     displacement=degrees*(MagickRealType) (x-width/2.0);
@@ -1888,7 +1898,7 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
         break;
       }
     }
-    if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -1896,14 +1906,14 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_YShearImage)
 #endif
         proceed=SetImageProgress(image,YShearImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -2016,8 +2026,8 @@ MagickExport Image *RotateImage(const Image *image,const double degrees,
       height=image->columns;
     }
   y_width=width+(long) (fabs(shear.x)*height+0.5);
-  x_offset=(long) (width+((fabs(shear.y)*height+0.5)-width)/2.0+0.5);
-  y_offset=(long) (height+((fabs(shear.y)*y_width+0.5)-height)/2.0+0.5);
+  x_offset=(long) (width+((fabs(shear.y)*height)-width)/2.0+0.5);
+  y_offset=(long) (height+((fabs(shear.y)*y_width)-height)/2.0+0.5);
   /*
     Surround image with a border.
   */
@@ -2121,8 +2131,8 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
   integral_image=CloneImage(image,0,0,MagickTrue,exception);
   if (integral_image == (Image *) NULL)
     ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
-  shear.x=(-tan(DegreesToRadians(x_shear)));
-  shear.y=tan(DegreesToRadians(y_shear));
+  shear.x=(-tan(DegreesToRadians(fmod(x_shear,360.0))));
+  shear.y=tan(DegreesToRadians(fmod(y_shear,360.0)));
   if ((shear.x == 0.0) && (shear.y == 0.0))
     return(integral_image);
   if (SetImageStorageClass(integral_image,DirectClass) == MagickFalse)
@@ -2137,10 +2147,9 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
     Compute image size.
   */
   y_width=image->columns+(long) (fabs(shear.x)*image->rows+0.5);
-  x_offset=(long) (image->columns+((fabs(shear.x)*image->rows)-
-    image->columns)/2.0+0.5);
-  y_offset=(long) (image->rows+((fabs(shear.y)*y_width+0.5)-image->rows)/2.0+
-    0.5);
+  x_offset=(long) (image->columns+((fabs(shear.x)*image->rows)-image->columns)/
+    2.0+0.5);
+  y_offset=(long) (image->rows+((fabs(shear.y)*y_width)-image->rows)/2.0+0.5);
   /*
     Surround image with border.
   */
