@@ -16,7 +16,7 @@
 %                              December 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -497,6 +497,9 @@ MagickExport MagickBooleanType SignatureImage(Image *image)
   unsigned char
     *pixels;
 
+  CacheView
+    *image_view;
+
   /*
     Compute image digital signature.
   */
@@ -521,17 +524,20 @@ MagickExport MagickBooleanType SignatureImage(Image *image)
   signature=AcquireStringInfo(quantum_info->extent);
   pixels=GetQuantumPixels(quantum_info);
   exception=(&image->exception);
+  image_view=AcquireCacheView(image);
   for (y=0; y < (long) image->rows; y++)
   {
-    p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    length=ExportQuantumPixels(image,(const ViewInfo *) NULL,quantum_info,
-      quantum_type,pixels,&image->exception);
+    length=ExportQuantumPixels(image,image_view,quantum_info,quantum_type,
+      pixels,&image->exception);
     SetStringInfoLength(signature,length);
     SetStringInfoDatum(signature,pixels);
     UpdateSignature(signature_info,signature);
   }
+  image_view=DestroyCacheView(image_view);
+  quantum_info=DestroyQuantumInfo(quantum_info);
   FinalizeSignature(signature_info);
   hex_signature=StringInfoToHexString(GetSignatureDigest(signature_info));
   (void) DeleteImageProperty(image,"signature");
@@ -787,7 +793,7 @@ MagickExport void UpdateSignature(SignatureInfo *signature_info,
       i=GetStringInfoLength(signature_info->message)-signature_info->offset;
       if (i > n)
         i=n;
-      (void) memcpy(GetStringInfoDatum(signature_info->message)+
+      (void) CopyMagickMemory(GetStringInfoDatum(signature_info->message)+
         signature_info->offset,p,i);
       n-=i;
       p+=i;
@@ -804,7 +810,7 @@ MagickExport void UpdateSignature(SignatureInfo *signature_info,
     n-=GetStringInfoLength(signature_info->message);
     TransformSignature(signature_info);
   }
-  (void) memcpy(GetStringInfoDatum(signature_info->message),p,n);
+  (void) CopyMagickMemory(GetStringInfoDatum(signature_info->message),p,n);
   signature_info->offset=n;
   /*
     Reset working registers.
