@@ -654,7 +654,8 @@ MagickExport char *ConstantString(const char *source)
 %
 %  CopyMagickString() copies the source string to the destination string.  The
 %  destination buffer is always null-terminated even if the string must be
-%  truncated.
+%  truncated.  The return value is the minimum of the source string length
+%  or the length parameter.
 %
 %  The format of the CopyMagickString method is:
 %
@@ -713,7 +714,6 @@ MagickExport size_t CopyMagickString(char *destination,const char *source,
     }
   if (length != 0)
     *q='\0';
-  while (*p++ != '\0') ;
   return((size_t) (p-source-1));
 }
 
@@ -1014,7 +1014,11 @@ MagickExport long FormatMagickSize(const MagickSizeType size,char *format)
   static const char
     *units[] = { "b", "kb", "mb", "gb", "tb", "pb", "eb", (char *) NULL };
 
+#if defined(_MSC_VER) && (_MSC_VER == 1200)
+  length=(double) ((MagickOffsetType) size);
+#else
   length=(double) size;
+#endif
   for (i=0; (length >= 1024.0) && (units[i+1] != (const char *) NULL); i++)
     length/=1024.0;
   for (j=2; j < 10; j++)
@@ -1124,27 +1128,31 @@ MagickExport long FormatMagickTime(const time_t time,const size_t length,
     count;
 
   struct tm
-    *local_time,
-    *utc_time;
+    gm_time,
+    local_time;
 
   time_t
     timezone;
 
   assert(timestamp != (char *) NULL);
-  local_time=localtime(&time);
-  if (local_time == (struct tm *) NULL)
-    return(0);
-  utc_time=gmtime(&time);
-  if (utc_time == (struct tm *) NULL)
-    return(0);
-  timezone=(time_t) ((local_time->tm_min-utc_time->tm_min)/60+
-    local_time->tm_hour-utc_time->tm_hour+24*((local_time->tm_year-
-    utc_time->tm_year) != 0 ? (local_time->tm_year-utc_time->tm_year) :
-    (local_time->tm_yday-utc_time->tm_yday)));
+#if defined(MAGICKCORE_HAVE_LOCALTIME_R)
+  (void) localtime_r(&time,&local_time);
+#else
+  (void) memcpy(&local_time,localtime(&time),sizeof(local_time));
+#endif
+#if defined(MAGICKCORE_HAVE_GMTIME_R)
+  (void) gmtime_r(&time,&gm_time);
+#else
+  (void) memcpy(&gm_time,gmtime(&time),sizeof(gm_time));
+#endif
+  timezone=(time_t) ((local_time.tm_min-gm_time.tm_min)/60+
+    local_time.tm_hour-gm_time.tm_hour+24*((local_time.tm_year-
+    gm_time.tm_year) != 0 ? (local_time.tm_year-gm_time.tm_year) :
+    (local_time.tm_yday-gm_time.tm_yday)));
   count=FormatMagickString(timestamp,length,
-    "%04d-%02d-%02dT%02d:%02d:%02d%+03ld:00",local_time->tm_year+1900,
-    local_time->tm_mon+1,local_time->tm_mday,local_time->tm_hour,
-    local_time->tm_min,local_time->tm_sec,(long) timezone);
+    "%04d-%02d-%02dT%02d:%02d:%02d%+03ld:00",local_time.tm_year+1900,
+    local_time.tm_mon+1,local_time.tm_mday,local_time.tm_hour,
+    local_time.tm_min,local_time.tm_sec,(long) timezone);
   return(count);
 }
 
@@ -2140,9 +2148,6 @@ MagickExport char **StringToList(const char *text)
   char
     **textlist;
 
-  register char
-    *q;
-
   register const char
     *p;
 
@@ -2160,6 +2165,9 @@ MagickExport char **StringToList(const char *text)
       break;
   if (*p == '\0')
     {
+      register const char
+        *q;
+
       /*
         Convert string to an ASCII list.
       */
@@ -2174,7 +2182,7 @@ MagickExport char **StringToList(const char *text)
       p=text;
       for (i=0; i < (long) lines; i++)
       {
-        for (q=(char *) p; *q != '\0'; q++)
+        for (q=p; *q != '\0'; q++)
           if ((*q == '\r') || (*q == '\n'))
             break;
         textlist[i]=(char *) AcquireQuantumMemory((size_t) (q-p)+MaxTextExtent,
@@ -2192,6 +2200,9 @@ MagickExport char **StringToList(const char *text)
       char
         hex_string[MaxTextExtent];
 
+      register char
+        *q;
+
       register long
         j;
 
@@ -2383,8 +2394,8 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   assert(buffer != (char **) NULL);
   assert(*buffer != (char *) NULL);
-  assert(search != (const char*) NULL);
-  assert(replace != (const char*) NULL);
+  assert(search != (const char *) NULL);
+  assert(replace != (const char *) NULL);
   if (strcmp(search,replace) == 0)
     return(MagickTrue);
   source=(*buffer);
@@ -2434,8 +2445,7 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
         if (result == (char *) NULL)
           ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
       }
-    (void) ConcatenateMagickString(result+destination_offset,replace,(size_t)
-      (length+MaxTextExtent)*sizeof(*result));
+    (void) ConcatenateMagickString(result+destination_offset,replace,length);
     destination_offset+=replace_length;
     /*
       Find next match.
@@ -2458,7 +2468,7 @@ MagickExport MagickBooleanType SubstituteString(char **buffer,
         ThrowFatalException(ResourceLimitFatalError,"UnableToAcquireString");
     }
   (void) ConcatenateMagickString(result+destination_offset,source,(size_t)
-    (length+MaxTextExtent-destination_offset)*sizeof(*result));
+    (length+MaxTextExtent-destination_offset));
   (void) RelinquishMagickMemory(*buffer);
   *buffer=result;
   return(MagickTrue);
