@@ -17,7 +17,7 @@
 %                                 October 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -69,6 +69,7 @@
 #include "magick/quantize.h"
 #include "magick/quantum.h"
 #include "magick/random_.h"
+#include "magick/random-private.h"
 #include "magick/resize.h"
 #include "magick/resource_.h"
 #include "magick/segment.h"
@@ -160,9 +161,9 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
   MagickRealType
     number_pixels;
 
-  ViewInfo
-    **image_view,
-    **threshold_view;
+  CacheView
+    *image_view,
+    *threshold_view;
 
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -188,10 +189,10 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
   progress=0;
   GetMagickPixelPacket(image,&zero);
   number_pixels=(MagickRealType) width*height;
-  image_view=AcquireCacheViewThreadSet(image);
-  threshold_view=AcquireCacheViewThreadSet(threshold_image);
+  image_view=AcquireCacheView(image);
+  threshold_view=AcquireCacheView(threshold_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
@@ -199,35 +200,33 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
       sync;
 
     register const IndexPacket
-      *indexes;
+      *__restrict indexes;
 
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register IndexPacket
-      *threshold_indexes;
+      *__restrict threshold_indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],-((long) width/2L),y-height/2L,
+    p=GetCacheViewVirtualPixels(image_view,-((long) width/2L),y-height/2L,
       image->columns+width,height,exception);
-    q=GetCacheViewAuthenticPixels(threshold_view[id],0,y,
-      threshold_image->columns,1,exception);
+    q=GetCacheViewAuthenticPixels(threshold_view,0,y,threshold_image->columns,1,
+      exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewVirtualIndexes(image_view[id]);
-    threshold_indexes=GetCacheViewAuthenticIndexes(threshold_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    threshold_indexes=GetCacheViewAuthenticIndexQueue(threshold_view);
     for (x=0; x < (long) image->columns; x++)
     {
       long
@@ -279,7 +278,7 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
       p++;
       q++;
     }
-    sync=SyncCacheViewAuthenticPixels(threshold_view[id],exception);
+    sync=SyncCacheViewAuthenticPixels(threshold_view,exception);
     if (sync == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
@@ -288,7 +287,7 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_AdaptiveThresholdImage)
 #endif
         proceed=SetImageProgress(image,ThresholdImageTag,progress++,
           image->rows);
@@ -296,8 +295,8 @@ MagickExport Image *AdaptiveThresholdImage(const Image *image,
           status=MagickFalse;
       }
   }
-  threshold_view=DestroyCacheViewThreadSet(threshold_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  threshold_view=DestroyCacheView(threshold_view);
+  image_view=DestroyCacheView(image_view);
   if (status == MagickFalse)
     threshold_image=DestroyImage(threshold_image);
   return(threshold_image);
@@ -370,8 +369,8 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
   MagickBooleanType
     status;
 
-  ViewInfo
-    **image_view;
+  CacheView
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -385,33 +384,30 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
   status=MagickTrue;
   progress=0;
   exception=(&image->exception);
-  image_view=AcquireCacheViewThreadSet(image);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
-      *indexes;
+      *__restrict indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewAuthenticPixels(image_view[id],0,y,image->columns,1,
-      exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     if (channel == DefaultChannels)
       {
         for (x=0; x < (long) image->columns; x++)
@@ -450,7 +446,7 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
             0 : QuantumRange);
         q++;
       }
-    if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -458,7 +454,7 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_BilevelImageChannel)
 #endif
         proceed=SetImageProgress(image,ThresholdImageTag,progress++,
           image->rows);
@@ -466,7 +462,7 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -488,21 +484,36 @@ MagickExport MagickBooleanType BilevelImageChannel(Image *image,
 %  The format of the BlackThresholdImage method is:
 %
 %      MagickBooleanType BlackThresholdImage(Image *image,const char *threshold)
+%      MagickBooleanType BlackThresholdImageChannel(Image *image,
+%        const ChannelType channel,const char *threshold,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o image: the image.
 %
+%    o channel: the channel or channels to be thresholded.
+%
 %    o threshold: Define the threshold value.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
 MagickExport MagickBooleanType BlackThresholdImage(Image *image,
   const char *threshold)
 {
-#define ThresholdImageTag  "Threshold/Image"
+  MagickBooleanType
+    status;
 
-  ExceptionInfo
-    *exception;
+  status=BlackThresholdImageChannel(image,DefaultChannels,threshold,
+    &image->exception);
+  return(status);
+}
+
+MagickExport MagickBooleanType BlackThresholdImageChannel(Image *image,
+  const ChannelType channel,const char *thresholds,ExceptionInfo *exception)
+{
+#define ThresholdImageTag  "Threshold/Image"
 
   GeometryInfo
     geometry_info;
@@ -515,95 +526,95 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
     status;
 
   MagickPixelPacket
-    pixel;
+    threshold;
 
   MagickStatusType
     flags;
 
-  ViewInfo
-    **image_view;
+  CacheView
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  if (threshold == (const char *) NULL)
+  if (thresholds == (const char *) NULL)
     return(MagickTrue);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
-  GetMagickPixelPacket(image,&pixel);
-  flags=ParseGeometry(threshold,&geometry_info);
-  pixel.red=geometry_info.rho;
-  pixel.green=geometry_info.sigma;
+  GetMagickPixelPacket(image,&threshold);
+  flags=ParseGeometry(thresholds,&geometry_info);
+  threshold.red=geometry_info.rho;
+  threshold.green=geometry_info.sigma;
   if ((flags & SigmaValue) == 0)
-    pixel.green=pixel.red;
-  pixel.blue=geometry_info.xi;
+    threshold.green=threshold.red;
+  threshold.blue=geometry_info.xi;
   if ((flags & XiValue) == 0)
-    pixel.blue=pixel.red;
-  pixel.opacity=geometry_info.psi;
+    threshold.blue=threshold.red;
+  threshold.opacity=geometry_info.psi;
   if ((flags & PsiValue) == 0)
-    pixel.opacity=(MagickRealType) OpaqueOpacity;
+    threshold.opacity=threshold.red;
+  threshold.index=geometry_info.chi;
+  if ((flags & ChiValue) == 0)
+    threshold.index=threshold.red;
   if ((flags & PercentValue) != 0)
     {
-      pixel.red*=(MagickRealType) (QuantumRange/100.0);
-      pixel.green*=(MagickRealType) (QuantumRange/100.0);
-      pixel.blue*=(MagickRealType) (QuantumRange/100.0);
-      pixel.opacity*=(MagickRealType) (QuantumRange/100.0);
+      threshold.red*=(QuantumRange/100.0);
+      threshold.green*=(QuantumRange/100.0);
+      threshold.blue*=(QuantumRange/100.0);
+      threshold.opacity*=(QuantumRange/100.0);
+      threshold.index*=(QuantumRange/100.0);
     }
   /*
     Black threshold image.
   */
   status=MagickTrue;
   progress=0;
-  exception=(&image->exception);
-  image_view=AcquireCacheViewThreadSet(image);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,8) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
+    register IndexPacket
+      *__restrict indexes;
+
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewAuthenticPixels(image_view[id],0,y,image->columns,1,
-      exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    if (IsMagickGray(&pixel) != MagickFalse)
-      for (x=0; x < (long) image->columns; x++)
-      {
-        if ((MagickRealType) PixelIntensityToQuantum(q) < pixel.red)
-          {
-            q->red=(Quantum) 0;
-            q->green=(Quantum) 0;
-            q->blue=(Quantum) 0;
-          }
-        q++;
-      }
-    else
-      for (x=0; x < (long) image->columns; x++)
-      {
-        if ((MagickRealType) q->red < pixel.red)
-          q->red=(Quantum) 0;
-        if ((MagickRealType) q->green < pixel.green)
-          q->green=(Quantum) 0;
-        if ((MagickRealType) q->blue < pixel.blue)
-          q->blue=(Quantum) 0;
-        if ((MagickRealType) q->opacity < pixel.opacity)
-          q->opacity=(Quantum) 0;
-        q++;
-      }
-    if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    for (x=0; x < (long) image->columns; x++)
+    {
+      if (((channel & RedChannel) != 0) &&
+          ((MagickRealType) q->red < threshold.red))
+        q->red=(Quantum) 0;
+      if (((channel & GreenChannel) != 0) &&
+          ((MagickRealType) q->green < threshold.green))
+        q->green=(Quantum) 0;
+      if (((channel & BlueChannel) != 0) &&
+          ((MagickRealType) q->blue < threshold.blue))
+        q->blue=(Quantum) 0;
+      if (((channel & OpacityChannel) != 0) &&
+          ((MagickRealType) q->opacity < threshold.opacity))
+        q->opacity=(Quantum) 0;
+      if (((channel & IndexChannel) != 0) &&
+          (image->colorspace == CMYKColorspace) &&
+          ((MagickRealType) indexes[x] < threshold.index))
+        indexes[x]=(Quantum) 0;
+      q++;
+    }
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -611,7 +622,7 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_BlackThresholdImageChannel)
 #endif
         proceed=SetImageProgress(image,ThresholdImageTag,progress++,
           image->rows);
@@ -619,7 +630,7 @@ MagickExport MagickBooleanType BlackThresholdImage(Image *image,
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
 
@@ -691,9 +702,21 @@ MagickExport ThresholdMap *DestroyThresholdMap(ThresholdMap *map)
 MagickExport ThresholdMap *GetThresholdMapFile(const char *xml,
   const char *filename,const char *map_id,ExceptionInfo *exception)
 {
-  const char *attr, *content;
-  XMLTreeInfo *thresholds,*threshold,*description,*levels;
-  ThresholdMap *map;
+  const char
+    *attr,
+    *content;
+
+  double
+    value;
+
+ ThresholdMap
+     *map;
+
+  XMLTreeInfo
+     *description,
+     *levels,
+     *threshold,
+     *thresholds;
 
   map = (ThresholdMap *)NULL;
   (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
@@ -833,14 +856,15 @@ MagickExport ThresholdMap *GetThresholdMapFile(const char *xml,
       }
       content = p;
     }
-    (void) strtol(content, &p, 10);
-    if ( p != content ) {
-       (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
-         "XmlInvalidContent", "<level> too many values, map \"%s\"", map_id);
-      thresholds = DestroyXMLTree(thresholds);
-      map = DestroyThresholdMap(map);
-      return(map);
-    }
+    value=(double) strtol(content,&p,10);
+    if (p != content)
+      {
+        (void) ThrowMagickException(exception,GetMagickModule(),OptionError,
+          "XmlInvalidContent", "<level> too many values, map \"%s\"", map_id);
+       thresholds=DestroyXMLTree(thresholds);
+       map=DestroyThresholdMap(map);
+       return(map);
+     }
   }
 
   thresholds = DestroyXMLTree(thresholds);
@@ -1046,7 +1070,7 @@ MagickExport MagickBooleanType ListThresholdMaps(FILE *file,
 %  images to monochrome using positional information to retain as much
 %  information as possible.
 %
-%  WARNING: This function is depreciated, and is now just a call to
+%  WARNING: This function is deprecated, and is now just a call to
 %  the more more powerful OrderedPosterizeImage(); function.
 %
 %  The format of the OrderedDitherImage method is:
@@ -1168,8 +1192,8 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
   ThresholdMap
     *map;
 
-  ViewInfo
-    **image_view;
+  CacheView
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -1180,7 +1204,11 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
   if (threshold_map == (const char *) NULL)
     return(MagickTrue);
   {
-    char token[MaxTextExtent], *p;
+    char
+      token[MaxTextExtent];
+
+    register const char
+      *p;
 
     p=(char *)threshold_map;
     while (((isspace((int) ((unsigned char) *p)) != 0) || (*p == ',')) &&
@@ -1209,7 +1237,7 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
   { /* parse directly as a comma seperated list of integers */
     char *p;
 
-    p = strchr(threshold_map,',');
+    p = strchr((char *) threshold_map,',');
     if ( p != (char *)NULL && isdigit((int) ((unsigned char) *(++p))) )
       levels.index = (unsigned long) strtol(p, &p, 10);
     else
@@ -1224,7 +1252,7 @@ MagickExport MagickBooleanType OrderedPosterizeImageChannel(Image *image,
 
     /* if more than a single number, each channel has a separate value */
     if ( p != (char *) NULL && *p == ',' ) {
-      p=strchr(threshold_map,',');
+      p=strchr((char *) threshold_map,',');
       p++;
       if ((channel & RedChannel) != 0)
         levels.red = (unsigned long) strtol(p, &p, 10),   (void)(*p == ',' && p++);
@@ -1277,33 +1305,30 @@ printf("DEBUG levels  r=%ld g=%ld b=%ld a=%ld i=%ld\n",
       }
     status=MagickTrue;
     progress=0;
-    image_view=AcquireCacheViewThreadSet(image);
+    image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-    #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
     for (y=0; y < (long) image->rows; y++)
     {
+      register IndexPacket
+        *__restrict indexes;
+
       register long
-        id,
         x;
 
-      register IndexPacket
-        *indexes;
-
       register PixelPacket
-        *q;
+        *__restrict q;
 
       if (status == MagickFalse)
         continue;
-      id=GetCacheViewThreadId();
-      q=GetCacheViewAuthenticPixels(image_view[id],0,y,image->columns,1,
-        exception);
+      q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         {
           status=MagickFalse;
           continue;
         }
-      indexes=GetCacheViewAuthenticIndexes(image_view[id]);
+      indexes=GetCacheViewAuthenticIndexQueue(image_view);
       for (x=0; x < (long) image->columns; x++)
       {
         register int
@@ -1357,7 +1382,7 @@ printf("DEBUG levels  r=%ld g=%ld b=%ld a=%ld i=%ld\n",
         }
         q++;
       }
-      if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+      if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
         status=MagickFalse;
       if (image->progress_monitor != (MagickProgressMonitor) NULL)
         {
@@ -1365,14 +1390,14 @@ printf("DEBUG levels  r=%ld g=%ld b=%ld a=%ld i=%ld\n",
             proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-          #pragma omp critical
+  #pragma omp critical (MagickCore_OrderedPosterizeImageChannel)
 #endif
           proceed=SetImageProgress(image,DitherImageTag,progress++,image->rows);
           if (proceed == MagickFalse)
             status=MagickFalse;
         }
     }
-    image_view=DestroyCacheViewThreadSet(image_view);
+    image_view=DestroyCacheView(image_view);
   }
   map=DestroyThresholdMap(map);
   return(MagickTrue);
@@ -1451,8 +1476,11 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
     min_threshold,
     max_threshold;
 
-  ViewInfo
-    **image_view;
+  RandomInfo
+    **random_info;
+
+  CacheView
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -1495,33 +1523,37 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
       if (AcquireImageColormap(image,2) == MagickFalse)
         ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
           image->filename);
-      image_view=AcquireCacheViewThreadSet(image);
+      random_info=AcquireRandomInfoThreadSet();
+      image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-      #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
       for (y=0; y < (long) image->rows; y++)
       {
+        MagickBooleanType
+          sync;
+
         register IndexPacket
-          *indexes;
+          *__restrict indexes;
 
         register long
           id,
           x;
 
         register PixelPacket
-          *q;
+          *__restrict q;
 
         if (status == MagickFalse)
           continue;
-        id=GetCacheViewThreadId();
-        q=GetCacheViewAuthenticPixels(image_view[id],0,y,image->columns,1,
+        q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,
           exception);
         if (q == (PixelPacket *) NULL)
           {
             status=MagickFalse;
             continue;
           }
-        indexes=GetCacheViewAuthenticIndexes(image_view[id]);
+        indexes=GetCacheViewAuthenticIndexQueue(image_view);
+        id=GetOpenMPThreadId();
         for (x=0; x < (long) image->columns; x++)
         {
           IndexPacket
@@ -1537,12 +1569,13 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
             threshold.index=max_threshold;
           else
             threshold.index=(MagickRealType)(QuantumRange*
-              GetPseudoRandomValue());
+              GetPseudoRandomValue(random_info[id]));
           index=(IndexPacket) (intensity <= threshold.index ? 0 : 1);
           indexes[x]=index;
           *q++=image->colormap[(long) index];
         }
-        if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+        sync=SyncCacheViewAuthenticPixels(image_view,exception);
+        if (sync == MagickFalse)
           status=MagickFalse;
         if (image->progress_monitor != (MagickProgressMonitor) NULL)
           {
@@ -1550,7 +1583,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-            #pragma omp critical
+  #pragma omp critical (MagickCore_RandomThresholdImageChannel)
 #endif
             proceed=SetImageProgress(image,ThresholdImageTag,progress++,
               image->rows);
@@ -1558,7 +1591,8 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               status=MagickFalse;
           }
       }
-      image_view=DestroyCacheViewThreadSet(image_view);
+      image_view=DestroyCacheView(image_view);
+      random_info=DestroyRandomInfoThreadSet(random_info);
       return(status);
     }
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
@@ -1566,33 +1600,33 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
       InheritException(exception,&image->exception);
       return(MagickFalse);
     }
-  image_view=AcquireCacheViewThreadSet(image);
+  random_info=AcquireRandomInfoThreadSet();
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
     register IndexPacket
-      *indexes;
+      *__restrict indexes;
 
     register long
       id,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewAuthenticPixels(image_view[id],0,y,image->columns,1,
-      exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    id=GetOpenMPThreadId();
     for (x=0; x < (long) image->columns; x++)
     {
       if ((channel & RedChannel) != 0)
@@ -1604,7 +1638,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               threshold.red=max_threshold;
             else
               threshold.red=(MagickRealType) (QuantumRange*
-                GetPseudoRandomValue());
+                GetPseudoRandomValue(random_info[id]));
         }
       if ((channel & GreenChannel) != 0)
         {
@@ -1615,7 +1649,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               threshold.green=max_threshold;
             else
               threshold.green=(MagickRealType) (QuantumRange*
-                GetPseudoRandomValue());
+                GetPseudoRandomValue(random_info[id]));
         }
       if ((channel & BlueChannel) != 0)
         {
@@ -1626,7 +1660,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               threshold.blue=max_threshold;
             else
               threshold.blue=(MagickRealType) (QuantumRange*
-                GetPseudoRandomValue());
+                GetPseudoRandomValue(random_info[id]));
         }
       if ((channel & OpacityChannel) != 0)
         {
@@ -1637,7 +1671,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               threshold.opacity=max_threshold;
             else
               threshold.opacity=(MagickRealType) (QuantumRange*
-                GetPseudoRandomValue());
+                GetPseudoRandomValue(random_info[id]));
         }
       if (((channel & IndexChannel) != 0) &&
           (image->colorspace == CMYKColorspace))
@@ -1649,7 +1683,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
               threshold.index=max_threshold;
             else
               threshold.index=(MagickRealType) (QuantumRange*
-                GetPseudoRandomValue());
+                GetPseudoRandomValue(random_info[id]));
         }
       if ((channel & RedChannel) != 0)
         q->red=(Quantum) ((MagickRealType) q->red <= threshold.red ? 0 :
@@ -1669,7 +1703,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
           threshold.index ? 0 : QuantumRange);
       q++;
     }
-    if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -1677,7 +1711,7 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_RandomThresholdImageChannel)
 #endif
         proceed=SetImageProgress(image,ThresholdImageTag,progress++,
           image->rows);
@@ -1685,7 +1719,8 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
+  random_info=DestroyRandomInfoThreadSet(random_info);
   return(status);
 }
 
@@ -1707,21 +1742,36 @@ MagickExport MagickBooleanType RandomThresholdImageChannel(Image *image,
 %  The format of the WhiteThresholdImage method is:
 %
 %      MagickBooleanType WhiteThresholdImage(Image *image,const char *threshold)
+%      MagickBooleanType WhiteThresholdImageChannel(Image *image,
+%        const ChannelType channel,const char *threshold,
+%        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
 %    o image: the image.
 %
-%    o threshold: Define the threshold value
+%    o channel: the channel or channels to be thresholded.
+%
+%    o threshold: Define the threshold value.
+%
+%    o exception: return any errors or warnings in this structure.
 %
 */
 MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
   const char *threshold)
 {
-#define ThresholdImageTag  "Threshold/Image"
+  MagickBooleanType
+    status;
 
-  ExceptionInfo
-    *exception;
+  status=WhiteThresholdImageChannel(image,DefaultChannels,threshold,
+    &image->exception);
+  return(status);
+}
+
+MagickExport MagickBooleanType WhiteThresholdImageChannel(Image *image,
+  const ChannelType channel,const char *thresholds,ExceptionInfo *exception)
+{
+#define ThresholdImageTag  "Threshold/Image"
 
   GeometryInfo
     geometry_info;
@@ -1734,95 +1784,95 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
     status;
 
   MagickPixelPacket
-    pixel;
+    threshold;
 
   MagickStatusType
     flags;
 
-  ViewInfo
-    **image_view;
+  CacheView
+    *image_view;
 
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  if (threshold == (const char *) NULL)
+  if (thresholds == (const char *) NULL)
     return(MagickTrue);
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
-  GetMagickPixelPacket(image,&pixel);
-  flags=ParseGeometry(threshold,&geometry_info);
-  pixel.red=geometry_info.rho;
-  pixel.green=geometry_info.sigma;
+  flags=ParseGeometry(thresholds,&geometry_info);
+  GetMagickPixelPacket(image,&threshold);
+  threshold.red=geometry_info.rho;
+  threshold.green=geometry_info.sigma;
   if ((flags & SigmaValue) == 0)
-    pixel.green=pixel.red;
-  pixel.blue=geometry_info.xi;
+    threshold.green=threshold.red;
+  threshold.blue=geometry_info.xi;
   if ((flags & XiValue) == 0)
-    pixel.blue=pixel.red;
-  pixel.opacity=geometry_info.psi;
+    threshold.blue=threshold.red;
+  threshold.opacity=geometry_info.psi;
   if ((flags & PsiValue) == 0)
-    pixel.opacity=(MagickRealType) OpaqueOpacity;
+    threshold.opacity=threshold.red;
+  threshold.index=geometry_info.chi;
+  if ((flags & ChiValue) == 0)
+    threshold.index=threshold.red;
   if ((flags & PercentValue) != 0)
     {
-      pixel.red*=(MagickRealType) (QuantumRange/100.0);
-      pixel.green*=(MagickRealType) (QuantumRange/100.0);
-      pixel.blue*=(MagickRealType) (QuantumRange/100.0);
-      pixel.opacity*=(MagickRealType) (QuantumRange/100.0);
+      threshold.red*=(QuantumRange/100.0);
+      threshold.green*=(QuantumRange/100.0);
+      threshold.blue*=(QuantumRange/100.0);
+      threshold.opacity*=(QuantumRange/100.0);
+      threshold.index*=(QuantumRange/100.0);
     }
   /*
     White threshold image.
   */
   status=MagickTrue;
   progress=0;
-  exception=(&image->exception);
-  image_view=AcquireCacheViewThreadSet(image);
+  image_view=AcquireCacheView(image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,8) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
+    register IndexPacket
+      *__restrict indexes;
+
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    q=GetCacheViewAuthenticPixels(image_view[id],0,y,image->columns,1,
-      exception);
+    q=GetCacheViewAuthenticPixels(image_view,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       {
         status=MagickFalse;
         continue;
       }
-    if (IsMagickGray(&pixel) != MagickFalse)
-      for (x=0; x < (long) image->columns; x++)
-      {
-        if ((MagickRealType) PixelIntensityToQuantum(q) > pixel.red)
-          {
-            q->red=(Quantum) QuantumRange;
-            q->green=(Quantum) QuantumRange;
-            q->blue=(Quantum) QuantumRange;
-          }
-        q++;
-      }
-    else
-      for (x=0; x < (long) image->columns; x++)
-      {
-        if ((MagickRealType) q->red > pixel.red)
-          q->red=(Quantum) QuantumRange;
-        if ((MagickRealType) q->green > pixel.green)
-          q->green=(Quantum) QuantumRange;
-        if ((MagickRealType) q->blue > pixel.blue)
-          q->blue=(Quantum) QuantumRange;
-        if ((MagickRealType) q->opacity > pixel.opacity)
-          q->opacity=(Quantum) QuantumRange;
-        q++;
-      }
-    if (SyncCacheViewAuthenticPixels(image_view[id],exception) == MagickFalse)
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    for (x=0; x < (long) image->columns; x++)
+    {
+      if (((channel & RedChannel) != 0) &&
+          ((MagickRealType) q->red > threshold.red))
+        q->red=(Quantum) QuantumRange;
+      if (((channel & GreenChannel) != 0) &&
+          ((MagickRealType) q->green > threshold.green))
+        q->green=(Quantum) QuantumRange;
+      if (((channel & BlueChannel) != 0) &&
+          ((MagickRealType) q->blue > threshold.blue))
+        q->blue=(Quantum) QuantumRange;
+      if (((channel & OpacityChannel) != 0) &&
+          ((MagickRealType) q->opacity > threshold.opacity))
+        q->opacity=(Quantum) QuantumRange;
+      if (((channel & IndexChannel) != 0) &&
+          (image->colorspace == CMYKColorspace) &&
+          ((MagickRealType) indexes[x] > threshold.index))
+        indexes[x]=(Quantum) QuantumRange;
+      q++;
+    }
+    if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -1830,7 +1880,7 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_WhiteThresholdImageChannel)
 #endif
         proceed=SetImageProgress(image,ThresholdImageTag,progress++,
           image->rows);
@@ -1838,6 +1888,6 @@ MagickExport MagickBooleanType WhiteThresholdImage(Image *image,
           status=MagickFalse;
       }
   }
-  image_view=DestroyCacheViewThreadSet(image_view);
+  image_view=DestroyCacheView(image_view);
   return(status);
 }
