@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -111,7 +111,7 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   register long
     i;
 
-  ViewInfo
+  CacheView
     *chop_view,
     *image_view;
 
@@ -159,25 +159,25 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   for (y=0; y < (long) extent.y; y++)
   {
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register IndexPacket
-      *chop_indexes,
-      *indexes;
+      *__restrict chop_indexes,
+      *__restrict indexes;
 
     register long
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     p=GetCacheViewVirtualPixels(image_view,0,i++,image->columns,1,exception);
     q=QueueCacheViewAuthenticPixels(chop_view,0,j++,chop_image->columns,1,
       exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
-    indexes=GetCacheViewAuthenticIndexes(image_view);
-    chop_indexes=GetCacheViewAuthenticIndexes(chop_view);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    chop_indexes=GetCacheViewAuthenticIndexQueue(chop_view);
     for (x=0; x < (long) image->columns; x++)
     {
       if ((x < extent.x) || (x >= (long) (extent.x+extent.width)))
@@ -205,25 +205,25 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   for (y=0; y < (long) (image->rows-(extent.y+extent.height)); y++)
   {
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register IndexPacket
-      *chop_indexes,
-      *indexes;
+      *__restrict chop_indexes,
+      *__restrict indexes;
 
     register long
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     p=GetCacheViewVirtualPixels(image_view,0,i++,image->columns,1,exception);
     q=QueueCacheViewAuthenticPixels(chop_view,0,j++,chop_image->columns,1,
       exception);
     if ((p == (PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
-    indexes=GetCacheViewAuthenticIndexes(image_view);
-    chop_indexes=GetCacheViewAuthenticIndexes(chop_view);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    chop_indexes=GetCacheViewAuthenticIndexQueue(chop_view);
     for (x=0; x < (long) image->columns; x++)
     {
       if ((x < extent.x) || (x >= (long) (extent.x+extent.width)))
@@ -306,17 +306,17 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
       break;
     if (SetImageStorageClass(cmyk_image,DirectClass) == MagickFalse)
       break;
-    cmyk_image->colorspace=CMYKColorspace;
+    (void) SetImageColorspace(cmyk_image,CMYKColorspace);
     for (y=0; y < (long) images->rows; y++)
     {
       register const PixelPacket
-        *p;
+        *__restrict p;
 
       register long
         x;
 
       register PixelPacket
-        *q;
+        *__restrict q;
 
       p=GetVirtualPixels(images,0,y,images->columns,1,exception);
       q=QueueAuthenticPixels(cmyk_image,0,y,cmyk_image->columns,1,exception);
@@ -337,13 +337,13 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     for (y=0; y < (long) images->rows; y++)
     {
       register const PixelPacket
-        *p;
+        *__restrict p;
 
       register long
         x;
 
       register PixelPacket
-        *q;
+        *__restrict q;
 
       p=GetVirtualPixels(images,0,y,images->columns,1,exception);
       q=GetAuthenticPixels(cmyk_image,0,y,cmyk_image->columns,1,exception);
@@ -364,13 +364,13 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     for (y=0; y < (long) images->rows; y++)
     {
       register const PixelPacket
-        *p;
+        *__restrict p;
 
       register long
         x;
 
       register PixelPacket
-        *q;
+        *__restrict q;
 
       p=GetVirtualPixels(images,0,y,images->columns,1,exception);
       q=GetAuthenticPixels(cmyk_image,0,y,cmyk_image->columns,1,exception);
@@ -391,16 +391,16 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
     for (y=0; y < (long) images->rows; y++)
     {
       register const PixelPacket
-        *p;
+        *__restrict p;
 
       register IndexPacket
-        *indexes;
+        *__restrict indexes;
 
       register long
         x;
 
       register PixelPacket
-        *q;
+        *__restrict q;
 
       p=GetVirtualPixels(images,0,y,images->columns,1,exception);
       q=GetAuthenticPixels(cmyk_image,0,y,cmyk_image->columns,1,exception);
@@ -471,9 +471,9 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
     bounding_box,
     page;
 
-  ViewInfo
-    **crop_view,
-    **image_view;
+  CacheView
+    *crop_view,
+    *image_view;
 
   /*
     Check crop geometry.
@@ -514,8 +514,8 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
       crop_image->page=bounding_box;
       crop_image->page.x=(-1);
       crop_image->page.y=(-1);
-      if ( crop_image->dispose == BackgroundDispose )
-        crop_image->dispose = NoneDispose;
+      if (crop_image->dispose == BackgroundDispose)
+        crop_image->dispose=NoneDispose;
       return(crop_image);
     }
   if ((page.x < 0) && (bounding_box.x >= 0))
@@ -554,6 +554,12 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
       page.height=geometry->height;
   bounding_box.x+=page.x;
   bounding_box.y+=page.y;
+  if ((page.width == 0) || (page.height == 0))
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
+        "GeometryDoesNotContainImage","`%s'",image->filename);
+      return((Image *) NULL);
+    }
   /*
     Initialize crop image attributes.
   */
@@ -575,34 +581,30 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  crop_view=AcquireCacheViewThreadSet(crop_image);
+  image_view=AcquireCacheView(image);
+  crop_view=AcquireCacheView(crop_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) crop_image->rows; y++)
   {
     register const IndexPacket
-      *indexes;
+      *__restrict indexes;
 
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register IndexPacket
-      *crop_indexes;
-
-    register long
-      id;
+      *__restrict crop_indexes;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],page.x,page.y+y,
-      crop_image->columns,1,exception);
-    q=QueueCacheViewAuthenticPixels(crop_view[id],0,y,crop_image->columns,1,
+    p=GetCacheViewVirtualPixels(image_view,page.x,page.y+y,crop_image->columns,
+      1,exception);
+    q=QueueCacheViewAuthenticPixels(crop_view,0,y,crop_image->columns,1,
       exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
@@ -610,15 +612,15 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
         continue;
       }
     (void) CopyMagickMemory(q,p,(size_t) crop_image->columns*sizeof(*q));
-    indexes=GetCacheViewVirtualIndexes(image_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
     if (indexes != (IndexPacket *) NULL)
       {
-        crop_indexes=GetCacheViewAuthenticIndexes(crop_view[id]);
+        crop_indexes=GetCacheViewAuthenticIndexQueue(crop_view);
         if (crop_indexes != (IndexPacket *) NULL)
           (void) CopyMagickMemory(crop_indexes,indexes,(size_t)
             crop_image->columns*sizeof(*crop_indexes));
       }
-    if (SyncCacheViewAuthenticPixels(crop_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(crop_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -626,15 +628,15 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_CropImage)
 #endif
         proceed=SetImageProgress(image,CropImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  crop_view=DestroyCacheViewThreadSet(crop_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  crop_view=DestroyCacheView(crop_view);
+  image_view=DestroyCacheView(image_view);
   crop_image->type=image->type;
   if (status == MagickFalse)
     crop_image=DestroyImage(crop_image);
@@ -684,9 +686,9 @@ MagickExport Image *ExcerptImage(const Image *image,
   MagickBooleanType
     status;
 
-  ViewInfo
-    **excerpt_view,
-    **image_view;
+  CacheView
+    *excerpt_view,
+    *image_view;
 
   /*
     Allocate excerpt image.
@@ -707,48 +709,44 @@ MagickExport Image *ExcerptImage(const Image *image,
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  excerpt_view=AcquireCacheViewThreadSet(excerpt_image);
+  image_view=AcquireCacheView(image);
+  excerpt_view=AcquireCacheView(excerpt_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) excerpt_image->rows; y++)
   {
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register IndexPacket
-      *excerpt_indexes,
-      *indexes;
-
-    register long
-      id;
+      *__restrict excerpt_indexes,
+      *__restrict indexes;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],geometry->x,geometry->y+y,
+    p=GetCacheViewVirtualPixels(image_view,geometry->x,geometry->y+y,
       geometry->width,1,exception);
-    q=GetCacheViewAuthenticPixels(excerpt_view[id],0,y,excerpt_image->columns,
-      1,exception);
+    q=GetCacheViewAuthenticPixels(excerpt_view,0,y,excerpt_image->columns,1,
+      exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
     (void) CopyMagickMemory(q,p,(size_t) excerpt_image->columns*sizeof(*q));
-    indexes=GetCacheViewAuthenticIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     if (indexes != (IndexPacket *) NULL)
       {
-        excerpt_indexes=GetCacheViewAuthenticIndexes(excerpt_view[id]);
+        excerpt_indexes=GetCacheViewAuthenticIndexQueue(excerpt_view);
         if (excerpt_indexes != (IndexPacket *) NULL)
           (void) CopyMagickMemory(excerpt_indexes,indexes,(size_t)
             excerpt_image->columns*sizeof(*excerpt_indexes));
       }
-    if (SyncCacheViewAuthenticPixels(excerpt_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(excerpt_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -756,15 +754,15 @@ MagickExport Image *ExcerptImage(const Image *image,
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_ExcerptImage)
 #endif
         proceed=SetImageProgress(image,ExcerptImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  excerpt_view=DestroyCacheViewThreadSet(excerpt_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  excerpt_view=DestroyCacheView(excerpt_view);
+  image_view=DestroyCacheView(image_view);
   excerpt_image->type=image->type;
   if (status == MagickFalse)
     excerpt_image=DestroyImage(excerpt_image);
@@ -874,9 +872,9 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  ViewInfo
-    **flip_view,
-    **image_view;
+  CacheView
+    *flip_view,
+    *image_view;
 
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -892,47 +890,45 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  flip_view=AcquireCacheViewThreadSet(flip_image);
+  image_view=AcquireCacheView(image);
+  flip_view=AcquireCacheView(flip_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) flip_image->rows; y++)
   {
+    register const IndexPacket
+      *__restrict indexes;
+
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register IndexPacket
-      *flip_indexes,
-      *indexes;
-
-    register long
-      id;
+      *__restrict flip_indexes;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],0,y,image->columns,1,exception);
-    q=GetCacheViewAuthenticPixels(flip_view[id],0,(long) (flip_image->rows-y-1),
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=QueueCacheViewAuthenticPixels(flip_view,0,(long) (flip_image->rows-y-1),
       flip_image->columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    (void) CopyMagickMemory(q,p,(size_t) flip_image->columns*sizeof(*q));
-    indexes=GetCacheViewAuthenticIndexes(image_view[id]);
-    if (indexes != (IndexPacket *) NULL)
+    (void) CopyMagickMemory(q,p,(size_t) image->columns*sizeof(*q));
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    if (indexes != (const IndexPacket *) NULL)
       {
-        flip_indexes=GetCacheViewAuthenticIndexes(flip_view[id]);
+        flip_indexes=GetCacheViewAuthenticIndexQueue(flip_view);
         if (flip_indexes != (IndexPacket *) NULL)
-          (void) CopyMagickMemory(flip_indexes,indexes,(size_t)
-            flip_image->columns*sizeof(*flip_indexes));
+          (void) CopyMagickMemory(flip_indexes,indexes,(size_t) image->columns*
+            sizeof(*flip_indexes));
       }
-    if (SyncCacheViewAuthenticPixels(flip_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(flip_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -940,15 +936,15 @@ MagickExport Image *FlipImage(const Image *image,ExceptionInfo *exception)
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_FlipImage)
 #endif
         proceed=SetImageProgress(image,FlipImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  flip_view=DestroyCacheViewThreadSet(flip_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  flip_view=DestroyCacheView(flip_view);
+  image_view=DestroyCacheView(image_view);
   flip_image->type=image->type;
   if (status == MagickFalse)
     flip_image=DestroyImage(flip_image);
@@ -994,9 +990,9 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  ViewInfo
-    **flop_view,
-    **image_view;
+  CacheView
+    *flop_view,
+    *image_view;
 
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -1012,32 +1008,32 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  flop_view=AcquireCacheViewThreadSet(flop_image);
+  image_view=AcquireCacheView(image);
+  flop_view=AcquireCacheView(flop_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) flop_image->rows; y++)
   {
-    register IndexPacket
-      *flop_indexes,
-      *indexes;
+    register const IndexPacket
+      *__restrict indexes;
 
     register const PixelPacket
-      *p;
+      *__restrict p;
+
+    register IndexPacket
+      *__restrict flop_indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],0,y,image->columns,1,exception);
-    q=QueueCacheViewAuthenticPixels(flop_view[id],0,y,flop_image->columns,1,
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=QueueCacheViewAuthenticPixels(flop_view,0,y,flop_image->columns,1,
       exception);
     if ((p == (PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
@@ -1045,20 +1041,16 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
         continue;
       }
     q+=flop_image->columns;
-    indexes=GetCacheViewAuthenticIndexes(image_view[id]);
-    flop_indexes=GetCacheViewAuthenticIndexes(flop_view[id]);
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    flop_indexes=GetCacheViewAuthenticIndexQueue(flop_view);
     for (x=0; x < (long) flop_image->columns; x++)
     {
-      q--;
-      *q=(*p);
-      if (indexes != (IndexPacket *) NULL)
-        {
-          if (flop_indexes != (IndexPacket *) NULL)
-            flop_indexes[flop_image->columns-x-1]=indexes[x];
-        }
-      p++;
+      (*--q)=(*p++);
+      if ((indexes != (const IndexPacket *) NULL) &&
+          (flop_indexes != (IndexPacket *) NULL))
+        flop_indexes[flop_image->columns-x-1]=indexes[x];
     }
-    if (SyncCacheViewAuthenticPixels(flop_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(flop_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -1066,15 +1058,15 @@ MagickExport Image *FlopImage(const Image *image,ExceptionInfo *exception)
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_FlopImage)
 #endif
         proceed=SetImageProgress(image,FlopImageTag,progress++,image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  flop_view=DestroyCacheViewThreadSet(flop_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  flop_view=DestroyCacheView(flop_view);
+  image_view=DestroyCacheView(image_view);
   flop_image->type=image->type;
   if (status == MagickFalse)
     flop_image=DestroyImage(flop_image);
@@ -1122,15 +1114,15 @@ static inline MagickBooleanType CopyImageRegion(Image *destination,
   MagickBooleanType
     status;
 
-  ViewInfo
-    **source_view,
-    **destination_view;
+  CacheView
+    *source_view,
+    *destination_view;
 
   status=MagickTrue;
-  source_view=AcquireCacheViewThreadSet(source);
-  destination_view=AcquireCacheViewThreadSet(destination);
+  source_view=AcquireCacheView(source);
+  destination_view=AcquireCacheView(destination);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(status)
+  #pragma omp parallel for schedule(dynamic,4) shared(status)
 #endif
   for (y=0; y < (long) rows; y++)
   {
@@ -1138,48 +1130,45 @@ static inline MagickBooleanType CopyImageRegion(Image *destination,
       sync;
 
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register IndexPacket
-      *indexes,
-      *destination_indexes;
+      *__restrict indexes,
+      *__restrict destination_indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     /*
       Transfer scanline.
     */
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(source_view[id],sx,sy+y,columns,1,exception);
-    q=GetCacheViewAuthenticPixels(destination_view[id],dx,dy+y,columns,1,
-      exception);
+    p=GetCacheViewVirtualPixels(source_view,sx,sy+y,columns,1,exception);
+    q=GetCacheViewAuthenticPixels(destination_view,dx,dy+y,columns,1,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexes(source_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(source_view);
     for (x=0; x < (long) columns; x++)
       *q++=(*p++);
     if (indexes != (IndexPacket *) NULL)
       {
-        destination_indexes=GetCacheViewAuthenticIndexes(destination_view[id]);
+        destination_indexes=GetCacheViewAuthenticIndexQueue(destination_view);
         for (x=0; x < (long) columns; x++)
           destination_indexes[x]=indexes[x];
       }
-    sync=SyncCacheViewAuthenticPixels(destination_view[id],exception);
+    sync=SyncCacheViewAuthenticPixels(destination_view,exception);
     if (sync == MagickFalse)
       status=MagickFalse;
   }
-  destination_view=DestroyCacheViewThreadSet(destination_view);
-  source_view=DestroyCacheViewThreadSet(source_view);
+  destination_view=DestroyCacheView(destination_view);
+  source_view=DestroyCacheView(source_view);
   return(status);
 }
 
@@ -1191,7 +1180,7 @@ MagickExport Image *RollImage(const Image *image,const long x_offset,
   Image
     *roll_image;
 
-  MagickBooleanType
+  MagickStatusType
     status;
 
   RectangleInfo
@@ -1358,7 +1347,7 @@ MagickExport Image *SpliceImage(const Image *image,
   register long
     i;
 
-  ViewInfo
+  CacheView
     *image_view,
     *splice_view;
 
@@ -1383,8 +1372,7 @@ MagickExport Image *SpliceImage(const Image *image,
       splice_image=DestroyImage(splice_image);
       return((Image *) NULL);
     }
-  if (image->background_color.opacity != OpaqueOpacity)
-    splice_image->matte=MagickTrue;
+  (void) SetImageBackgroundColor(splice_image);
   /*
     Respect image geometry.
   */
@@ -1448,24 +1436,27 @@ MagickExport Image *SpliceImage(const Image *image,
   progress=0;
   image_view=AcquireCacheView(image);
   splice_view=AcquireCacheView(splice_image);
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
   for (y=0; y < (long) splice_geometry.y; y++)
   {
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register IndexPacket
-      *indexes,
-      *splice_indexes;
+      *__restrict indexes,
+      *__restrict splice_indexes;
 
     register long
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    p=GetCacheViewVirtualPixels(image_view,0,i++,image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
     q=QueueCacheViewAuthenticPixels(splice_view,0,y,splice_image->columns,1,
       exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
@@ -1473,25 +1464,35 @@ MagickExport Image *SpliceImage(const Image *image,
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexes(image_view);
-    splice_indexes=GetCacheViewAuthenticIndexes(splice_view);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    splice_indexes=GetCacheViewAuthenticIndexQueue(splice_view);
     for (x=0; x < splice_geometry.x; x++)
     {
-      *q++=(*p++);
-      if (splice_image->colorspace == CMYKColorspace)
+      q->red=p->red;
+      q->green=p->green;
+      q->blue=p->blue;
+      q->opacity=OpaqueOpacity;
+      if (image->matte != MagickFalse)
+        q->opacity=p->opacity;
+      if (image->colorspace == CMYKColorspace)
         splice_indexes[x]=(*indexes++);
+      p++;
+      q++;
     }
     for ( ; x < (long) (splice_geometry.x+splice_geometry.width); x++)
-    {
-      *q++=image->background_color;
-      if (splice_image->colorspace == CMYKColorspace)
-        splice_indexes[x]=(IndexPacket) 0;
-    }
+      q++;
     for ( ; x < (long) splice_image->columns; x++)
     {
-      *q++=(*p++);
-      if (splice_image->colorspace == CMYKColorspace)
+      q->red=p->red;
+      q->green=p->green;
+      q->blue=p->blue;
+      q->opacity=OpaqueOpacity;
+      if (image->matte != MagickFalse)
+        q->opacity=p->opacity;
+      if (image->colorspace == CMYKColorspace)
         splice_indexes[x]=(*indexes++);
+      p++;
+      q++;
     }
     if (SyncCacheViewAuthenticPixels(splice_view,exception) == MagickFalse)
       status=MagickFalse;
@@ -1501,70 +1502,38 @@ MagickExport Image *SpliceImage(const Image *image,
         MagickBooleanType
           proceed;
 
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical (MagickCore_TransposeImage)
+#endif
         proceed=SetImageProgress(image,SpliceImageTag,progress++,
           splice_image->rows);
         if (proceed == MagickFalse)
           status=MagickFalse;
       }
   }
-  for ( ; y < (long) (splice_geometry.y+splice_geometry.height); y++)
-  {
-    register IndexPacket
-      *splice_indexes;
-
-    register long
-      x;
-
-    register PixelPacket
-      *q;
-
-    if (status == MagickFalse)
-      continue;
-    q=QueueCacheViewAuthenticPixels(splice_view,0,y,splice_image->columns,1,
-      exception);
-    if (q == (PixelPacket *) NULL)
-      {
-        status=MagickFalse;
-        continue;
-      }
-    splice_indexes=GetCacheViewAuthenticIndexes(splice_view);
-    for (x=0; x < (long) splice_image->columns; x++)
-    {
-      *q++=image->background_color;
-      if (splice_image->colorspace == CMYKColorspace)
-        splice_indexes[x]=(IndexPacket) 0;
-    }
-    if (SyncCacheViewAuthenticPixels(splice_view,exception) == MagickFalse)
-      status=MagickFalse;
-    if (image->progress_monitor != (MagickProgressMonitor) NULL)
-      {
-        MagickBooleanType
-          proceed;
-
-        proceed=SetImageProgress(image,SpliceImageTag,progress++,
-          splice_image->rows);
-        if (proceed == MagickFalse)
-          status=MagickFalse;
-      }
-  }
-  for ( ; y < (long) splice_image->rows; y++)
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
+#endif
+  for (y=(long) (splice_geometry.y+splice_geometry.height);
+       y < (long) splice_image->rows; y++)
   {
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register IndexPacket
-      *indexes,
-      *splice_indexes;
+      *__restrict indexes,
+      *__restrict splice_indexes;
 
     register long
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    p=GetCacheViewVirtualPixels(image_view,0,i++,image->columns,1,exception);
+    p=GetCacheViewVirtualPixels(image_view,0,y-splice_geometry.height,
+      image->columns,1,exception);
     q=QueueCacheViewAuthenticPixels(splice_view,0,y,splice_image->columns,1,
       exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
@@ -1572,25 +1541,35 @@ MagickExport Image *SpliceImage(const Image *image,
         status=MagickFalse;
         continue;
       }
-    indexes=GetCacheViewAuthenticIndexes(image_view);
-    splice_indexes=GetCacheViewAuthenticIndexes(splice_view);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
+    splice_indexes=GetCacheViewAuthenticIndexQueue(splice_view);
     for (x=0; x < splice_geometry.x; x++)
     {
-      *q++=(*p++);
-      if (splice_image->colorspace == CMYKColorspace)
+      q->red=p->red;
+      q->green=p->green;
+      q->blue=p->blue;
+      q->opacity=OpaqueOpacity;
+      if (image->matte != MagickFalse)
+        q->opacity=p->opacity;
+      if (image->colorspace == CMYKColorspace)
         splice_indexes[x]=(*indexes++);
+      p++;
+      q++;
     }
     for ( ; x < (long) (splice_geometry.x+splice_geometry.width); x++)
-    {
-      *q++=image->background_color;
-      if (splice_image->colorspace == CMYKColorspace)
-        splice_indexes[x]=(IndexPacket) 0;
-    }
+      q++;
     for ( ; x < (long) splice_image->columns; x++)
     {
-      *q++=(*p++);
-      if (splice_image->colorspace == CMYKColorspace)
+      q->red=p->red;
+      q->green=p->green;
+      q->blue=p->blue;
+      q->opacity=OpaqueOpacity;
+      if (image->matte != MagickFalse)
+        q->opacity=p->opacity;
+      if (image->colorspace == CMYKColorspace)
         splice_indexes[x]=(*indexes++);
+      p++;
+      q++;
     }
     if (SyncCacheViewAuthenticPixels(splice_view,exception) == MagickFalse)
       status=MagickFalse;
@@ -1599,6 +1578,9 @@ MagickExport Image *SpliceImage(const Image *image,
         MagickBooleanType
           proceed;
 
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  #pragma omp critical (MagickCore_TransposeImage)
+#endif
         proceed=SetImageProgress(image,SpliceImageTag,progress++,
           splice_image->rows);
         if (proceed == MagickFalse)
@@ -1660,7 +1642,7 @@ MagickExport MagickBooleanType TransformImage(Image **image,
   assert(image != (Image **) NULL);
   assert((*image)->signature == MagickSignature);
   if ((*image)->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*image)->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",(*image)->filename);
   transform_image=(*image);
   if (crop_geometry != (const char *) NULL)
     {
@@ -1674,7 +1656,8 @@ MagickExport MagickBooleanType TransformImage(Image **image,
         Crop image to a user specified size.
       */
       crop_image=NewImageList();
-      flags=ParseGravityGeometry(transform_image,crop_geometry,&geometry);
+      flags=ParseGravityGeometry(transform_image,crop_geometry,&geometry,
+        &(*image)->exception);
       if (((geometry.width == 0) && (geometry.height == 0)) ||
           ((flags & XValue) != 0) || ((flags & YValue) != 0))
         {
@@ -1735,7 +1718,9 @@ MagickExport MagickBooleanType TransformImage(Image **image,
                 break;
             }
           }
-      if (crop_image != (Image *) NULL)
+      if (crop_image == (Image *) NULL)
+        transform_image=CloneImage(*image,0,0,MagickTrue,&(*image)->exception);
+      else
         {
           transform_image=DestroyImage(transform_image);
           transform_image=GetFirstImageInList(crop_image);
@@ -1747,7 +1732,8 @@ MagickExport MagickBooleanType TransformImage(Image **image,
   /*
     Scale image to a user specified size.
   */
-  flags=ParseSizeGeometry(transform_image,image_geometry,&geometry);
+  flags=ParseRegionGeometry(transform_image,image_geometry,&geometry,
+    &(*image)->exception);
   if ((transform_image->columns == geometry.width) &&
       (transform_image->rows == geometry.height))
     return(MagickTrue);
@@ -1806,7 +1792,8 @@ MagickExport MagickBooleanType TransformImages(Image **images,
   assert(images != (Image **) NULL);
   assert((*images)->signature == MagickSignature);
   if ((*images)->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),(*images)->filename);
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
+      (*images)->filename);
   image_list=ImageListToArray(*images,&(*images)->exception);
   if (image_list == (Image **) NULL)
     return(MagickFalse);
@@ -1865,9 +1852,9 @@ MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
   RectangleInfo
     page;
 
-  ViewInfo
-    **image_view,
-    **transpose_view;
+  CacheView
+    *image_view,
+    *transpose_view;
 
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -1884,48 +1871,44 @@ MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  transpose_view=AcquireCacheViewThreadSet(transpose_image);
+  image_view=AcquireCacheView(image);
+  transpose_view=AcquireCacheView(transpose_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register IndexPacket
-      *transpose_indexes,
-      *indexes;
-
-    register long
-      id;
+      *__restrict transpose_indexes,
+      *__restrict indexes;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],0,(long) image->rows-y-1,
+    p=GetCacheViewVirtualPixels(image_view,0,(long) image->rows-y-1,
       image->columns,1,exception);
-    q=QueueCacheViewAuthenticPixels(transpose_view[id],(long) (image->rows-y-1),
-      0,1,transpose_image->rows,exception);
+    q=QueueCacheViewAuthenticPixels(transpose_view,(long) (image->rows-y-1),0,
+      1,transpose_image->rows,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
         status=MagickFalse;
         continue;
       }
     (void) CopyMagickMemory(q,p,(size_t) image->columns*sizeof(*q));
-    indexes=GetCacheViewAuthenticIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     if (indexes != (IndexPacket *) NULL)
       {
-        transpose_indexes=GetCacheViewAuthenticIndexes(transpose_view[id]);
+        transpose_indexes=GetCacheViewAuthenticIndexQueue(transpose_view);
         if (transpose_indexes != (IndexPacket *) NULL)
           (void) CopyMagickMemory(transpose_indexes,indexes,(size_t)
             image->columns*sizeof(*transpose_indexes));
       }
-    if (SyncCacheViewAuthenticPixels(transpose_view[id],exception) == MagickFalse)
+    if (SyncCacheViewAuthenticPixels(transpose_view,exception) == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
       {
@@ -1933,7 +1916,7 @@ MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_TransposeImage)
 #endif
         proceed=SetImageProgress(image,TransposeImageTag,progress++,
           image->rows);
@@ -1941,8 +1924,8 @@ MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
           status=MagickFalse;
       }
   }
-  transpose_view=DestroyCacheViewThreadSet(transpose_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  transpose_view=DestroyCacheView(transpose_view);
+  image_view=DestroyCacheView(image_view);
   transpose_image->type=image->type;
   page=transpose_image->page;
   Swap(page.width,page.height);
@@ -1997,9 +1980,9 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
   RectangleInfo
     page;
 
-  ViewInfo
-    **image_view,
-    **transverse_view;
+  CacheView
+    *image_view,
+    *transverse_view;
 
   assert(image != (const Image *) NULL);
   assert(image->signature == MagickSignature);
@@ -2016,10 +1999,10 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
   */
   status=MagickTrue;
   progress=0;
-  image_view=AcquireCacheViewThreadSet(image);
-  transverse_view=AcquireCacheViewThreadSet(transverse_image);
+  image_view=AcquireCacheView(image);
+  transverse_view=AcquireCacheView(transverse_image);
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-  #pragma omp parallel for schedule(static,16) shared(progress,status)
+  #pragma omp parallel for schedule(dynamic,4) shared(progress,status)
 #endif
   for (y=0; y < (long) image->rows; y++)
   {
@@ -2027,24 +2010,22 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
       sync;
 
     register const PixelPacket
-      *p;
+      *__restrict p;
 
     register IndexPacket
-      *transverse_indexes,
-      *indexes;
+      *__restrict transverse_indexes,
+      *__restrict indexes;
 
     register long
-      id,
       x;
 
     register PixelPacket
-      *q;
+      *__restrict q;
 
     if (status == MagickFalse)
       continue;
-    id=GetCacheViewThreadId();
-    p=GetCacheViewVirtualPixels(image_view[id],0,y,image->columns,1,exception);
-    q=QueueCacheViewAuthenticPixels(transverse_view[id],(long) (image->rows-y-
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    q=QueueCacheViewAuthenticPixels(transverse_view,(long) (image->rows-y-
       1),0,1,transverse_image->rows,exception);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       {
@@ -2054,15 +2035,15 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
     q+=image->columns;
     for (x=0; x < (long) image->columns; x++)
       *--q=(*p++);
-    indexes=GetCacheViewAuthenticIndexes(image_view[id]);
+    indexes=GetCacheViewAuthenticIndexQueue(image_view);
     if (indexes != (IndexPacket *) NULL)
       {
-        transverse_indexes=GetCacheViewAuthenticIndexes(transverse_view[id]);
+        transverse_indexes=GetCacheViewAuthenticIndexQueue(transverse_view);
         if (transverse_indexes != (IndexPacket *) NULL)
           for (x=0; x < (long) image->columns; x++)
             transverse_indexes[image->columns-x-1]=indexes[x];
       }
-    sync=SyncCacheViewAuthenticPixels(transverse_view[id],exception);
+    sync=SyncCacheViewAuthenticPixels(transverse_view,exception);
     if (sync == MagickFalse)
       status=MagickFalse;
     if (image->progress_monitor != (MagickProgressMonitor) NULL)
@@ -2071,7 +2052,7 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
           proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-        #pragma omp critical
+  #pragma omp critical (MagickCore_TransverseImage)
 #endif
         proceed=SetImageProgress(image,TransverseImageTag,progress++,
           image->rows);
@@ -2079,8 +2060,8 @@ MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
           status=MagickFalse;
       }
   }
-  transverse_view=DestroyCacheViewThreadSet(transverse_view);
-  image_view=DestroyCacheViewThreadSet(image_view);
+  transverse_view=DestroyCacheView(transverse_view);
+  image_view=DestroyCacheView(image_view);
   transverse_image->type=image->type;
   page=transverse_image->page;
   Swap(page.width,page.height);
