@@ -27,11 +27,14 @@ extern "C" {
 */
 #define MagickPackageName "ImageMagick"
 #define MagickCopyright  "Copyright (C) 1999-2009 ImageMagick Studio LLC"
-#define MagickLibVersion  0x648
-#define MagickLibVersionText  "6.4.8"
-#define MagickLibVersionNumber  1,0,0
-#define MagickReleaseDate  "2008-12-30"
-#define MagickChangeDate   "20081228"
+#define MagickLibVersion  0x654
+#define MagickLibVersionText  "6.5.4"
+#define MagickLibVersionNumber  2,0,0
+#define MagickLibSubversion  "-5"
+#define MagickReleaseDate  "2009-07-23"
+#define MagickChangeDate   "20090723"
+#define MagickAuthoritativeURL  "http://www.imagemagick.org"
+#define MagickHomeURL  "file:///usr/share/doc/ImageMagick-6.5.4/index.html"
 #if (MAGICKCORE_QUANTUM_DEPTH == 8)
 #define MagickQuantumDepth  "Q8"
 #define MagickQuantumRange  "255"
@@ -59,10 +62,9 @@ extern "C" {
 #define MagickOPENMPSupport "OpenMP "
 #endif
 #define MagickSupport MagickHDRISupport MagickOPENMPSupport
-#define MagickVersion MagickPackageName " " MagickLibVersionText " " \
-  MagickReleaseDate " " MagickQuantumDepth " " MagickSupport \
-  "http://www.imagemagick.org"
-#define MagickHomeURL  "file:///usr/share/doc/ImageMagick-6.4.8/index.html"
+#define MagickVersion MagickPackageName " " MagickLibVersionText \
+  MagickLibSubversion " " MagickReleaseDate " " MagickQuantumDepth " " \
+  MagickSupport MagickAuthoritativeURL
 
 extern MagickExport char
   *GetMagickHomeURL(void);
