@@ -17,7 +17,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2007 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,6 +46,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <math.h>
 #include <time.h>
 #include "wand/MagickWand.h"
 #if defined(__WINDOWS__)
@@ -70,8 +71,12 @@ int main(int argc,char **argv)
   char
     *option;
 
+  double
+    elapsed_time,
+    user_time;
+
   ExceptionInfo
-    exception;
+    *exception;
 
   ImageInfo
     *image_info;
@@ -83,28 +88,52 @@ int main(int argc,char **argv)
   register long
     i;
 
+  TimerInfo
+    timer;
+
+  unsigned long
+    iterations;
+
   MagickCoreGenesis(*argv,MagickTrue);
-  GetExceptionInfo(&exception);
+  exception=AcquireExceptionInfo();
+  iterations=1;
+  status=MagickFalse;
   regard_warnings=MagickFalse;
-  for (i=1; i < (long) argc; i++)
+  for (i=1; i < (long) (argc-1); i++)
   {
     option=argv[i];
     if ((strlen(option) == 1) || ((*option != '-') && (*option != '+')))
       continue;
+    if (LocaleCompare("bench",option+1) == 0)
+      iterations=(unsigned long) atol(argv[++i]);
+    if (LocaleCompare("debug",option+1) == 0)
+      (void) SetLogEventMask(argv[++i]);
     if (LocaleCompare("regard-warnings",option+1) == 0)
       regard_warnings=MagickTrue;
   }
-  image_info=AcquireImageInfo();
-  status=MontageImageCommand(image_info,argc,argv,(char **) NULL,&exception);
-  if ((status == MagickFalse) || (exception.severity != UndefinedException))
+  GetTimerInfo(&timer);
+  for (i=0; i < (long) iterations; i++)
+  {
+    image_info=AcquireImageInfo();
+    status=MontageImageCommand(image_info,argc,argv,(char **) NULL,exception);
+    if (exception->severity != UndefinedException)
+      {
+        if ((exception->severity > ErrorException) ||
+            (regard_warnings != MagickFalse))
+          status=MagickTrue;
+        CatchException(exception);
+      }
+    image_info=DestroyImageInfo(image_info);
+  }
+  if (iterations > 1)
     {
-      if ((exception.severity < ErrorException) &&
-          (regard_warnings == MagickFalse))
-        status=MagickTrue;
-      CatchException(&exception);
+      elapsed_time=GetElapsedTime(&timer);
+      user_time=GetUserTime(&timer);
+      (void) fprintf(stderr,"Performance: %lui %gips %0.3fu %ld:%02ld\n",
+        iterations,1.0*iterations/elapsed_time,user_time,(long)
+        (elapsed_time/60.0+0.5),(long) ceil(fmod(elapsed_time,60.0)));
     }
-  image_info=DestroyImageInfo(image_info);
-  (void) DestroyExceptionInfo(&exception);
+  exception=DestroyExceptionInfo(exception);
   MagickCoreTerminus();
-  return(status == MagickFalse ? 1 : 0);
+  return(status == MagickFalse ? 0 : 1);
 }
