@@ -17,7 +17,7 @@
 %                                July 1992                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2009 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -33,6 +33,8 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
+%  Use the display program to display an image or image sequence on any X
+%  server.
 %
 */
 
@@ -88,7 +90,7 @@
 %
 */
 
-static void DisplayUsage(void)
+static MagickBooleanType DisplayUsage(void)
 {
   const char
     **p;
@@ -124,6 +126,7 @@ static void DisplayUsage(void)
       "-despeckle           reduce the speckles within an image",
       "-edge factor         apply a filter to detect edges in the image",
       "-enhance             apply a digital filter to enhance a noisy image",
+      "-equalize            perform histogram equalization to an image",
       "-extract geometry    extract area from image",
       "-flip                flip image in the vertical direction",
       "-flop                flop image in the horizontal direction",
@@ -132,6 +135,7 @@ static void DisplayUsage(void)
       "-gamma value         level of gamma correction",
       "-monochrome          transform image to black and white",
       "-negate              replace every pixel with its complementary color",
+      "-normalize           transform image to span the full range of colors",
       "-raise value         lighten/darken image edges to create a 3-D effect",
       "-resample geometry   change the resolution of an image",
       "-resize geometry     resize the image",
@@ -147,7 +151,8 @@ static void DisplayUsage(void)
     },
     *settings[]=
     {
-      "-alpha option        activate, deactivate, reset, or set the alpha channel",
+      "-alpha option        on, activate, off, deactivate, set, opaque, copy",
+      "                     transparent, extract, background, or shape",
       "-antialias           remove pixel-aliasing",
       "-authenticate password",
       "                     decipher image with this password",
@@ -167,8 +172,9 @@ static void DisplayUsage(void)
       "-dither method       apply error diffusion to image",
       "-endian type         endianness (MSB or LSB) of the image",
       "-filter type         use this filter when resizing an image",
-      "-format \"string\"     output formatted image characteristics",
+      "-format string     output formatted image characteristics",
       "-geometry geometry   preferred size and location of the Image window",
+      "-gravity type        horizontal and vertical backdrop placement",
       "-identify            identify the format and characteristics of the image",
       "-immutable           displayed image cannot be modified",
       "-interlace type      type of image interlacing scheme",
@@ -186,7 +192,7 @@ static void DisplayUsage(void)
       "-regard-warnings     pay attention to warning messages",
       "-remote command      execute a command in an remote display process",
       "-repage geometry     size and location of an image canvas (operator)",
-      "-respect-parenthesis settings remain in effect until parenthesis boundary",
+      "-respect-parentheses settings remain in effect until parenthesis boundary",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
       "-seed value          seed a new sequence of pseudo-random numbers",
@@ -248,7 +254,7 @@ static void DisplayUsage(void)
   (void) printf("\nButtons: \n");
   for (p=buttons; *p != (char *) NULL; p++)
     (void) printf("  %s\n",*p);
-  exit(0);
+  return(MagickFalse);
 }
 
 WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
@@ -282,7 +288,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
 #define ThrowDisplayInvalidArgumentException(option,argument) \
 { \
   (void) ThrowMagickException(exception,GetMagickModule(),OptionError, \
-    "InvalidArgument","`%s': %s",argument,option); \
+    "InvalidArgument","`%s': %s",option,argument); \
   DestroyDisplay(); \
   return(MagickFalse); \
 }
@@ -374,12 +380,12 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
     ThrowDisplayException(ResourceLimitError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   image_marker=(unsigned long *) AcquireQuantumMemory((size_t) argc+1UL,
     sizeof(*image_marker));
   if (image_marker == (unsigned long *) NULL)
     ThrowDisplayException(ResourceLimitError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   for (i=0; i <= argc; i++)
     image_marker[i]=(unsigned long) argc;
   /*
@@ -405,7 +411,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
       }
     if ((LocaleCompare("help",option+1) == 0) ||
         (LocaleCompare("-help",option+1) == 0))
-      DisplayUsage();
+      return(DisplayUsage());
   }
   /*
     Get user defaults from X resource database.
@@ -510,7 +516,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
         if (images == (Image *) NULL)
           continue;
         AppendImageStack(images);
-        FireImageStack(MagickFalse,MagickFalse,MagickTrue);
+        FinalizeImageSettings(image_info,image,MagickFalse);
         do
         {
           /*
@@ -537,10 +543,14 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
               */
               nexus=XDisplayImage(display,&resource_info,argv,argc,&image,
                 &state);
+              status&=nexus != (Image *) NULL;
               if (nexus == (Image *) NULL)
                 break;
               while ((nexus != (Image *) NULL) && ((state & ExitState) == 0))
               {
+                Image
+                  *next;
+
                 if (nexus->montage != (char *) NULL)
                   {
                     /*
@@ -550,9 +560,9 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
                     image=nexus;
                     break;
                   }
-                image=XDisplayImage(display,&resource_info,argv,argc,&nexus,
+                next=XDisplayImage(display,&resource_info,argv,argc,&nexus,
                   &state);
-                if ((image == (Image *) NULL) &&
+                if ((next == (Image *) NULL) &&
                     (GetNextImageInList(nexus) != (Image *) NULL))
                   {
                     RemoveAllImageStack()
@@ -563,7 +573,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
                   {
                     if (nexus != image)
                       nexus=DestroyImageList(nexus);
-                    nexus=image;
+                    nexus=next;
                   }
               }
             } while ((state & ExitState) == 0);
@@ -574,8 +584,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
               */
               (void) CopyMagickString(image->filename,
                 resource_info.write_filename,MaxTextExtent);
-              (void) SetImageInfo(image_info,MagickTrue,
-                &image->exception);
+              (void) SetImageInfo(image_info,MagickTrue,&image->exception);
               status&=WriteImage(image_info,image);
               GetImageException(image,exception);
             }
@@ -638,9 +647,10 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
           }
         if (resource_info.window_id != (char *) NULL)
           state|=ExitState;
-        if (iterations != 0)
-          if (iteration == (long) iterations)
-            state|=ExitState;
+        if ((iterations != 0) && (iteration == (long) iterations))
+          state|=ExitState;
+        if (LocaleCompare(filename,"-") == 0)
+          state|=ExitState;
         iteration++;
         continue;
       }
@@ -1031,6 +1041,8 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("enhance",option+1) == 0)
           break;
+        if (LocaleCompare("equalize",option+1) == 0)
+          break;
         if (LocaleCompare("extract",option+1) == 0)
           {
             if (*option == '+')
@@ -1147,6 +1159,23 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             resource_info.image_geometry=ConstantString(argv[i]);
             break;
           }
+        if (LocaleCompare("gravity",option+1) == 0)
+          {
+            long
+              gravity;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) (argc-1))
+              ThrowDisplayException(OptionError,"MissingArgument",option);
+            gravity=ParseMagickOption(MagickGravityOptions,MagickFalse,
+              argv[i]);
+            if (gravity < 0)
+              ThrowDisplayException(OptionError,"UnrecognizedGravityType",
+                argv[i]);
+            break;
+          }
         ThrowDisplayException(OptionError,"UnrecognizedOption",option);
       }
       case 'h':
@@ -1235,6 +1264,9 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             char
               *p;
 
+            double
+              value;
+
             long
               resource;
 
@@ -1251,11 +1283,29 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             i++;
             if (i == (long) argc)
               ThrowDisplayException(OptionError,"MissingArgument",option);
-            (void) strtod(argv[i],&p);
+            value=strtod(argv[i],&p);
             if ((p == argv[i]) && (LocaleCompare("unlimited",argv[i]) != 0))
               ThrowDisplayInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("list",option+1) == 0)
+          {
+            long
+              list;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (long) argc)
+              ThrowDisplayException(OptionError,"MissingArgument",option);
+            list=ParseMagickOption(MagickListOptions,MagickFalse,argv[i]);
+            if (list < 0)
+              ThrowDisplayException(OptionError,"UnrecognizedListType",argv[i]);
+            (void) MogrifyImageInfo(image_info,(int) (i-j+1),(const char **)
+              argv+j,exception);
+            DestroyDisplay();
+            return(MagickTrue);
+          }
         if (LocaleCompare("log",option+1) == 0)
           {
             if (*option == '+')
@@ -1300,7 +1350,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             resource_info.map_type=(char *) NULL;
             if (*option == '+')
               break;
-            (void) CopyMagickString(argv[i]+1,"sans",MaxTextExtent);
+            (void) strcpy(argv[i]+1,"san");
             i++;
             if (i == (long) argc)
               ThrowDisplayException(OptionError,"MissingArgument",option);
@@ -1348,6 +1398,8 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
           break;
         if (LocaleCompare("noop",option+1) == 0)
           break;
+        if (LocaleCompare("normalize",option+1) == 0)
+          break;
         ThrowDisplayException(OptionError,"UnrecognizedOption",option);
       }
       case 'p':
@@ -1462,7 +1514,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
               ThrowDisplayInvalidArgumentException(option,argv[i]);
             break;
           }
-        if (LocaleCompare("respect-parenthesis",option+1) == 0)
+        if (LocaleNCompare("respect-parentheses",option+1,17) == 0)
           {
             respect_parenthesis=(*option == '-') ? MagickTrue : MagickFalse;
             break;
@@ -1513,11 +1565,6 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
               ThrowDisplayInvalidArgumentException(option,argv[i]);
             break;
           }
-        if (LocaleCompare("sans",option+1) == 0)
-          {
-            i++;
-            break;
-          }
         if (LocaleCompare("scenes",option+1) == 0)
           {
             if (*option == '+')
@@ -1784,11 +1831,12 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
             if (IsPathAccessible(resource_info.write_filename) != MagickFalse)
               {
                 char
-                  answer[2];
+                  answer[2],
+                  *p;
 
                 (void) fprintf(stderr,"Overwrite %s? ",
                   resource_info.write_filename);
-                (void) fgets(answer,(int) sizeof(answer),stdin);
+                p=fgets(answer,(int) sizeof(answer),stdin);
                 if (((*answer != 'y') && (*answer != 'Y')))
                   return(MagickFalse);
               }
@@ -1801,7 +1849,7 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
       default:
         ThrowDisplayException(OptionError,"UnrecognizedOption",option);
     }
-    fire=ParseMagickOption(MagickImageListOptions,MagickFalse,option+1) < 0 ? 
+    fire=ParseMagickOption(MagickImageListOptions,MagickFalse,option+1) < 0 ?
       MagickFalse : MagickTrue;
     if (fire != MagickFalse)
       FireImageStack(MagickFalse,MagickTrue,MagickTrue);
@@ -1816,9 +1864,10 @@ WandExport MagickBooleanType DisplayImageCommand(ImageInfo *image_info,
   DestroyDisplay();
   return(status != 0 ? MagickTrue : MagickFalse);
 #else
+  (void) argc;
+  (void) argv;
   (void) ThrowMagickException(exception,GetMagickModule(),MissingDelegateError,
     "DelegateLibrarySupportNotBuiltIn","`%s' (X11)",image_info->filename);
-  DisplayUsage();
-  return(MagickFalse);
+  return(DisplayUsage());
 #endif
 }
