@@ -33,9 +33,14 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Identify describes the format and characteristics of one or more image
-%  files.  It will also report if an image is incomplete or corrupt.
-%
+%  The identify program describes the format and characteristics of one or more
+%  image files. It also reports if an image is incomplete or corrupt. The
+%  information returned includes the image number, the file name, the width and
+%  height of the image, whether the image is colormapped or not, the number of
+%  colors in the image, the number of bytes in the image, the format of the
+%  image (JPEG, PNM, etc.), and finally the number of seconds it took to read
+%  and process the image. Many more attributes are available with the verbose
+%  option.
 %
 */
 
@@ -101,7 +106,8 @@ static MagickBooleanType IdentifyUsage(void)
     },
     *settings[]=
     {
-      "-alpha option        activate, deactivate, reset, or set the alpha channel",
+      "-alpha option        on, activate, off, deactivate, set, opaque, copy",
+      "                     transparent, extract, background, or shape",
       "-antialias           remove pixel-aliasing",
       "-authenticate password",
       "                     decipher image with this password",
@@ -123,7 +129,7 @@ static MagickBooleanType IdentifyUsage(void)
       "-ping                efficiently determine image attributes",
       "-quiet               suppress all warning messages",
       "-regard-warnings     pay attention to warning messages",
-      "-respect-parenthesis settings remain in effect until parenthesis boundary",
+      "-respect-parentheses settings remain in effect until parenthesis boundary",
       "-sampling-factor geometry",
       "                     horizontal and vertical sampling factor",
       "-seed value          seed a new sequence of pseudo-random numbers",
@@ -156,7 +162,7 @@ static MagickBooleanType IdentifyUsage(void)
   (void) printf(
     "image type as the filename suffix (i.e. image.ps).  Specify 'file' as\n");
   (void) printf("'-' for standard input or output.\n");
-  return(MagickTrue);
+  return(MagickFalse);
 }
 
 WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
@@ -228,11 +234,14 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
           (void) fprintf(stdout,"Version: %s\n",
             GetMagickVersion((unsigned long *) NULL));
           (void) fprintf(stdout,"Copyright: %s\n\n",GetMagickCopyright());
-          return(MagickTrue);
+          return(MagickFalse);
         }
     }
   if (argc < 2)
-    return(IdentifyUsage());
+    {
+      (void) IdentifyUsage();
+      return(MagickTrue);
+    }
   count=0;
   format=NULL;
   j=1;
@@ -248,7 +257,7 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
     ThrowIdentifyException(ResourceLimitError,"MemoryAllocationFailed",
-      strerror(errno));
+      GetExceptionMessage(errno));
   image_info->ping=MagickTrue;
   for (i=1; i < (long) argc; i++)
   {
@@ -301,7 +310,7 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
         if (images == (Image *) NULL)
           continue;
         AppendImageStack(images);
-        FireImageStack(MagickFalse,MagickFalse,MagickTrue);
+        FinalizeImageSettings(image_info,image,MagickFalse);
         for ( ; image != (Image *) NULL; image=GetNextImageInList(image))
         {
           if (image->scene == 0)
@@ -319,7 +328,7 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
               text=InterpretImageProperties(image_info,image,format);
               if (text == (char *) NULL)
                 ThrowIdentifyException(ResourceLimitError,
-                  "MemoryAllocationFailed",strerror(errno));
+                  "MemoryAllocationFailed",GetExceptionMessage(errno));
               (void) ConcatenateString(&(*metadata),text);
               text=DestroyString(text);
               if (LocaleCompare(format,"%n") == 0)
@@ -653,7 +662,7 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
       {
         if (LocaleCompare("regard-warnings",option+1) == 0)
           break;
-        if (LocaleCompare("respect-parenthesis",option+1) == 0)
+        if (LocaleNCompare("respect-parentheses",option+1,17) == 0)
           {
             respect_parenthesis=(*option == '-') ? MagickTrue : MagickFalse;
             break;
